@@ -294,13 +294,14 @@ __pmSockAddr *
 __pmStringToSockAddr(const char *cp)
 {
     __pmSockAddr *addr = __pmSockAddrAlloc();
+
     if (addr) {
-        if (cp == NULL || strcmp(cp, "INADDR_ANY") == 0) {
+	if (cp == NULL || strcmp(cp, "INADDR_ANY") == 0) {
 	    addr->sockaddr.inet.sin_addr.s_addr = INADDR_ANY;
 	    /* Set the address family to 0, meaning "not set". */
 	    addr->sockaddr.raw.sa_family = 0;
 	}
-        else if (strcmp(cp, "INADDR_LOOPBACK") == 0) {
+	else if (strcmp(cp, "INADDR_LOOPBACK") == 0) {
 	    addr->sockaddr.inet.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
 	    /* Set the address family to 0, meaning "not set". */
 	    addr->sockaddr.raw.sa_family = 0;
@@ -352,10 +353,10 @@ __pmStringToSockAddr(const char *cp)
 	    }
 	    else {
 		addr->sockaddr.raw.sa_family = AF_INET;
-	        sts = inet_pton(AF_INET, cp, &addr->sockaddr.inet.sin_addr);
+		sts = inet_pton(AF_INET, cp, &addr->sockaddr.inet.sin_addr);
 	    }
 	    if (sts <= 0) {
-	        __pmSockAddrFree(addr);
+		__pmSockAddrFree(addr);
 		addr = NULL;
 	    }
 	}
@@ -659,18 +660,55 @@ int
 __pmConnect(int fd, void *addr, __pmSockLen addrlen)
 {
     __pmSockAddr *sock = (__pmSockAddr *)addr;
+    int		sts;
     if (sock->sockaddr.raw.sa_family == AF_INET)
-        return connect(fd, &sock->sockaddr.raw, sizeof(sock->sockaddr.inet));
-    if (sock->sockaddr.raw.sa_family == AF_INET6)
-        return connect(fd, &sock->sockaddr.raw, sizeof(sock->sockaddr.ipv6));
+        sts = connect(fd, &sock->sockaddr.raw, sizeof(sock->sockaddr.inet));
+    else if (sock->sockaddr.raw.sa_family == AF_INET6)
+        sts = connect(fd, &sock->sockaddr.raw, sizeof(sock->sockaddr.ipv6));
 #if defined(HAVE_STRUCT_SOCKADDR_UN)
-    if (sock->sockaddr.raw.sa_family == AF_UNIX)
-        return connect(fd, &sock->sockaddr.raw, sizeof(sock->sockaddr.local));
+    else if (sock->sockaddr.raw.sa_family == AF_UNIX)
+        sts = connect(fd, &sock->sockaddr.raw, sizeof(sock->sockaddr.local));
 #endif
-    pmNotifyErr(LOG_ERR,
+    else {
+	pmNotifyErr(LOG_ERR,
 		"%s:__pmConnect: Invalid address family: %d\n", __FILE__, sock->sockaddr.raw.sa_family);
-    errno = EAFNOSUPPORT;
-    return -1; /* failure */
+	errno = EAFNOSUPPORT;
+	return -1; /* failure */
+    }
+
+    if (sts >= 0)
+	return sts;
+
+    /* something bad happened ... */
+    sts = neterror();
+    if (sts == EINPROGRESS) {
+	/*
+	 * not sure if connect() is done, wait on select() ... 5 secs seems
+	 * long enough to wait given that for EINPROGRESS it seems safe
+	 * to assume select() will return "real soon now"
+	 */
+	__pmFdSet	fdset;
+	struct timeval	wait = { 5,0 };
+	int		lsts;
+
+	__pmFD_ZERO(&fdset);
+	__pmFD_SET(fd, &fdset);
+	if ((lsts = __pmSelectWrite(fd+1, &fdset, &wait)) == 1) {
+	    sts = __pmConnectCheckError(fd);
+	}
+	else if (lsts == 0)
+	    sts = ETIMEDOUT;
+	else 
+	    sts = neterror();
+    }
+
+    if (sts == 0)
+	/* OK now ... */
+	return 0;
+
+    /* persistent badness ... */
+    errno = sts;
+    return -1;
 }
 
 int
@@ -897,7 +935,7 @@ __pmAuxConnectPMCDPort(const char *hostname, int pmcd_port)
     __pmHostEnt		*servInfo;
     int			fd;
     int			sts;
-    int			fdFlags[FD_SETSIZE];
+    int			fdFlags[PM_FDSET_SIZE];
     __pmFdSet		allFds;
     __pmFdSet		readyFds;
     int			maxFd;
@@ -909,11 +947,11 @@ __pmAuxConnectPMCDPort(const char *hostname, int pmcd_port)
 
     if ((servInfo = __pmGetAddrInfo(hostname)) == NULL) {
 	if (pmDebugOptions.context) {
-	    const char	*errmsg;
 	    PM_LOCK(__pmLock_extcall);
-	    errmsg = hoststrerror();		/* THREADSAFE */
-	    fprintf(stderr, "%s:__pmAuxConnectPMCDPort(%s, %d) : hosterror=%d, ``%s''\n",
-		    __FILE__, hostname, pmcd_port, hosterror(), errmsg);
+	    fprintf(stderr, "%s:__pmAuxConnectPMCDPort(%s, %d) : "
+			    "hosterror=%d, ``%s''\n",
+		    __FILE__, hostname, pmcd_port, hosterror(),
+		    hoststrerror());		/* THREADSAFE */
 	    PM_UNLOCK(__pmLock_extcall);
 	}
 	return -EHOSTUNREACH;
@@ -964,7 +1002,7 @@ __pmAuxConnectPMCDPort(const char *hostname, int pmcd_port)
 			  __FILE__, hostname, pmcd_port, __pmSockAddrGetFamily(myAddr));
 	    fd = -EINVAL;
 	}
-	if (fd < 0) {
+	if (fd < 0 || fd >= PM_FDSET_SIZE) {
 	    if (pmDebugOptions.context && pmDebugOptions.desperate) {
 		char *sockname = __pmSockAddrToString(myAddr);
 		fprintf(stderr, "__pmAuxConnectPMCDPort: failed to create socket %s, errno=%d\n", sockname, errno);
