@@ -14,11 +14,15 @@
 
 #include "pmapi.h"
 #include "libpcp.h"
+#include <sys/stat.h>
 #define SOCKET_INTERNAL
 #include "internal.h"
 #if defined(HAVE_GETPEERUCRED)
 #include <ucred.h>
 #endif
+#ifdef HAVE_SYSTEMD_SD_DAEMON_H
+#include <systemd/sd-daemon.h>
+#endif
 
 #define STRINGIFY(s)	#s
 #define TO_STRING(s)	STRINGIFY(s)
@@ -1146,3 +1150,167 @@ __pmServerStart(int argc, char **argv, int flags)
     /* don't chdir("/") -- we may still need to call pmOpenLog() */
 }
 #endif
+
+#ifdef HAVE_SYSTEMD
+/*
+ * Portable sd_notify(). Some platforms use systemd but do not
+ * readily have the devel headers/libs needed to call sd_notify,
+ * so on those platforms send to a UNIX domain socket directly.
+ * Returns negative on failure or >= 0 on success.
+ */
+static int
+__pm_sd_notify(int clear, const char *notify_socket, const char *msg)
+{
+    int sts = 0;
+    int fd = -1;
+
+    if (notify_socket == NULL) {
+    	/* no-op - warning issued by caller */
+	return 0;
+    }
+
+#ifdef HAVE_SYSTEMD_SD_DAEMON_H
+    /* use the devel library call and return it's status */
+    sts = sd_notify(clear, msg);
+    if (pmDebugOptions.services)
+	fprintf(stderr, "__pm_sd_notify: using libsystemd:sd_notify(). NOTIFY_SOCKET=\"%s\". msg=\"%s\" sts=%d\n", 
+		notify_socket, msg, sts);
+    goto out;
+#else
+
+#if defined(HAVE_STRUCT_SOCKADDR_UN)
+    /* send the message via a UNIX domain socket */
+    {
+	struct sockaddr_un su = { 0 };
+	struct iovec iov = { 0 };
+	struct msghdr hdr = { 0 };
+	int sendto_flags = 0;
+
+	if (pmDebugOptions.services)
+	    fprintf(stderr, "__pm_sd_notify: using AF_UNIX socket for notification\n");
+
+	if ((strchr("@/", notify_socket[0])) == NULL || strlen(notify_socket) < 2) {
+	    pmNotifyErr(LOG_ERR, "__pm_sd_notify: invalid notify_socket=\"%s\"\n", notify_socket);
+	    sts = PM_ERR_GENERIC;
+	    goto out;
+	}
+
+	if ((fd = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1) {
+	    pmNotifyErr(LOG_ERR, "__pm_sd_notify: failed to create AF_UNIX socket: %s\n", strerror(errno));
+	    sts = PM_ERR_GENERIC;
+	    goto out;
+	}
+
+	su.sun_family = AF_UNIX;
+	strncpy(su.sun_path, notify_socket, sizeof(su.sun_path)-1);
+	su.sun_path[sizeof(su.sun_path)-1] = '\0';
+
+	if (notify_socket[0] == '@')
+	    su.sun_path[0] = '\0';
+
+	iov.iov_base = (char *)msg;
+	iov.iov_len = strlen(msg);
+
+	hdr.msg_name = &su;
+	hdr.msg_namelen = offsetof(struct sockaddr_un, sun_path) + strlen(notify_socket);
+	hdr.msg_iov = &iov;
+	hdr.msg_iovlen = 1;
+
+	if (clear)
+	    unsetenv("NOTIFY_SOCKET");
+	sendto_flags |= MSG_NOSIGNAL; /* no sigpipes etc */
+	if (sendmsg(fd, &hdr, sendto_flags) < 0) {
+	    /* FAILED to send */
+	    pmNotifyErr(LOG_ERR, "Error __pm_sd_notify: sendmsg \"%s\" failed: %s\n", msg, strerror(errno));
+	    sts = PM_ERR_GENERIC;
+	    goto out;
+	}
+    }
+#else
+    pmNotifyErr(LOG_ERR, "Warning __pm_sd_notify: failed, no sd_notify() or AF_UNIX sockets on this platform\n");
+    sts = PM_ERR_NYI;
+#endif /* HAVE_STRUCT_SOCKADDR_UN */
+#endif /* HAVE_SYSTEMD_SD_DAEMON_H */
+
+out:
+    if (fd >= 0)
+    	close(fd);
+    return sts;
+}
+#endif /* HAVE_SYSTEMD */
+
+
+#ifdef HAVE_SYSTEMD
+/*
+ * Notify the service manager of status. On platforms using systemd
+ * AND the service unit is using Type=notify, then ${NOTIFY_SOCKET} will
+ * be set in the environment and this call will notify the service manager
+ * via that socket that the service is either READY or STOPPING, and provide
+ * the pid of the main daemon process so that systemd can track it.
+ * See sd_notify(3) for more details.
+ *
+ * On systemd platforms that are NOT using Type=notify, and on non-systemd
+ * platforms, these calls do nothing. i.e. On platforms using systemd that
+ * are NOT using Type=notify (e.g. if the service unit is Type=forking)
+ * then ${NOTIFY_SOCKET} will NOT be set in the environment and calls to
+ * __pmServerNotifySystemd() will do nothing (it's a cheap no-op that avoids
+ * messy conditional code in the caller).  The same is true on non-systemd
+ * platforms - these functions can safely be called unconditionally and
+ * they will simply return PM_ERR_NYI, which can (and should be) ignored on
+ * those non-systemd platforms.
+ */
+static int
+__pmServerNotifySystemd(const char *msg)
+{
+    const char *notify_socket = getenv("NOTIFY_SOCKET"); /* may be NULL */
+    int sts = 0;
+
+    if (notify_socket) {
+	sts = __pm_sd_notify(0, notify_socket, msg);
+	if (pmDebugOptions.services)
+	    fprintf(stderr, "__pmServerNotifySystemd: NOTIFY_SOCKET=\"%s\" msg=\"%s\" sts=%d\n",
+		    notify_socket, msg, sts);
+    } else {
+	/*
+	 * we were not launched by systemd ... this is sort of expected,
+	 * e.g. pmlogger launched by record mode, stand alone logging or
+	 * (especially) the QA suite
+	 */
+    	if (pmDebugOptions.services)
+	    fprintf(stderr, "__pmServerNotifySystemd: NOTIFY_SOCKET not set:"
+	    	    " not launched by systemd, or service unit is not using Type=notify\n");
+	sts = PM_ERR_GENERIC;
+    }
+    /* negative return indicates error */
+    return sts;
+}
+#endif /* HAVE_SYSTEMD */
+
+int
+__pmServerNotifyServiceManagerReady(pid_t pid)
+{
+#ifdef HAVE_SYSTEMD
+    char msg[64];
+
+    snprintf(msg, sizeof(msg), "READY=1\nMAINPID=%" FMT_PID, pid);
+    return __pmServerNotifySystemd((const char *)msg);
+#else
+    /* no systemd - not fatal but log a debug message */
+    if (pmDebugOptions.services)
+	fprintf(stderr, "__pmServerNotifyServiceManagerReady: no service manager on this platform\n");
+    return PM_ERR_NYI;
+#endif
+}
+
+int
+__pmServerNotifyServiceManagerStopping(pid_t pid)
+{
+#ifdef HAVE_SYSTEMD
+    return __pmServerNotifySystemd("STOPPING=1");
+#else
+    /* no systemd - not fatal but log a debug message */
+    if (pmDebugOptions.services)
+	fprintf(stderr, "__pmServerNotifyServiceManagerStopping: no service manager on this platform\n");
+    return PM_ERR_NYI;
+#endif
+}
