@@ -42,7 +42,9 @@ __pmIsConnectLock(void *lock)
 void
 init_connect_lock(void)
 {
+#if defined(PM_MULTI_THREAD)
     __pmInitMutex(&connect_lock);
+#endif
 }
 
 static int
@@ -131,7 +133,7 @@ check_feature_flags(int ctxflags, int features, int local_conn)
 	    /* PMCD requires a client cert, but we are not even setup for secure connections */
 	    return PM_ERR_NEEDCLIENTCERT;
 	}
-     }
+    }
 
     if (ctxflags) {
 	/*
@@ -238,7 +240,7 @@ __pmConnectHandshake(int fd, const char *hostname, int ctxflags, __pmHashCtl *at
     __pmPDU	*pb;
     int		ok;
     int		version;
-    int		challenge;
+    int		challenge = 0;
     int		sts;
     int		pinpdu;
 
