@@ -1,7 +1,7 @@
 /*
  * Common argument parsing for all PMAPI client tools.
  *
- * Copyright (c) 2014-2018 Red Hat.
+ * Copyright (c) 2014-2018,2020 Red Hat.
  * Copyright (C) 1987-2014 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or modify it
@@ -516,7 +516,6 @@ __pmAddOptArchiveFolio(pmOptions *opts, char *arg)
 	opts->flags |= PM_OPTFLAG_RUNTIME_ERR;
 	opts->errors++;
     } else {
-	size_t length;
 	char *p, *log, *dir;
 	int line, sep = pmPathSeparator();
 
@@ -565,12 +564,15 @@ __pmAddOptArchiveFolio(pmOptions *opts, char *arg)
 	    p = skip_nonwhitespace(p);
 	    *p = '\0';
 
-	    length = strlen(dir) + 1 + strlen(log) + 1;
-	    if ((p = (char *)malloc(length)) == NULL)
-		pmNoMem("pmGetOptions(archive)", length, PM_FATAL_ERR);
-	    pmsprintf(p, length, "%s%c%s", dir, sep, log);
-	    __pmAddOptArchive(opts, p);
-	    free(p);
+	    if (__pmAbsolutePath(log)) {
+		/* absolute path to archive - do not prefix with dir */
+		__pmAddOptArchive(opts, log);
+	    }
+	    else {
+		char archive[MAXPATHLEN];
+		pmsprintf(archive, sizeof(archive), "%s%c%s", dir, sep, log);
+		__pmAddOptArchive(opts, archive);
+	    }
 	}
 
 	fclose(fp);
@@ -757,7 +759,8 @@ __pmSetSampleCount(pmOptions *opts, char *arg)
 
     opts->samples = (int)strtol(arg, &endnum, 10);
     if (*endnum != '\0' || opts->samples < 0) {
-	pmprintf("%s: -s requires numeric argument\n", pmGetProgname());
+	pmprintf("%s: -s requires a positive numeric argument\n",
+		pmGetProgname());
 	opts->errors++;
     }
 }
