@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2012-2013 Red Hat.
  * Copyright (c) 1995 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This library is free software; you can redistribute it and/or modify it
@@ -13,7 +14,10 @@
  */
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
+#include "internal.h"
+
+#define LIMIT_CREDS	1024
 
 /*
  * PDU for process credentials (PDU_CREDS)
@@ -27,12 +31,12 @@ typedef struct {
 int
 __pmSendCreds(int fd, int from, int credcount, const __pmCred *credlist)
 {
-    size_t	need = 0;
-    creds_t	*pp = NULL;
+    size_t	need;
+    creds_t	*pp;
     int		i;
     int		sts;
 
-    if (credcount <= 0 || credlist == NULL)
+    if (credcount <= 0 || credcount > LIMIT_CREDS || credlist == NULL)
 	return PM_ERR_IPC;
 
     need = sizeof(creds_t) + ((credcount-1) * sizeof(__pmCred));
@@ -42,11 +46,9 @@ __pmSendCreds(int fd, int from, int credcount, const __pmCred *credlist)
     pp->hdr.type = PDU_CREDS;
     pp->hdr.from = from;
     pp->numcreds = htonl(credcount);
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_CONTEXT)
+    if (pmDebugOptions.context)
 	for (i = 0; i < credcount; i++)
 	    fprintf(stderr, "__pmSendCreds: #%d = %x\n", i, *(unsigned int*)&(credlist[i]));
-#endif
     /* swab and fix bitfield order */
     for (i = 0; i < credcount; i++)
 	pp->credlist[i] = __htonpmCred(credlist[i]);
@@ -61,12 +63,20 @@ __pmDecodeCreds(__pmPDU *pdubuf, int *sender, int *credcount, __pmCred **credlis
 {
     creds_t	*pp;
     int		i;
+    int		len;
+    int		need;
     int		numcred;
     __pmCred	*list;
 
     pp = (creds_t *)pdubuf;
+    len = pp->hdr.len;		/* ntohl() converted already in __pmGetPDU() */
     numcred = ntohl(pp->numcreds);
-    if (pp == NULL || numcred < 0) return PM_ERR_IPC;
+    if (numcred < 0 || numcred > LIMIT_CREDS)
+	return PM_ERR_IPC;
+    need = sizeof(creds_t) + ((numcred-1) * sizeof(__pmCred));
+    if (need != len)
+	return PM_ERR_IPC;
+
     *sender = pp->hdr.from;	/* ntohl() converted already in __pmGetPDU() */
     if ((list = (__pmCred *)malloc(sizeof(__pmCred) * numcred)) == NULL)
 	return -oserror();
@@ -76,13 +86,11 @@ __pmDecodeCreds(__pmPDU *pdubuf, int *sender, int *credcount, __pmCred **credlis
 	list[i] = __ntohpmCred(pp->credlist[i]);
     }
 
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_CONTEXT)
+    if (pmDebugOptions.context)
 	for (i = 0; i < numcred; i++)
 	    fprintf(stderr, "__pmDecodeCreds: #%d = { type=0x%x a=0x%x b=0x%x c=0x%x }\n",
 		i, list[i].c_type, list[i].c_vala,
 		list[i].c_valb, list[i].c_valc);
-#endif
 
     *credlist = list;
     *credcount = numcred;
