@@ -34,6 +34,7 @@
 #include "internal.h"
 #include "fault.h"
 #include <assert.h>
+#include <errno.h>
 
 #ifdef PM_MULTI_THREAD
 static pthread_mutex_t	pdu_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -236,10 +237,12 @@ pduread(int fd, char *buf, int len, int part, int timeout)
 		return PM_ERR_TIMEOUT;
 	    }
 	    else if (status < 0) {
-		char	errmsg[PM_MAXERRMSGLEN];
 		status = -neterror();
-		pmNotifyErr(LOG_ERR, "pduread: select() on fd=%d status=%d: %s",
+		if (status != -EINTR) {
+		    char	errmsg[PM_MAXERRMSGLEN];
+		    pmNotifyErr(LOG_ERR, "pduread: select() on fd=%d status=%d: %s",
 			fd, status, netstrerror_r(errmsg, sizeof(errmsg)));
+		}
 		setoserror(neterror());
 		return status;
 	    }
@@ -251,12 +254,21 @@ pduread(int fd, char *buf, int len, int part, int timeout)
 	    status = read(fd, buf, len);
 	}
 	__pmOverrideLastFd(fd);
-	if (status < 0)
-	    /* error */
+	if (status <= 0) {
+	    /* end of file or error */
+	    if (pmDebugOptions.pdu && pmDebugOptions.desperate) {
+		char	errmsg[PM_MAXERRMSGLEN];
+		fprintf(stderr, "pduread(%d, ...): eof/error %d from %s: ",
+			fd, status, socketipc == 0 ? "read" : "__pmRecv");
+		fprintf(stderr, "%s\n", osstrerror_r(errmsg, sizeof(errmsg)));
+	    }
+	    if (status == 0)
+		/* return what we have, or nothing */
+		break;
+
+	    /* else error return */
 	    return status;
-	else if (status == 0)
-	    /* return what we have, or nothing */
-	    break;
+	}
 
 	have += status;
 	buf += status;
@@ -267,6 +279,9 @@ pduread(int fd, char *buf, int len, int part, int timeout)
 	}
     }
 
+    if (pmDebugOptions.pdu && pmDebugOptions.desperate)
+	fprintf(stderr, "pduread(%d, ...): return %d\n", fd, have);
+
     return have;
 }
 
@@ -436,9 +451,7 @@ PM_FAULT_RETURN(PM_FAULT_TIMEOUT);
 
     if (len < (int)sizeof(__pmPDUHdr)) {
 	if (len == -1) {
-	    if (__pmSocketClosed()) {
-		len = 0;
-	    } else {
+	    if (! __pmSocketClosed()) {
 		char	errmsg[PM_MAXERRMSGLEN];
 		pmNotifyErr(LOG_ERR, "__pmGetPDU: fd=%d hdr read: len=%d: %s", fd, len, pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
 	    }
