@@ -436,7 +436,7 @@ pmGetPMNSLocation(void)
  * linked-list structures)
  */
 static void
-dumptree(FILE *f, int level, __pmnsNode *rp, int verbosity)
+dumptree(FILE *f, int level, const __pmnsNode *rp, int verbosity)
 {
     int		i;
     __pmID_int	*pp;
@@ -828,11 +828,16 @@ backname(__pmnsNode *np, char **name)
 
     nch = 0;
     xp = np;
-    while (xp->parent != NULL) {
+    while (xp && xp->parent != NULL) {
 	nch += (int)strlen(xp->name)+1;
 	xp = xp->parent;
     }
 
+    if (!nch) {
+	/* invalid PMNS argument */
+    	return PM_ERR_PMNS;
+    }
+
     if ((p = (char *)malloc(nch)) == NULL)
 	return -oserror();
 
@@ -1685,7 +1690,7 @@ pmUnloadNameSpace(void)
  * current context is not locked.
  */
 int
-pmLookupName_ctx(__pmContext *ctxp, int derive_locked, int numpmid, char *namelist[], pmID pmidlist[])
+pmLookupName_ctx(__pmContext *ctxp, int derive_locked, int numpmid, const char *namelist[], pmID pmidlist[])
 {
     int		pmns_location;
     int		sts = 0;
@@ -1934,7 +1939,7 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_TIMEOUT);
 	nfail = 0;
 	for (i = 0; i < numpmid; i++) {
 	    if (pmidlist[i] == PM_ID_NULL) {
-		lsts = __dmgetpmid(derive_locked, namelist[i], &pmidlist[i]);
+		lsts = __dmgetpmid(ctxp, derive_locked, namelist[i], &pmidlist[i]);
 		if (lsts < 0) {
 		    nfail++;
 		}
@@ -1996,7 +2001,7 @@ pmapi_return:
 }
 
 int
-pmLookupName(int numpmid, char *namelist[], pmID pmidlist[])
+pmLookupName(int numpmid, const char *namelist[], pmID pmidlist[])
 {
     int	sts;
     sts = pmLookupName_ctx(NULL, PM_NOT_LOCKED, numpmid, namelist, pmidlist);
@@ -2413,7 +2418,7 @@ report:
 	    if (statuslist != NULL)
 		__pmDumpNameAndStatusList(stderr, num, *offspring, *statuslist);
 	    else
-		__pmDumpNameList(stderr, num, *offspring);
+		__pmDumpNameList(stderr, num, (const char **)*offspring);
 	}
 	else {
 	    char	errmsg[PM_MAXERRMSGLEN];
@@ -2618,7 +2623,7 @@ pmNameID(pmID pmid, char **name)
      * failed everything else, try derived metric, but if this fails
      * return last error from above ...
      */
-    lsts = __dmgetname(pmid, name);
+    lsts = __dmgetname(ctxp, pmid, name);
     if (lsts >= 0)
 	sts = lsts;
 
@@ -2790,7 +2795,7 @@ pmNameAll_ctx(__pmContext *ctxp, pmID pmid, char ***namelist)
 	sts = -oserror();
 	goto pmapi_return;
     }
-    n = __dmgetname(pmid, tmp);
+    n = __dmgetname(ctxp, pmid, tmp);
     if (n < 0) {
 	free(tmp);
 	if (sts >= 0)
@@ -3185,7 +3190,7 @@ __pmDumpNameSpace(FILE *f, int verbosity)
  * helper method ... linux_proc PMDA seems to be the only caller
  */
 void
-__pmDumpNameNode(FILE *f, __pmnsNode *node, int verbosity)
+__pmDumpNameNode(FILE *f, const __pmnsNode *node, int verbosity)
 {
     dumptree(f, 0, node, verbosity);
 }
