@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2018 Red Hat.
+ * Copyright (c) 2014,2018-2019 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -50,10 +50,8 @@ typedef struct connectionContext {
     int			*numUrls;	/* Size of the results */
     char		***urls;	/* The results */
     const connectionOptions *options;	/* Connection options */
-#if PM_MULTI_THREAD
     __pmMutex		addrLock;	/* lock for the above address/port */
     __pmMutex		urlLock;	/* lock for the above results */
-#endif
 } connectionContext;
 
 /*
@@ -196,8 +194,8 @@ attemptConnections(void *arg)
 		    serviceInfo.protocol = SERVER_PROTOCOL;
 		else if (strcmp(context->service, PM_SERVER_PROXY_SPEC) == 0)
 		    serviceInfo.protocol = PROXY_PROTOCOL;
-		else if (strcmp(context->service, PM_SERVER_WEBD_SPEC) == 0)
-		    serviceInfo.protocol = PMWEBD_PROTOCOL;
+		else if (strcmp(context->service, PM_SERVER_WEBAPI_SPEC) == 0)
+		    serviceInfo.protocol = WEBAPI_PROTOCOL;
 
 		PM_LOCK(context->urlLock);
 		*context->numUrls =
@@ -359,8 +357,8 @@ addTarget(char *target, connectionOptions *options)
 	bytes = options->maxTargets * sizeof(struct targetInfo);
 	if ((tp = realloc(options->targets, bytes)) == NULL) {
 	    if (pmDebugOptions.discovery)
-		pmNotifyErr(LOG_ERR, "%s: failed targets realloc: %lld bytes",
-				PROBE, (long long)bytes);
+		pmNotifyErr(LOG_ERR, "%s: failed targets realloc: %" FMT_INT64 " bytes",
+				PROBE, (__int64_t)bytes);
 	    return;
 	}
 	options->targets = tp;
