@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2012-2015 Red Hat.
  * Copyright (c) 1995-2001,2004 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -10,25 +11,65 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include "pmcd.h"
 
-PMCD_INTERN ClientInfo	*client;
-PMCD_INTERN int		nClients;	/* Number in array, (not all in use) */
-PMCD_INTERN int		this_client_id;
+PMCD_DATA ClientInfo	*client;
+PMCD_DATA int		nClients;	/* Number in array, (not all in use) */
+PMCD_DATA int		this_client_id;
+
+/*
+ * Expose ClientInfo struct for client #n
+ */
+ClientInfo *
+GetClient(int n)
+{
+    if (0 <= n && n < nClients && client[n].status.connected)
+	return &client[n];
+    return NULL;
+}
+
+/*
+ * Modify an attribute value for this client
+ */
+int
+SetClientAttribute(int client, int attr, char *value)
+{
+    int			sts;
+    ClientInfo		*cp;
+    __pmHashNode	*node;
+
+    if ((cp = GetClient(client)) == NULL)
+	return -EINVAL;
+
+    if ((value = strdup(value)) == NULL)
+	return -ENOMEM;
+
+    /* Add supplied value into attribute hash, permanently */
+    if ((node = __pmHashSearch(attr, &cp->attrs)) == NULL) {	/* insert */
+	if ((sts = __pmHashAdd(attr, (void *)value, &cp->attrs)) < 0) {
+	    free(value);
+	    return sts;
+	}
+    } else {	/* replace existing */
+	free(node->data);
+	node->data = (void *)value;
+    }
+
+    /* Flag a need to inform PMDAs that attributes changed */
+    cp->status.attributes = 1;
+    return 0;
+}
 
 void
 ShowClients(FILE *f)
 {
     int			i;
-    struct hostent	*hp;
+    char		*sbuf;
+    char		*hostName;
 
     fprintf(f, "     fd  client connection from                    ipc ver  operations denied\n");
     fprintf(f, "     ==  ========================================  =======  =================\n");
@@ -38,20 +79,15 @@ ShowClients(FILE *f)
 
 	fprintf(f, "    %3d  ", client[i].fd);
 
-	hp = gethostbyaddr((void *)&client[i].addr.sin_addr.s_addr, sizeof(client[i].addr.sin_addr.s_addr), AF_INET);
-	if (hp == NULL) {
-	    char	*p = (char *)&client[i].addr.sin_addr.s_addr;
-	    int	k;
-
-	    for (k = 0; k < 4; k++) {
-		if (k > 0)
-		    fputc('.', f);
-		fprintf(f, "%d", p[k] & 0xff);
-	    }
+	hostName = __pmGetNameInfo(client[i].addr);
+	if (hostName == NULL) {
+	    sbuf = __pmSockAddrToString(client[i].addr);
+	    fprintf(f, "%s", sbuf);
+	    free(sbuf);
+	} else {
+	    fprintf(f, "%-40.40s", hostName);
+	    free(hostName);
 	}
-	else
-	    fprintf(f, "%-40.40s", hp->h_name);
-
 	fprintf(f, "  %7d", __pmVersionIPC(client[i].fd));
 
 	if (client[i].denyOps != 0) {
