@@ -17,10 +17,7 @@
 #define QMC_SOURCE_H
 
 #include "qmc.h"
-
-#include <qlist.h>
-#include <qstring.h>
-#include <qtextstream.h>
+#include "qmc_config.h"
 
 class QmcSource
 {
