@@ -1,17 +1,16 @@
 /*
  * Copyright (c) 2019 Red Hat.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
- * option) any later version.
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
  */
-
 #include "pmwebapi.h"
 #include "libpcp.h"
 #include "util.h"
@@ -71,7 +70,7 @@ pmIniFileSetup(const char *progname)
 {
     dict	*config;
 
-    if ((config = dictCreate(&sdsDictCallBacks, "pmIniFileSetup")) == NULL)
+    if ((config = dictCreate(&sdsOwnDictCallBacks, "pmIniFileSetup")) == NULL)
 	return NULL;
     if (pmIniFileParse(progname, dict_handler, config) == 0)
 	return config;
@@ -88,7 +87,6 @@ pmIniFileUpdate(dict *config, const char *group, const char *key, sds value)
     if (pmDebugOptions.libweb)
 	fprintf(stderr, "pmIniFileUpdate set %s = %s\n", name, value);
     dictReplace(config, name, value);
-    sdsfree(name);
 }
 
 sds
