@@ -70,7 +70,7 @@ static uint64_t
 sdsHashCallBack(const void *key)
 {
     return dictGenHashFunction((unsigned char *)key, sdslen((char *)key));
-} 
+}
 
 static int
 sdsCompareCallBack(void *privdata, const void *key1, const void *key2)
@@ -104,6 +104,13 @@ dictType sdsKeyDictCallBacks = {
     .keyDestructor	= sdsFreeCallBack,
 };
 
+dictType sdsOwnDictCallBacks = {
+    .hashFunction	= sdsHashCallBack,
+    .keyCompare		= sdsCompareCallBack,
+    .keyDestructor	= sdsFreeCallBack,
+    .valDestructor	= sdsFreeCallBack,
+};
+
 dictType sdsDictCallBacks = {
     .hashFunction	= sdsHashCallBack,
     .keyCompare		= sdsCompareCallBack,
