@@ -1905,8 +1905,13 @@ pmDiscoverSetup(pmDiscoverModule *module, pmDiscoverCallBacks *cbs, void *arg)
     if ((option = pmIniFileLookup(config, "discover", "exclude.metrics"))) {
 	if ((data->pmids = dictCreate(&intKeyDictCallBacks, NULL)) == NULL)
 	    return -ENOMEM;
-	/* parse regular expression string for matching on metric names */
-	regcomp(&data->exclude_names, option, REG_EXTENDED|REG_NOSUB);
+	/* parse comma-separated metric name glob patterns, in 'option' */
+	if ((ids = sdssplitlen(option, sdslen(option), ",", 1, &nids))) {
+	    data->exclude_names = nids;
+	    for (i = 0; i < nids; i++)
+		ids[i] = sdstrim(ids[i], " ");
+	    data->patterns = ids;
+	}
     }
     if ((option = pmIniFileLookup(config, "discover", "exclude.indoms"))) {
 	if ((data->indoms = dictCreate(&intKeyDictCallBacks, NULL)) == NULL)
@@ -1948,11 +1953,20 @@ void
 pmDiscoverClose(pmDiscoverModule *module)
 {
     discoverModuleData	*discover = (discoverModuleData *)module->privdata;
+    unsigned int	i;
 
     if (discover) {
 	pmDiscoverUnregister(discover->handle);
 	if (!discover->shareslots)
 	    redisSlotsFree(discover->slots);
+	for (i = 0; i < discover->exclude_names; i++)
+	    sdsfree(discover->patterns[i]);
+	if (discover->patterns)
+	    free(discover->patterns);
+	if (discover->pmids)
+	    dictRelease(discover->pmids);
+	if (discover->indoms)
+	    dictRelease(discover->indoms);
 	memset(discover, 0, sizeof(*discover));
 	free(discover);
     }
