@@ -77,13 +77,13 @@ zmalloc(size_t size)
 }
 
 void *
-zcalloc(size_t size)
+zcalloc(size_t elem, size_t size)
 {
     void	*p;
 
-    p = calloc(1, size);
+    p = calloc(elem, size);
     if (UNLIKELY(p == NULL))
-	oom_handler(size);
+	oom_handler(elem * size);
     return p;
 }
 
@@ -126,7 +126,7 @@ timeval_stream_str(struct timeval *stamp, char *buffer, int buflen)
     __uint64_t	fractions;
     __uint64_t	crossover = stamp->tv_usec / 1000;
 
-    millipart = stamp->tv_sec * 1000;
+    millipart = ((__uint64_t)stamp->tv_sec) * 1000;
     millipart += crossover;
     fractions = stamp->tv_usec % 1000;
     pmsprintf(buffer, buflen, "%" FMT_UINT64 "-%"FMT_UINT64,
@@ -321,9 +321,9 @@ pmwebapi_source_meta(context_t *c, char *buffer, int length)
     char	host[MAXHOSTNAMELEN];
     int		sts;
 
-    if ((pmGetContextHostName_r(c->context, host, sizeof(host))) == NULL) {
+    if ((sts = pmGetHostName(c->context, host, sizeof(host))) < 0) {
 	c->setup = 0;
-	return PM_ERR_GENERIC;
+	return sts;
     }
     c->host = sdsnew(host);
     sts = pmGetContextLabels(set);
@@ -337,7 +337,7 @@ pmwebapi_source_meta(context_t *c, char *buffer, int length)
 static int
 context_labels_str(context_t *c, char *buffer, int length)
 {
-    return pmMergeLabelSets(&c->labelset, 1, buffer, length, labels, NULL);
+    return pmMergeLabelSets(&c->labelset, 1, buffer, length, NULL, NULL);
 }
 
 int
@@ -488,6 +488,22 @@ pmwebapi_source_hash(unsigned char *hash, const char *labels, int length)
     return 0;
 }
 
+int
+pmwebapi_search_hash(unsigned char *hash, const char *string, int length)
+{
+    SHA1_CTX		shactx;
+    const char		prefix[] = "{\"series\":\"search\",";
+    const char		suffix[] = "}";
+
+    /* Calculate unique source identifier 20-byte SHA1 hash */
+    SHA1Init(&shactx);
+    SHA1Update(&shactx, (unsigned char *)prefix, sizeof(prefix)-1);
+    SHA1Update(&shactx, (unsigned char *)string, length);
+    SHA1Update(&shactx, (unsigned char *)suffix, sizeof(suffix)-1);
+    SHA1Final(hash, &shactx);
+    return 0;
+}
+
 int
 pmwebapi_context_hash(context_t *context)
 {
@@ -591,8 +607,8 @@ pmwebapi_new_context(context_t *cp)
     /* establish PMAPI context */
     if ((sts = cp->context = pmNewContext(cp->type, cp->name.sds)) < 0) {
 	if (cp->type == PM_CONTEXT_HOST)
-	    infofmt(msg, "cannot connect to PMCD on host \"%s\": %s",
-		    cp->name.sds, pmErrStr_r(sts, pmmsg, sizeof(pmmsg)));
+	    infofmt(msg, "cannot connect to PMCD: %s",
+		    pmErrStr_r(sts, pmmsg, sizeof(pmmsg)));
 	else if (cp->type == PM_CONTEXT_LOCAL)
 	    infofmt(msg, "cannot make standalone connection on localhost: %s",
 		    pmErrStr_r(sts, pmmsg, sizeof(pmmsg)));
