@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019-2020 Red Hat.
+ * Copyright (c) 2019-2021 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -23,7 +23,10 @@
 #endif
 #include <fnmatch.h>
 
-#define DEFAULT_TIMEOUT 5000
+#define DEFAULT_WORK_TIMER 2000
+static unsigned int default_worker;	/* BG work delta, milliseconds */
+
+#define DEFAULT_POLL_TIMEOUT 5000
 static unsigned int default_timeout;	/* timeout in milliseconds */
 
 #define DEFAULT_BATCHSIZE 256
@@ -35,7 +38,7 @@ static sds PARAM_HOSTNAME, PARAM_HOSTSPEC, PARAM_CTXNUM, PARAM_CTXID,
            PARAM_PMIDS, PARAM_PMID, PARAM_INDOM, PARAM_INSTANCE,
            PARAM_INAME, PARAM_MVALUE, PARAM_TARGET, PARAM_EXPR, PARAM_MATCH;
 static sds AUTH_USERNAME, AUTH_PASSWORD;
-static sds EMPTYSTRING, LOCALHOST, TIMEOUT, BATCHSIZE;
+static sds EMPTYSTRING, LOCALHOST, WORK_TIMER, POLL_TIMEOUT, BATCHSIZE;
 
 enum matches { MATCH_EXACT, MATCH_GLOB, MATCH_REGEX };
 enum profile { PROFILE_ADD, PROFILE_DEL };
@@ -45,6 +48,8 @@ typedef struct webgroups {
     mmv_registry_t	*metrics;
     struct dict		*config;
     uv_loop_t		*events;
+    unsigned int	active;
+    uv_timer_t		timer;
 } webgroups;
 
 static struct webgroups *
@@ -55,29 +60,43 @@ webgroups_lookup(pmWebGroupModule *module)
     return (struct webgroups *)module->privdata;
 }
 
+static int
+webgroup_deref_context(struct context *cp)
+{
+    if (cp == NULL)
+	return 1;
+    if (cp->refcount == 0)
+	return 0;
+    return (--cp->refcount > 0);
+}
+
 static void
 webgroup_release_context(uv_handle_t *handle)
 {
     struct context	*context = (struct context *)handle->data;
 
-    if (pmDebugOptions.http)
-	fprintf(stderr, "releasing context %p\n", context);
-
+    if (pmDebugOptions.http || pmDebugOptions.libweb)
+	fprintf(stderr, "releasing context %p [refcount=%u]\n",
+			context, context->refcount);
     pmwebapi_free_context(context);
 }
 
 static void
-webgroup_destroy_context(struct context *context, struct webgroups *groups)
+webgroup_drop_context(struct context *context, struct webgroups *groups)
 {
-    context->garbage = 1;
+    if (pmDebugOptions.http || pmDebugOptions.libweb)
+	fprintf(stderr, "destroying context %p [refcount=%u]\n",
+			context, context->refcount);
 
-    if (pmDebugOptions.http)
-	fprintf(stderr, "destroying context %p\n", context);
-
-    uv_timer_stop(&context->timer);
-    if (groups)
-	dictUnlink(groups->contexts, &context->randomid);
-    uv_close((uv_handle_t *)&context->timer, webgroup_release_context);
+    if (webgroup_deref_context(context) == 0) {
+	if (context->garbage == 0) {
+	    context->garbage = 1;
+	    uv_timer_stop(&context->timer);
+	}
+	if (groups)
+	    dictUnlink(groups->contexts, &context->randomid);
+	uv_close((uv_handle_t *)&context->timer, webgroup_release_context);
+    }
 }
 
 static void
@@ -85,12 +104,20 @@ webgroup_timeout_context(uv_timer_t *arg)
 {
     uv_handle_t		*handle = (uv_handle_t *)arg;
     struct context	*cp = (struct context *)handle->data;
-    struct webgroups	*gp = (struct webgroups *)cp->privdata;
 
-    if (pmDebugOptions.http)
+    if (pmDebugOptions.http || pmDebugOptions.libweb)
 	fprintf(stderr, "context %u timed out (%p)\n", cp->randomid, cp);
 
-    webgroup_destroy_context(cp, gp);
+    /*
+     * Cannot free data structures in the timeout handler, as
+     * they may still be actively in use - wait until reference
+     * is returned to zero by the caller, or background cleanup
+     * finds this context and cleans it.
+     */
+    if (cp->refcount == 0 && cp->garbage == 0) {
+	cp->garbage = 1;
+	uv_timer_stop(&cp->timer);
+    }
 }
 
 static int
@@ -148,7 +175,7 @@ webgroup_new_context(pmWebGroupSettings *sp, dict *params,
 {
     struct webgroups	*groups = webgroups_lookup(&sp->module);
     struct context	*cp;
-    unsigned int	polltime = DEFAULT_TIMEOUT;
+    unsigned int	polltime = DEFAULT_POLL_TIMEOUT;
     uv_handle_t		*handle;
     pmWebAccess		access;
     double		seconds;
@@ -223,36 +250,82 @@ webgroup_new_context(pmWebGroupSettings *sp, dict *params,
 
     cp->privdata = groups;
     cp->setup = 1;
+
+    if (pmDebugOptions.http || pmDebugOptions.libweb)
+	fprintf(stderr, "new context[%d] setup (%p)\n", cp->randomid, cp);
+
     return cp;
 }
 
+static void
+webgroup_garbage_collect(struct webgroups *groups)
+{
+    dictIterator        *iterator = dictGetSafeIterator(groups->contexts);
+    dictEntry           *entry;
+    context_t		*cp;
+
+    if (pmDebugOptions.http || pmDebugOptions.libweb)
+	fprintf(stderr, "%s: started\n", "webgroup_garbage_collect");
+
+    while ((entry = dictNext(iterator)) != NULL) {
+	cp = (context_t *)dictGetVal(entry);
+	if (cp->garbage && cp->privdata == groups) {
+	    if (pmDebugOptions.http || pmDebugOptions.libweb)
+		fprintf(stderr, "GC context %u (%p)\n", cp->randomid, cp);
+	    webgroup_drop_context(cp, groups);
+	}
+    }
+    dictReleaseIterator(iterator);
+
+    if (pmDebugOptions.http || pmDebugOptions.libweb)
+	fprintf(stderr, "%s: finished\n", "webgroup_garbage_collect");
+}
+
+static void
+webgroup_worker(uv_timer_t *arg)
+{
+    uv_handle_t		*handle = (uv_handle_t *)arg;
+    struct webgroups	*groups = (struct webgroups *)handle->data;
+
+    webgroup_garbage_collect(groups);
+}
+
 static struct context *
 webgroup_use_context(struct context *cp, int *status, sds *message, void *arg)
 {
     char		errbuf[PM_MAXERRMSGLEN];
     int			sts;
+    struct webgroups    *gp = (struct webgroups *)cp->privdata;
 
-    if (pmDebugOptions.http)
-	fprintf(stderr, "context %u timer set (%p) to %u msec\n",
-			cp->randomid, cp, cp->timeout);
-
-    /* if already started, uv_timer_start updates the timer */
-    uv_timer_start(&cp->timer, webgroup_timeout_context, cp->timeout, 0);
-
-    if (cp->setup == 0) {
-	if ((sts = pmReconnectContext(cp->context)) < 0) {
-	    infofmt(*message, "cannot reconnect context: %s",
+    if (cp->garbage == 0) {
+	if (cp->setup == 0) {
+	    if ((sts = pmReconnectContext(cp->context)) < 0) {
+		infofmt(*message, "cannot reconnect context: %s",
+			pmErrStr_r(sts, errbuf, sizeof(errbuf)));
+		*status = sts;
+		return NULL;
+	    }
+	    cp->setup = 1;
+	}
+	if ((sts = pmUseContext(cp->context)) < 0) {
+	    infofmt(*message, "cannot use existing context: %s",
 			pmErrStr_r(sts, errbuf, sizeof(errbuf)));
 	    *status = sts;
 	    return NULL;
 	}
-	cp->setup = 1;
-    }
 
-    if ((sts = pmUseContext(cp->context)) < 0) {
-	infofmt(*message, "cannot use existing context: %s",
-			pmErrStr_r(sts, errbuf, sizeof(errbuf)));
-	*status = sts;
+	if (pmDebugOptions.http || pmDebugOptions.libweb)
+	    fprintf(stderr, "context %u timer set (%p) to %u msec\n",
+			cp->randomid, cp, cp->timeout);
+
+	/* refresh current time: https://github.com/libuv/libuv/issues/1068 */
+	uv_update_time(gp->events);
+
+	/* if already started, uv_timer_start updates the existing timer */
+	uv_timer_start(&cp->timer, webgroup_timeout_context, cp->timeout, 0);
+    } else {
+	infofmt(*message, "expired context identifier: %u", cp->randomid);
+	*status = -ENOTCONN;
 	return NULL;
     }
 
@@ -269,6 +342,15 @@ webgroup_lookup_context(pmWebGroupSettings *sp, sds *id, dict *params,
     pmWebAccess		access;
     char		*endptr = NULL;
 
+    if (groups->active == 0) {
+	groups->active = 1;
+	/* install general background work timer (GC) */
+	uv_timer_init(groups->events, &groups->timer);
+	groups->timer.data = (void *)groups;
+	uv_timer_start(&groups->timer, webgroup_worker,
+			default_worker, default_worker);
+    }
+
     if (*id == NULL) {
 	if (!(cp = webgroup_new_context(sp, params, status, message, arg)))
 	    return NULL;
@@ -285,20 +367,20 @@ webgroup_lookup_context(pmWebGroupSettings *sp, sds *id, dict *params,
 	    *status = -ENOTCONN;
 	    return NULL;
 	}
-	if (cp->garbage) {
-	    infofmt(*message, "expired context identifier: %u", key);
-	    *status = -ENOTCONN;
-	    return NULL;
+	if (cp->garbage == 0) {
+	    access.username = cp->username;
+	    access.password = cp->password;
+	    access.realm = cp->realm;
+	    if (sp->callbacks.on_check &&
+		sp->callbacks.on_check(*id, &access, status, message, arg) < 0) {
+		return NULL;
+	    }
 	}
-	access.username = cp->username;
-	access.password = cp->password;
-	access.realm = cp->realm;
-	if (sp->callbacks.on_check &&
-	    sp->callbacks.on_check(*id, &access, status, message, arg) < 0)
-	    return NULL;
     }
 
-    return webgroup_use_context(cp, status, message, arg);
+    if ((cp = webgroup_use_context(cp, status, message, arg)) != NULL)
+	cp->refcount++;
+    return cp;
 }
 
 int
@@ -326,6 +408,7 @@ pmWebGroupContext(pmWebGroupSettings *sp, sds id, dict *params, void *arg)
     }
 
     sp->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
     return sts;
 }
@@ -344,9 +427,9 @@ pmWebGroupDestroy(pmWebGroupSettings *settings, sds id, void *arg)
 	gp = settings->module.privdata;
 
 	if (pmDebugOptions.libweb)
-	    fprintf(stderr, "%s: destroy context %p\n", "pmWebGroupDestroy", cp);
+	    fprintf(stderr, "%s: destroy context %p gp=%p\n", "pmWebGroupDestroy", cp, gp);
 
-	webgroup_destroy_context(cp, gp);
+	webgroup_drop_context(cp, gp);
     }
     sdsfree(msg);
 }
@@ -358,16 +441,18 @@ ismarker(int c)
 }
 
 static int
-webgroup_derived_metrics(sds config, sds *errmsg)
+webgroup_derived_metrics(context_t *cp, sds config, sds *errmsg)
 {
     unsigned int	line = 0;
     char		*p, *end, *name = NULL, *expr, *error;
 
+    (void)cp;
+
     /*
      * Pick apart the derived metrics "configuration file" in expr.
      * - skip comments and blank lines
      * - split each line on name '=' expression
-     * - call pmRegisterDerivedMetric for each, propogating any errors
+     * - call pmAddDerivedMetric for each, propogating any errors
      *   (with an additional 'line number' and explanatory notes).
      */
     end = config + sdslen(config);
@@ -404,8 +489,8 @@ webgroup_derived_metrics(sds config, sds *errmsg)
 	    break;
 	*p = '\0';
 
-	/* register the derived metric and reset the parsing */
-	if (pmRegisterDerivedMetric(name, expr, &error) < 0) {
+	/* add the derived metric to this context and reset the parsing */
+	if (pmAddDerivedMetric(name, expr, &error) < 0) {
 	    infofmt(*errmsg, "failed to create derived metric \"%s\""
 			" on line %u from: %s\n%s", name, line, expr, error);
 	    free(error);
@@ -431,6 +516,7 @@ webgroup_derived_metrics(sds config, sds *errmsg)
 extern void
 pmWebGroupDerive(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 {
+    struct context	*cp;
     sds			msg = NULL, expr, metric, message;
     int			sts = 0;
 
@@ -440,17 +526,19 @@ pmWebGroupDerive(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
     } else {
 	metric = expr = NULL;
     }
+    if (!(cp = webgroup_lookup_context(settings, &id, params, &sts, &msg, arg)))
+	goto done;
+    id = cp->origin;
 
     if (expr && !metric) {	/* configuration file mode */
 	if (pmDebugOptions.libweb)
-	    fprintf(stderr, "%s: register metric group\n",
-			    "pmWebGroupDerive");
-	sts = webgroup_derived_metrics(expr, &msg);
+	    fprintf(stderr, "%s: register metric group\n", "pmWebGroupDerive");
+	sts = webgroup_derived_metrics(cp, expr, &msg);
     } else if (expr && metric) {
 	if (pmDebugOptions.libweb)
-	    fprintf(stderr, "%s: register metric %s\n",
-			    "pmWebGroupDerive", metric);
-	if ((sts = pmRegisterDerivedMetric(metric, expr, &message)) < 0) {
+	    fprintf(stderr, "%s: register metric %s\n", "pmWebGroupDerive",
+			    metric);
+	if ((sts = pmAddDerivedMetric(metric, expr, &message)) < 0) {
 	    infofmt(msg, "%s", message);
 	    free(message);
 	}
@@ -459,7 +547,9 @@ pmWebGroupDerive(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 	sts = -EINVAL;
     }
 
-    settings->callbacks.on_done(NULL, sts, msg, arg);
+done:
+    settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -666,7 +756,7 @@ webgroup_lookup_metric(pmWebGroupSettings *settings, context_t *cp, sds name, vo
 
     if ((mp = dictFetchValue(cp->metrics, name)) != NULL)
 	return mp;
-    if ((sts = pmLookupName(1, &name, &pmid)) < 0) {
+    if ((sts = pmLookupName(1, (const char **)&name, &pmid)) < 0) {
 	if (sts == PM_ERR_IPC)
 	    cp->setup = 0;
 	if (pmDebugOptions.libweb)
@@ -683,10 +773,10 @@ webgroup_fetch_names(pmWebGroupSettings *settings, context_t *cp, int fail,
 	sds *message, void *arg)
 {
     struct metric	*metric;
-    int			i, status = 0;
+    int			i, sts = 0;
 
-    if (webgroup_use_context(cp, &status, message, arg) == NULL)
-	return status;
+    if (webgroup_use_context(cp, &sts, message, arg) == NULL)
+	return sts;
 
     for (i = 0; i < numnames; i++) {
 	metric = mplist[i] = webgroup_lookup_metric(settings, cp, names[i], arg);
@@ -703,10 +793,10 @@ webgroup_fetch_pmids(pmWebGroupSettings *settings, context_t *cp, int fail,
 	sds *message, void *arg)
 {
     struct metric	*metric;
-    int			i, status = 0;
+    int			i, sts = 0;
 
-    if (webgroup_use_context(cp, &status, message, arg) == NULL)
-	return status;
+    if (webgroup_use_context(cp, &sts, message, arg) == NULL)
+	return sts;
 
     for (i = 0; i < numpmids; i++) {
 	metric = mplist[i] = webgroup_lookup_pmid(settings, cp, names[i], arg);
@@ -794,6 +884,7 @@ pmWebGroupFetch(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 
 done:
     settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -982,7 +1073,7 @@ profile:
 extern void
 pmWebGroupProfile(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 {
-    struct context	*cp;
+    struct context	*cp = NULL;
     struct metric	*mp;
     struct indom	*ip;
     enum profile	profile = PROFILE_DEL;
@@ -1061,6 +1152,7 @@ pmWebGroupProfile(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 
 done:
     settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -1135,6 +1227,7 @@ pmWebGroupChildren(pmWebGroupSettings *settings, sds id, dict *params, void *arg
 
 done:
     settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -1185,7 +1278,7 @@ webgroup_instances(pmWebGroupSettings *settings,
 void
 pmWebGroupInDom(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 {
-    struct context	*cp;
+    struct context	*cp = NULL;
     struct domain	*dp;
     struct metric	*mp;
     struct indom	*ip;
@@ -1285,6 +1378,7 @@ pmWebGroupInDom(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 
 done:
     settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -1313,6 +1407,34 @@ typedef struct weblookup {
     void		*arg;
 } weblookup_t;
 
+/* Metric namespace failure callback for use after pmTraversePMNS_r(3) */
+static void
+badname(const char *name, const char *errmsg, struct weblookup *lookup)
+{
+    pmWebGroupSettings	*settings = lookup->settings;
+    pmWebMetric		*metric = &lookup->metric;
+    context_t		*cp = lookup->context;
+    void		*arg = lookup->arg;
+
+    /* clear buffer contents from any previous call(s) */
+    sdsclear(metric->series);
+    sdsclear(metric->name);
+    sdsclear(metric->sem);
+    sdsclear(metric->type);
+    sdsclear(metric->units);
+    sdsclear(metric->labels);
+    sdsclear(metric->oneline);
+    sdsclear(metric->helptext);
+
+    /* inform caller (callback) about failure via ID_NULL and oneline */
+    metric->pmid = PM_ID_NULL;
+    metric->indom = PM_INDOM_NULL;
+    metric->name = sdscat(metric->name, name);
+    metric->oneline = sdscat(metric->oneline, errmsg);
+
+    settings->callbacks.on_metric(cp->origin, metric, arg);
+}
+
 /* Metric namespace traversal callback for use with pmTraversePMNS_r(3) */
 static void
 webmetric_lookup(const char *name, void *arg)
@@ -1323,6 +1445,7 @@ webmetric_lookup(const char *name, void *arg)
     seriesname_t	*snp = NULL;
     context_t		*cp = lookup->context;
     metric_t		*mp;
+    indom_t		*ip;
 
     if (webgroup_use_context(cp, &lookup->status, &lookup->message, arg) == NULL)
 	return;
@@ -1341,16 +1464,21 @@ webmetric_lookup(const char *name, void *arg)
 
     metric->name = sdscat(metric->name, name);
     mp = webgroup_lookup_metric(settings, cp, metric->name, arg);
-    if (mp == NULL)
+    if (mp == NULL) {
+	badname(name, "failed to lookup metric name", lookup);
 	return;
+    }
     snp = webgroup_lookup_series(mp->numnames, mp->names, name);
     if (snp == NULL)	/* a 'redirect' - pick the first series */
 	snp = &mp->names[0];
 
     pmwebapi_add_domain_labels(cp, mp->cluster->domain);
     pmwebapi_add_cluster_labels(cp, mp->cluster);
-    if (mp->indom)
-	pmwebapi_add_indom_labels(mp->indom);
+    if ((ip = mp->indom) != NULL) {
+	if (pmwebapi_add_indom_instances(cp, ip) > 0)
+	    pmwebapi_add_instances_labels(cp, ip);
+	pmwebapi_add_indom_labels(ip);
+    }
     pmwebapi_add_item_labels(cp, mp);
     pmwebapi_metric_hash(mp);
     pmwebapi_metric_help(cp, mp);
@@ -1382,7 +1510,7 @@ pmWebGroupMetric(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
     struct context	*cp;
     pmWebMetric		*metric = &lookup.metric;
     size_t		length;
-    char		errmsg[PM_MAXERRMSGLEN];
+    char		errmsg[PM_MAXERRMSGLEN], *error;
     sds			msg = NULL, prefix = NULL, *names = NULL;
     int			i, sts = 0, numnames = 0;
 
@@ -1434,14 +1562,24 @@ pmWebGroupMetric(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
     for (i = 0; i < numnames; i++) {
 	sts = pmTraversePMNS_r(names[i], webmetric_lookup, &lookup);
 	if (sts >= 0) {
+	    if (numnames != 1) {	/* already started with response */
+		sts = 0;
+		continue;
+	    }
 	    msg = lookup.message;
 	    if ((sts = (lookup.status < 0) ? lookup.status : 0) < 0)
 		break;
 	} else {
 	    if (sts == PM_ERR_IPC)
 		cp->setup = 0;
-	    infofmt(msg, "%s traversal failed - %s", names[i],
-			    pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	    error = pmErrStr_r(sts, errmsg, sizeof(errmsg));
+	    if (numnames != 1) {
+		badname(names[i], error, &lookup);
+		sts = 0;
+	    } else {
+		infofmt(msg, "%s traversal failed - %s", names[i], error);
+		break;
+	    }
 	}
     }
 
@@ -1457,6 +1595,7 @@ pmWebGroupMetric(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 done:
     sdsfreesplitres(names, numnames);
     settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -1780,6 +1919,7 @@ pmWebGroupScrape(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 
 done:
     settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -1813,12 +1953,15 @@ store_add_instid(struct instore *store, int id)
 static void
 store_add_profile(struct instore *store)
 {
+    int		sts;
+
     if (store->insts) {
-	    int sts;
 	sts = pmDelProfile(store->indom, 0, NULL);
-	fprintf(stderr, "pmDelProfile: sts=%d\n", sts);
+	if (pmDebugOptions.libweb)
+	    fprintf(stderr, "pmDelProfile: sts=%d\n", sts);
 	sts = pmAddProfile(store->indom, store->count, store->insts);
-	fprintf(stderr, "pmAddProfile: sts=%d\n", sts);
+	if (pmDebugOptions.libweb)
+	    fprintf(stderr, "pmAddProfile: sts=%d\n", sts);
 	free(store->insts);
 	store->insts = NULL;
     }
@@ -1890,10 +2033,10 @@ webgroup_store(struct context *context, struct metric *metric,
     bytes = sizeof(pmValueSet) + sizeof(pmValue) * (count - 1);
     if ((result = (pmResult *)calloc(1, sizeof(pmResult))) == NULL ||
 	(valueset = (pmValueSet *)calloc(1, bytes)) == NULL) {
-	if (atom.cp && metric->desc.type == PM_TYPE_STRING) {
-	    if (result) free(result);
+	if (atom.cp && metric->desc.type == PM_TYPE_STRING)
 	    free(atom.cp);
-	}
+	if (result)
+	    free(result);
 	return -ENOMEM;
     }
     result->vset[0] = valueset;
@@ -2016,6 +2159,7 @@ pmWebGroupStore(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 
 done:
     settings->callbacks.on_done(id, sts, msg, arg);
+    webgroup_deref_context(cp);
     sdsfree(msg);
 }
 
@@ -2051,7 +2195,8 @@ pmWebGroupSetup(pmWebGroupModule *module)
     /* generally needed strings, error messages */
     EMPTYSTRING = sdsnew("");
     LOCALHOST = sdsnew("localhost");
-    TIMEOUT = sdsnew("pmwebapi.timeout");
+    WORK_TIMER = sdsnew("pmwebapi.work");
+    POLL_TIMEOUT = sdsnew("pmwebapi.timeout");
     BATCHSIZE = sdsnew("pmwebapi.batchsize");
     AUTH_USERNAME = sdsnew("auth.username");
     AUTH_PASSWORD = sdsnew("auth.password");
@@ -2063,6 +2208,7 @@ pmWebGroupSetup(pmWebGroupModule *module)
 
     /* setup a dictionary mapping context number to data */
     groups->contexts = dictCreate(&intKeyDictCallBacks, NULL);
+
     return 0;
 }
 
@@ -2085,12 +2231,20 @@ pmWebGroupSetConfiguration(pmWebGroupModule *module, dict *config)
     char		*endnum;
     sds			value;
 
-    if ((value = dictFetchValue(config, TIMEOUT)) == NULL) {
-	default_timeout = DEFAULT_TIMEOUT;
+    if ((value = dictFetchValue(config, WORK_TIMER)) == NULL) {
+	default_worker = DEFAULT_WORK_TIMER;
+    } else {
+	default_worker = strtoul(value, &endnum, 0);
+	if (*endnum != '\0')
+	    default_worker = DEFAULT_WORK_TIMER;
+    }
+
+    if ((value = dictFetchValue(config, POLL_TIMEOUT)) == NULL) {
+	default_timeout = DEFAULT_POLL_TIMEOUT;
     } else {
 	default_timeout = strtoul(value, &endnum, 0);
 	if (*endnum != '\0')
-	    default_timeout = DEFAULT_TIMEOUT;
+	    default_timeout = DEFAULT_POLL_TIMEOUT;
     }
 
     if ((value = dictFetchValue(config, BATCHSIZE)) == NULL) {
@@ -2129,9 +2283,13 @@ pmWebGroupClose(pmWebGroupModule *module)
 
     if (groups) {
 	/* walk the contexts, stop timers and free resources */
+	if (groups->active) {
+	    groups->active = 0;
+	    uv_timer_stop(&groups->timer);
+	}
 	iterator = dictGetIterator(groups->contexts);
 	while ((entry = dictNext(iterator)) != NULL)
-	    webgroup_destroy_context((context_t *)dictGetVal(entry), NULL);
+	    webgroup_drop_context((context_t *)dictGetVal(entry), NULL);
 	dictReleaseIterator(iterator);
 	dictRelease(groups->contexts);
 	memset(groups, 0, sizeof(struct webgroups));
@@ -2159,7 +2317,8 @@ pmWebGroupClose(pmWebGroupModule *module)
     /* generally needed strings, error messages */
     sdsfree(EMPTYSTRING);
     sdsfree(LOCALHOST);
-    sdsfree(TIMEOUT);
+    sdsfree(WORK_TIMER);
+    sdsfree(POLL_TIMEOUT);
     sdsfree(BATCHSIZE);
     sdsfree(AUTH_USERNAME);
     sdsfree(AUTH_PASSWORD);
