@@ -12,23 +12,21 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
- * 
  * check help files build by newhelp
  *
  * Usage:
- *    chkhelp helpfile metric-name ...
+ *    chkhelp helpfile metricname ...
  */
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include "pmda.h"
 
 #define VERSION 2
 static int	version = VERSION;
 
+static int	handle;
+
 /*
  * Note: these two are from libpcp_pmda/src/help.c
  */
@@ -54,7 +52,7 @@ next(int *ident, int *type)
     static int		nextidx;
     pmID		pmid;
     __pmID_int		*pi = (__pmID_int *)&pmid;
-    extern void		*__pmdaHelpTab(void);
+    PMDA_CALL extern void *__pmdaHelpTab(void);
 
     if (hp == NULL) {
 	hp = (help_t *)__pmdaHelpTab();
@@ -85,6 +83,38 @@ next(int *ident, int *type)
     return 1;
 }
 
+
+/*
+ * with -e come here for every metric in the PMNS ...
+ */
+void
+dometric(const char *name)
+{
+    int		sts;
+    pmID	pmid;
+    char	*tp;
+
+    sts = pmLookupName(1, &name, &pmid);
+    if (sts < 0) {
+	fprintf(stderr, "pmLookupName: failed for \"%s\": %s\n", name, pmErrStr(sts));
+	return;
+    }
+    if (sts == 0) {
+	fprintf(stderr, "pmLookupName: failed for \"%s\"\n", name);
+	return;
+    }
+
+    tp = pmdaGetHelp(handle, pmid, PM_TEXT_ONELINE);
+    if (tp != NULL)
+	return;
+    tp = pmdaGetHelp(handle, pmid, PM_TEXT_HELP);
+    if (tp != NULL)
+	return;
+
+    /* no help text, report metric */
+    printf("%s\n", name);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -95,36 +125,38 @@ main(int argc, char **argv)
     char	*pmnsfile = PM_NS_DEFAULT;
     int		errflag = 0;
     int		aflag = 0;
+    int		eflag = 0;
     int		allpmid = 0;
     int		allindom = 0;
-    char	*filename;
-    int		handle;
+    const char	*filename;
+    const char	*name;
     char	*tp;
-    char	*name;
     int		id;
     int		next_type;
     char	*endnum;
 
-    __pmSetProgname(argv[0]);
+    pmSetProgname(argv[0]);
 
-    while ((c = getopt(argc, argv, "D:Hin:Opv:?")) != EOF) {
+    while ((c = getopt(argc, argv, "D:eHin:Opv:?")) != EOF) {
 	switch (c) {
 
-	case 'D':	/* debug flag */
-	    sts = __pmParseDebug(optarg);
+	case 'D':	/* debug options */
+	    sts = pmSetDebug(optarg);
 	    if (sts < 0) {
-		fprintf(stderr, "%s: unrecognized debug flag specification (%s)\n",
-		    pmProgname, optarg);
+		fprintf(stderr, "%s: unrecognized debug options specification (%s)\n",
+		    pmGetProgname(), optarg);
 		errflag++;
 	    }
-	    else
-		pmDebug |= sts;
+	    break;
+
+	case 'e':	/* help text exists? */
+	    eflag = 1;
 	    break;
 
 	case 'H':	/* help text */
 	    help = 1;
 	    break;
-	
+
 	case 'i':
 	    aflag++;
 	    allindom = 1;
@@ -146,13 +178,13 @@ main(int argc, char **argv)
 	case 'v':	/* version 2 only these days */
 	    version = (int)strtol(optarg, &endnum, 10);
 	    if (*endnum != '\0') {
-		fprintf(stderr, "%s: -v requires numeric argument\n", pmProgname);
+		fprintf(stderr, "%s: -v requires numeric argument\n", pmGetProgname());
 		errflag++;
 	    }
 	    if (version != 2) {
 		fprintf(stderr 
                        ,"%s: deprecated option - only version 2 is supported\n"
-                       , pmProgname);
+                       , pmGetProgname());
 		errflag++;
 	    }
 	    break;
@@ -164,15 +196,26 @@ main(int argc, char **argv)
 	}
     }
 
-    if (aflag && optind != argc - 1) {
-	fprintf(stderr, "%s: metric-name arguments cannot be used with -i or -p\n\n",
-	    pmProgname);
+    if (optind == argc) {
+	fprintf(stderr, "%s: missing helpfile argument\n\n", pmGetProgname());
+	errflag = 1;
+    }
+
+    if (aflag && optind < argc-1) {
+	fprintf(stderr, "%s: metricname arguments cannot be used with -i or -p\n\n",
+	    pmGetProgname());
 	errflag = 1;
     }
 
     if (aflag == 0 && optind == argc-1 && oneline+help != 0) {
-	fprintf(stderr, "%s: -O or -H require metric-name arguments or -i or -p\n\n",
-	    pmProgname);
+	fprintf(stderr, "%s: -O or -H require metricname arguments or -i or -p\n\n",
+	    pmGetProgname());
+	errflag = 1;
+    }
+
+    if (eflag && (allpmid || allindom)) {
+	fprintf(stderr, "%s: -e cannot be used with -i or -p\n\n",
+	    pmGetProgname());
 	errflag = 1;
     }
 
@@ -182,6 +225,7 @@ main(int argc, char **argv)
 "       %s [options] helpfile [metricname ...]\n"
 "\n"
 "Options:\n"
+"  -e           exists check, only report metrics with no help text\n"
 "  -H           display verbose help text\n"
 "  -i           process all the instance domains\n"
 "  -n pmnsfile  use an alternative PMNS\n"
@@ -190,7 +234,7 @@ main(int argc, char **argv)
 "  -v version   deprecated (only version 2 format supported)\n"
 "\n"
 "No options implies silently check internal integrity of the helpfile.\n",
-		pmProgname, pmProgname);
+		pmGetProgname(), pmGetProgname());
 	exit(1);
     }
 
@@ -204,8 +248,8 @@ main(int argc, char **argv)
 	exit(1);
     }
 
-    if ((sts = pmLoadNameSpace(pmnsfile)) < 0) {
-	fprintf(stderr, "pmLoadNameSpace: %s\n", pmErrStr(sts));
+    if ((sts = pmLoadASCIINameSpace(pmnsfile, 1)) < 0) {
+	fprintf(stderr, "pmLoadASCIINameSpace(%s, 1): %s\n", pmnsfile, pmErrStr(sts));
 	exit(1);
     }
 
@@ -217,29 +261,45 @@ main(int argc, char **argv)
 	/* no metric names, process all entries */
 	aflag = 1;
 
+    if (eflag) {
+	if (optind == argc) {
+	    sts = pmTraversePMNS("", dometric);
+	    if (sts < 0) {
+		fprintf(stderr, "Error: pmTraversePMNS(\"\", ...): %s\n", pmErrStr(sts));
+	    }
+	}
+	else {
+	    for ( ; optind < argc; optind++) {
+		sts = pmTraversePMNS(argv[optind], dometric);
+		if (sts < 0)
+		    fprintf(stderr, "Error: pmTraversePMNS(\"%s\", ...): %s\n", argv[optind], pmErrStr(sts));
+	    }
+	}
+	exit(0);
+    }
+
     while (optind < argc || aflag) {
 	if (aflag) {
 	    if (next(&id, &next_type) == 0)
 		break;
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL0 && allindom+allpmid == 0)
+	    if (pmDebugOptions.appl0 && allindom+allpmid == 0)
 		fprintf(stderr, "next_type=%d id=0x%x\n", next_type, id);
-#endif
 	    if (next_type == 2) {
 		if (!allindom)
 		    continue;
 		printf("\nInDom %s:", pmInDomStr((pmInDom)id));
 	    }
 	    else {
-		char		*p;
+		char		**names;
 		if (!allpmid)
 		    continue;
 
 		printf("\nPMID %s", pmIDStr((pmID)id));
-		sts = pmNameID(id, &p);
-		if (sts == 0) {
-		    printf(" %s", p);
-		    free(p);
+		sts = pmNameAll(id, &names);
+		if (sts > 0) {
+		    printf(" ");
+		    __pmPrintMetricNames(stdout, sts, names, " or ");
+		    free(names);
 		}
 		putchar(':');
 	    }
@@ -247,16 +307,15 @@ main(int argc, char **argv)
 	else {
 	    next_type = 1;
 	    name = argv[optind++];
-	    printf("\nPMID");
 	    if ((sts = pmLookupName(1, &name, (pmID *)&id)) < 0) {
-		printf(" %s: %s\n", name, pmErrStr(sts));
+		printf("\n%s: %s\n", name, pmErrStr(sts));
 		continue;
 	    }
-	    printf(" %s %s:", pmIDStr((pmID)id), name);
 	    if (id == PM_ID_NULL) {
-		printf(" unknown metric\n");
+		printf("\n%s: unknown metric\n", name);
 		continue;
 	    }
+	    printf("\nPMID %s %s:", pmIDStr((pmID)id), name);
 	}
 
 	if (oneline) {
@@ -265,12 +324,9 @@ main(int argc, char **argv)
 	    else
 		tp = pmdaGetInDomHelp(handle, (pmInDom)id, PM_TEXT_ONELINE);
 	    if (tp != NULL)
-		printf(" %s\n", tp);
-	    else
-		putchar('\n');
-	}
-	else
+		printf(" %s", tp);
 	    putchar('\n');
+	}
 
 	if (help) {
 	    if (next_type == 1)
@@ -280,6 +336,7 @@ main(int argc, char **argv)
 	    if (tp != NULL && *tp)
 		printf("%s\n", tp);
 	}
+
     }
 
     return 0;
