@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2014 Red Hat.
  * Copyright (c) 1995-2001,2003 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -21,15 +22,10 @@
  * version 1 that is based on ndbm.
  */
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/param.h>
-#include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 
 #define DEFAULT_HELP_VERSION 2
 
@@ -37,12 +33,12 @@
 #define MAXLINE	128
 #define MAXENTRY 1024
 
-static int	verbose = 0;
-static int	ln = 0;
+static int	verbose;
+static int	ln;
 static char	*filename;
-static int	status = 0;
+static int	status;
 static int	version = DEFAULT_HELP_VERSION;
-static FILE	*f = NULL;
+static FILE	*f;
 
 typedef struct {
     pmID	pmid;
@@ -50,8 +46,8 @@ typedef struct {
     __uint32_t	off_text;
 } help_idx_t;
 
-static help_idx_t	*hindex = NULL;
-static int		numindex = 0;
+static help_idx_t	*hindex;
+static int		numindex;
 static int		thisindex = -1;
 
 static void
@@ -78,7 +74,7 @@ newentry(char *buf)
     *p = '\0';	/* terminate metric name */
     end_name = p;
 
-    if ((n = pmLookupName(1, &name, &pmid)) < 0) {
+    if ((n = pmLookupName(1, (const char **)&name, &pmid)) < 0) {
 	/* apparently not a metric name */
 	int	domain;
 	int	cluster;
@@ -107,7 +103,7 @@ newentry(char *buf)
 	}
 	else {
 	    fprintf(stderr, "%s: [%s:%d] %s: %s, entry abandoned\n",
-		    pmProgname, filename, ln, buf, pmErrStr(n));
+		    pmGetProgname(), filename, ln, buf, pmErrStr(n));
 	    status = 2;
 	    return;
 	}
@@ -115,7 +111,7 @@ newentry(char *buf)
     else {
 	if (pmid == PM_ID_NULL) {
 	    fprintf(stderr, "%s: [%s:%d] %s: unknown metric, entry abandoned\n",
-		    pmProgname, filename, ln, name);
+		    pmGetProgname(), filename, ln, name);
 	    status = 2;
 	    return;
 	}
@@ -125,7 +121,7 @@ newentry(char *buf)
 	if (hindex[thisindex].pmid == pmid) {
 	    __pmInDom_int	*kp = (__pmInDom_int *)&pmid;
 	    fprintf(stderr, "%s: [%s:%d] duplicate key (", 
-		    pmProgname, filename, ln);
+		    pmGetProgname(), filename, ln);
 	    if (kp->flag == 0)
 		fprintf(stderr, "%s", pmIDStr(pmid));
 	    else {
@@ -144,7 +140,7 @@ newentry(char *buf)
 	else
 	    numindex *= 2;
 	if ((hindex = (help_idx_t *)realloc(hindex, numindex * sizeof(hindex[0]))) == NULL) {
-	    __pmNoMem("newentry", numindex * sizeof(hindex[0]), PM_FATAL_ERR);
+	    pmNoMem("newentry", numindex * sizeof(hindex[0]), PM_FATAL_ERR);
 	}
     }
 
@@ -169,15 +165,14 @@ newentry(char *buf)
     
     if (p - start == 1 && verbose) {
 	fprintf(stderr, "%s: [%s:%d] %s: warning, null oneline\n",
-	    pmProgname, filename, ln, name);
+	    pmGetProgname(), filename, ln, name);
 	warn = 1;
 	if (!status) status = 1;
     }
 
-    fwrite(start, sizeof(*start), p - start, f);
-    if (ferror(f)) {
+    if (fwrite(start, sizeof(*start), p - start, f) != p - start || ferror(f)) {
 	fprintf(stderr, "%s: [%s:%d] %s: write oneline failed, entry abandoned\n",
-		pmProgname, filename, ln, name);
+		pmGetProgname(), filename, ln, name);
 	thisindex--;
 	status = 2;
 	return;
@@ -201,16 +196,15 @@ newentry(char *buf)
 
     if (i == 0 && verbose) {
 	fprintf(stderr, "%s: [%s:%d] %s: warning, null help\n",
-	    pmProgname, filename, ln, name);
+	    pmGetProgname(), filename, ln, name);
 	warn = 1;
 	if (!status) status = 1;
     }
 
-    fwrite(p, sizeof(*p), i+1, f);
-    if (ferror(f)) {
+    if (fwrite(p, sizeof(*p), i+1, f) != i+1 || ferror(f)) {
 	fprintf(stderr, 
 		"%s: [%s:%d] %s: write help failed, entry abandoned\n",
-		pmProgname, filename, ln, name);
+		pmGetProgname(), filename, ln, name);
 	thisindex--;
 	status = 2;
 	return;
@@ -247,15 +241,32 @@ idcomp(const void *a, const void *b)
 	return 1;
 }
 
+static pmLongOptions longopts[] = {
+    PMAPI_OPTIONS_HEADER("General options"),
+    PMOPT_DEBUG,
+    PMOPT_NAMESPACE,
+    PMOPT_HELP,
+    PMAPI_OPTIONS_HEADER("Output options"),
+    { "output", 1, 'o', "FILE", "base name for output files" },
+    { "verbose", 0, 'V', 0, "verbose/diagnostic output" },
+    { "version", 0, 'v', 0, "deprecated (only version 2 format supported)" },
+    PMAPI_OPTIONS_END
+};
+
+static pmOptions opts = {
+    .short_options = "D:n:o:Vv:?",
+    .long_options = longopts,
+    .short_usage = "[options] [file ...]",
+};
+
 int
 main(int argc, char **argv)
 {
     int		n;
     int		c;
     int		i;
-    int		sts;
+    int		skip;
     char	*pmnsfile = PM_NS_DEFAULT;
-    int		errflag = 0;
     char	*fname = NULL;
     char	pathname[MAXPATHLEN];
     FILE	*inf;
@@ -263,31 +274,25 @@ main(int argc, char **argv)
     char	*endnum;
     char	*bp;
     char	*p;
-    int		skip;
     help_idx_t	hdr;
 
-    __pmSetProgname(argv[0]);
-
-    while ((c = getopt(argc, argv, "D:n:o:Vv:?")) != EOF) {
+    while ((c = pmgetopt_r(argc, argv, &opts)) != EOF) {
 	switch (c) {
 
-	case 'D':	/* debug flag */
-	    sts = __pmParseDebug(optarg);
-	    if (sts < 0) {
-		fprintf(stderr, "%s: unrecognized debug flag specification (%s)\n",
-		    pmProgname, optarg);
-		errflag++;
+	case 'D':	/* debug options */
+	    if (pmSetDebug(opts.optarg) < 0) {
+		pmprintf("%s: unrecognized debug options specification (%s)\n",
+		    pmGetProgname(), opts.optarg);
+		opts.errors++;
 	    }
-	    else
-		pmDebug |= sts;
 	    break;
 
 	case 'n':	/* alternative namespace file */
-	    pmnsfile = optarg;
+	    pmnsfile = opts.optarg;
 	    break;
 
 	case 'o':	/* alternative output file name */
-	    fname = optarg;
+	    fname = opts.optarg;
 	    break;
 
 	case 'V':	/* more chit-chat */
@@ -295,49 +300,38 @@ main(int argc, char **argv)
 	    break;
 
 	case 'v':	/* version 2 only these days */
-	    version = (int)strtol(optarg, &endnum, 10);
+	    version = (int)strtol(opts.optarg, &endnum, 10);
 	    if (*endnum != '\0') {
-		fprintf(stderr, "%s: -v requires numeric argument\n",
-			pmProgname);
-		errflag++;
+		pmprintf("%s: -v requires numeric argument\n", pmGetProgname());
+		opts.errors++;
 	    }
 	    if (version != 2) {
-		fprintf(stderr 
-                       ,"%s: deprecated option - only version 2 is supported\n"
-                       , pmProgname);
-		errflag++;
+		pmprintf("%s: deprecated option - only version 2 is supported\n",
+			pmGetProgname());
+		opts.errors++;
 	    }
 	    break;
 
 	case '?':
 	default:
-	    errflag++;
+	    opts.errors++;
 	    break;
 	}
     }
 
-    if (errflag) {
-	fprintf(stderr,
-"Usage: %s [options] [file ...]\n\
-\n\
-Options:\n\
-  -n pmnsfile    use an alternative PMNS\n\
-  -o outputfile  base name for output files\n\
-  -V             verbose/diagnostic output\n\
-  -v version     deprecated (only version 2 format supported)\n",
-		pmProgname);
+    if (opts.errors) {
+	pmUsageMessage(&opts);
 	exit(2);
     }
 
-    if ((n = pmLoadNameSpace(pmnsfile)) < 0) {
-	fprintf(stderr, "%s: pmLoadNameSpace: %s\n",
-		pmProgname, pmErrStr(n));
+    if ((n = pmLoadASCIINameSpace(pmnsfile, 1)) < 0) {
+	fprintf(stderr, "%s: pmLoadASCIINameSpace(%s, 1): %s\n", pmGetProgname(), pmnsfile, pmErrStr(n));
 	exit(2);
     }
 
     do {
-	if (optind < argc) {
-	    filename = argv[optind];
+	if (opts.optind < argc) {
+	    filename = argv[opts.optind];
 	    if ((inf = fopen(filename, "r")) == NULL) {
 		perror(filename);
 		exit(2);
@@ -349,7 +343,7 @@ Options:\n\
 	    if (fname == NULL) {
 		fprintf(stderr, 
 			"%s: need either a -o option or a filename "
-			"argument to name the output file\n", pmProgname);
+			"argument to name the output file\n", pmGetProgname());
 		exit(2);
 	    }
 	    filename = "<stdin>";
@@ -357,10 +351,10 @@ Options:\n\
 	}
 
 	if (version == 2 && f == NULL) {
-	    sprintf(pathname, "%s.pag", fname);
+	    pmsprintf(pathname, sizeof(pathname), "%s.pag", fname);
 	    if ((f = fopen(pathname, "w")) == NULL) {
 		fprintf(stderr, "%s: fopen(\"%s\", ...) failed: %s\n",
-		    pmProgname, pathname, osstrerror());
+		    pmGetProgname(), pathname, osstrerror());
 		exit(2);
 	    }
 	    /* header: 2 => pag cf 1 => dir */
@@ -397,7 +391,7 @@ Options:\n\
 		if (bp[0] == '\0') {
 		    if (verbose)
 			fprintf(stderr, "%s: [%s:%d] null entry?\n", 
-				pmProgname, filename, ln);
+				pmGetProgname(), filename, ln);
 		    skip = 1;
 		    bp = buf;
 		    if (!status) status = 1;
@@ -418,7 +412,7 @@ Options:\n\
 		*p++ = '\n';
 		*p = '\0';
 		fprintf(stderr, "%s: [%s:%d] long line split after ...\n%s",
-			    pmProgname, filename, ln, buf);
+			    pmGetProgname(), filename, ln, buf);
 		ln--;
 		if (!status) status = 1;
 	    }
@@ -428,24 +422,24 @@ Options:\n\
 		bp[-1] = '\0';
 		bp[-2] = '\n';
 		fprintf(stderr, "%s: [%s:%d] entry truncated after ... %s",
-			    pmProgname, filename, ln, &bp[-64]);
+			    pmGetProgname(), filename, ln, &bp[-64]);
 		skip = 1;
 		if (!status) status = 1;
 	    }
 	}
 
 	fclose(inf);
-	optind++;
-    } while (optind < argc);
+	opts.optind++;
+    } while (opts.optind < argc);
 
     if (f != NULL) {
 	fclose(f);
 
 	/* do the directory index ... */
-	sprintf(pathname, "%s.dir", fname);
+	pmsprintf(pathname, sizeof(pathname), "%s.dir", fname);
 	if ((f = fopen(pathname, "w")) == NULL) {
 	    fprintf(stderr, "%s: fopen(\"%s\", ...) failed: %s\n",
-		pmProgname, pathname, osstrerror());
+		pmGetProgname(), pathname, osstrerror());
 	    exit(2);
 	}
 
@@ -454,20 +448,19 @@ Options:\n\
 	/* "1" => dir, next char is version */
 	hdr.off_oneline = 0x31000000 | (('0' + version) << 16);
 	hdr.off_text = thisindex + 1;	/* # entries */
-	fwrite(&hdr, sizeof(hdr), 1, f);
-	if (ferror(f)) {
+	if (fwrite(&hdr, sizeof(hdr), 1, f) != 1 || ferror(f)) {
 	     fprintf(stderr, "%s: fwrite index failed: %s\n",
-		     pmProgname, osstrerror());
+		     pmGetProgname(), osstrerror());
 	     exit(2);
 	}
 
 	/* sort and write index */
 	qsort((void *)hindex, thisindex+1, sizeof(hindex[0]), idcomp);
 	for (i = 0; i <= thisindex; i++) {
-	    fwrite(&hindex[i], sizeof(hindex[0]), 1, f);
-	    if (ferror(f)) {
+	    if (fwrite(&hindex[i], sizeof(hindex[0]), 1, f) != 1
+		|| ferror(f)) {
 		 fprintf(stderr, "%s: fwrite index failed: %s\n",
-			 pmProgname, osstrerror());
+			 pmGetProgname(), osstrerror());
 		 exit(2);
 	    }
 	}
