@@ -12,7 +12,7 @@
 ** visualized for the user.
 ** 
 ** Copyright (C) 2000-2018 Gerlof Langeveld
-** Copyright (C) 2015-2019 Red Hat.
+** Copyright (C) 2015-2020 Red Hat.
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -127,7 +127,10 @@
 ** declaration of global variables
 */
 struct sysname	sysname;
+int		localhost;
 int		nodenamelen;
+struct timeval	start;
+struct timeval	finish;
 struct timeval	origin;
 struct timeval	pretime;	/* timing info				*/
 struct timeval	curtime;	/* timing info				*/
@@ -164,10 +167,11 @@ char		**argvp;
 int		fetchmode;
 int		fetchstep;
 
+long long	system_boottime;
 
 struct visualize vis = {generic_samp, generic_error,
 			generic_end,  generic_usage,
-                       generic_prep, generic_next};
+                        generic_prep, generic_next};
 
 /*
 ** argument values
@@ -465,11 +469,6 @@ main(int argc, char *argv[])
 	*/
 	initifprop();
 
-	/*
- 	** open socket to the IP layer to issue getsockopt() calls later on
-	*/
-	netatop_ipopen();
-
 	/*
 	** start the engine now .....
 	*/
@@ -503,20 +502,21 @@ engine(void)
 	*/
 	static struct tstat	*curtpres;	/* current present list      */
 	static unsigned int	curtlen;	/* size of present list      */
-	struct tstat		*curpexit;	/* exited process list	     */
+	static struct tstat	*curpexit;	/* exited process list	     */
+	static unsigned int	curtexitlen;	/* size of exited proc list  */
 
 	static struct devtstat	devtstat;	/* deviation info	     */
 
 	unsigned long		ntaskpres;	/* number of tasks present   */
 	unsigned long		nprocexit;	/* number of exited procs    */
-	unsigned long		nprocexitnet;	/* number of exited procs    */
-						/* via netatopd daemon       */
 
 	unsigned long		noverflow;
 
 	unsigned int		nrgpuproc = 0;	/* number of GPU processes   */
 	struct gpupidstat	*gp = NULL;
 
+	char			lastcmd = '\0';
+
 	/*
 	** initialization: allocate required memory dynamically
 	*/
@@ -562,8 +562,6 @@ engine(void)
 	*/
 	for (sampcnt=0; sampcnt < nsamples; sampcnt++)
 	{
-		char	lastcmd;
-
 		/*
 		** wait for alarm-signal to arrive (except first sample)
 		** or wait for SIGUSR1/SIGUSR2
@@ -580,14 +578,15 @@ engine(void)
 		cursstat = presstat;
 		presstat = hlpsstat;
 
-		lastcmd = photosyst(cursstat);	/* obtain new counters     */
-		if (lastcmd == 'r')
+		if (photosyst(cursstat) == MRESET) /* obtain new counters */
 		{
 			/* reset to zero */
 			if ((*vis.next)() < 0)
 				cleanstop(1);
+			lastcmd = MRESET;
 			goto reset;
-               }
+		}
+		system_boottime = cursstat->cpu.btime;
 
 		/*
 		** take a snapshot of the current task-level statistics 
@@ -599,6 +598,10 @@ engine(void)
 		memset(curtpres, 0, curtlen * sizeof(struct tstat));
 		ntaskpres = photoproc(&curtpres, &curtlen);
 
+		memset(curpexit, 0, curtexitlen * sizeof(struct tstat));
+		nprocexit = acctphotoproc(&curpexit, &curtexitlen,
+					&cursstat->stamp, &interval);
+
 		/*
 		** register processes that exited during last sample;
 		** first determine how many processes exited
@@ -606,8 +609,6 @@ engine(void)
 		** the number of exited processes is limited to avoid
 		** that atop explodes in memory and introduces OOM killing
 		*/
-		nprocexit = acctprocnt();	/* number of exited processes */
-
 		if (nprocexit > MAXACCTPROCS)
 		{
 			noverflow = nprocexit - MAXACCTPROCS;
@@ -616,42 +617,6 @@ engine(void)
 		else
 			noverflow = 0;
 
-		/*
-		** determine how many processes have been exited
-		** for the netatop module (only processes that have
-		** used the network)
-		*/
-		if (nprocexit > 0 && (supportflags & NETATOPD))
-			nprocexitnet = netatop_exitstore();
-		else
-			nprocexitnet = 0;
-
-		/*
-		** reserve space for the exited processes and read them
-		*/
-		if (nprocexit > 0)
-		{
-			curpexit = calloc(nprocexit, sizeof(struct tstat));
-
-			ptrverify(curpexit,
-			          "Malloc failed for %lu exited processes\n",
-			          nprocexit);
-
-			nprocexit = acctphotoproc(curpexit, nprocexit);
-
-			/*
- 			** reposition offset in accounting file when not
-			** all exited processes have been read (i.e. skip
-			** those processes)
-			*/
-			if (noverflow)
-				acctrepos(noverflow);
-		}
-		else
-		{
-			curpexit    = NULL;
-		}
-
 		/*
 		** calculate the deviations (i.e. calculate the activity
 		** during the last sample).  Note for PMAPI calls we had
@@ -676,6 +641,14 @@ engine(void)
 		if (sampcnt==0)
 			sampflags |= RRBOOT;
 
+		if ((rawreadflag && (sampflags & RRBOOT)) ||
+		    (lastcmd == MSAMPPREV || lastcmd == MSAMPBRANCH))
+		{
+			pmtimevalInc(&curtime, &interval);
+			lastcmd = '\0';
+			goto reset;
+		}
+
 		/*
 		** activate the installed print-function to visualize
 		** the deviations
@@ -685,30 +658,25 @@ engine(void)
 		           	     &devtstat, devsstat,
 		                     nprocexit, noverflow, sampflags);
                 /*
-                 */
+		** prepare for next phase of sampling after visualize
+                */
                 (*vis.prep)();
 
-		if (rawreadflag)
-			pmtimevalInc(&curtime, &interval);
-
-		/*
-		** release dynamically allocated memory
-		*/
-		if (nprocexit > 0)
-			free(curpexit);
-
-		if (nprocexitnet > 0)
-			netatop_exiterase();
 
 		if (gp)
 			free(gp);
 
-reset:
-		if (lastcmd == 'r')	/* reset requested ? */
-		{
-			sampcnt = -1;
+		if (rawreadflag && (lastcmd == 0 || lastcmd == MSAMPNEXT))
+			pmtimevalInc(&curtime, &interval);
 
-			curtime = origin;
+reset:
+		if (lastcmd == MRESET || lastcmd == MSAMPBRANCH || lastcmd == MSAMPPREV)
+		{
+			if (lastcmd == MRESET)
+			{
+				sampcnt = -1;
+				curtime = origin;
+			}
 
 			/* set current (will be 'previous') counters to 0 */
 			sstat_reset(cursstat);
@@ -720,6 +688,12 @@ reset:
 
 		sampflags = 0;
 	} /* end of main-loop */
+
+	/*
+	** release dynamically allocated memory
+	*/
+	if (curtexitlen > 0)
+		free(curpexit);
 }
 
 /*
