@@ -1,7 +1,7 @@
 #define MAXINTNM	32
 struct ifprop	{
 	char		name[MAXINTNM];	/* name of interface  		*/
-	long int	speed;		/* in megabits per second	*/
+	count_t		speed;		/* in megabits per second	*/
 	char		fullduplex;	/* boolean			*/
 	char		type;		/* type: 'e' (ethernet)		*/
 					/*       'w' (wireless)         */
