@@ -8,7 +8,7 @@
 ** figures.
 **
 ** Copyright (C) 2000-2010 Gerlof Langeveld
-** Copyright (C) 2015-2018 Red Hat.
+** Copyright (C) 2015-2020 Red Hat.
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -27,7 +27,6 @@
 #else
 #include <curses.h>
 #endif
-#include <regex.h>
 #include <pwd.h>
 #include <grp.h>
 #include <stdarg.h>
@@ -53,10 +52,12 @@ static char	showtype  = MPROCGEN;
 static char	showorder = MSORTCPU;
 
 static int	maxcpulines = 999;  /* maximum cpu       lines          */
+static int	maxgpulines = 999;  /* maximum gpu       lines          */
 static int	maxdsklines = 999;  /* maximum disk      lines          */
 static int	maxmddlines = 999;  /* maximum MDD       lines          */
 static int	maxlvmlines = 999;  /* maximum LVM       lines          */
 static int	maxintlines = 999;  /* maximum interface lines          */
+static int	maxifblines = 999;  /* maximum infinibnd lines          */
 static int	maxnfslines = 999;  /* maximum nfs mount lines          */
 static int	maxcontlines = 999; /* maximum container lines          */
 
@@ -81,6 +82,7 @@ static int	(*procsort[])(const void *, const void *) = {
 			[MSORTMEM&0x1f]=compmem, 
 			[MSORTDSK&0x1f]=compdsk, 
 			[MSORTNET&0x1f]=compnet, 
+			[MSORTGPU&0x1f]=compgpu, 
 };
 
 extern proc_printpair ownprocs[];
@@ -94,7 +96,7 @@ int	startoffset;
 ** print the deviation-counters on process- and system-level
 */
 char
-generic_samp(double curtime, double nsecs,
+generic_samp(double sampletime, double nsecs,
            struct devtstat *devtstat, struct sstat *sstat,
            int nexit, unsigned int noverflow, int flag)
 {
@@ -102,14 +104,15 @@ generic_samp(double curtime, double nsecs,
 	char		*p;
 
 	register int	i, curline, statline, nproc;
-	int		firstproc = 0, plistsz, alistsz /*, killpid, killsig */;
+	int		firstproc = 0, plistsz, alistsz, killpid, killsig;
 	int		lastchar;
-	char		format1[16], format2[16];
+	char		format1[16], format2[16], branchtime[32];
 	char		*statmsg = NULL, statbuf[80], genline[80];
-	char		 *lastsortp, curorder, autoorder;
-	char		buf[33];
-	struct passwd 	*pwd;
+	char		*lastsortp, curorder, autoorder;
+	char		buf[34];
+	struct timeval	timeval;
 	struct syscap	syscap;
+	struct passwd 	*pwd;
 
 	/*
 	** curlist points to the active list of tstat-pointers that
@@ -197,6 +200,10 @@ generic_samp(double curtime, double nsecs,
 			qsort(sstat->cpu.cpu, sstat->cpu.nrcpu,
 	 	               sizeof sstat->cpu.cpu[0], cpucompar);
 
+		if (sstat->gpu.nrgpus > 1 && maxgpulines > 0)
+			qsort(sstat->gpu.gpu, sstat->gpu.nrgpus,
+	 	               sizeof sstat->gpu.gpu[0], gpucompar);
+
 		if (sstat->dsk.nlvm > 1 && maxlvmlines > 0)
 			qsort(sstat->dsk.lvm, sstat->dsk.nlvm,
 			       sizeof sstat->dsk.lvm[0], diskcompar);
@@ -244,8 +251,8 @@ generic_samp(double curtime, double nsecs,
         	/*
         	** print general headerlines
         	*/
-        	convdate(curtime, format1, sizeof(format1)-1); /* ascii date */
-        	convtime(curtime, format2, sizeof(format2)-1); /* ascii time */
+        	convdate(sampletime, format1, sizeof(format1)-1); /* ascii date */
+        	convtime(sampletime, format2, sizeof(format2)-1); /* ascii time */
 
 		if (screen)
 			attron(A_REVERSE);
@@ -308,10 +315,12 @@ generic_samp(double curtime, double nsecs,
 			autoorder = showorder;
 
 		curline = prisyst(sstat, curline, nsecs, avgval,
-		                  fixedhead, &syssel, &autoorder,
-		                  maxcpulines, maxdsklines, maxmddlines,
-		                  maxlvmlines, maxintlines, maxnfslines,
-		                  maxcontlines);
+					fixedhead, &syssel, &autoorder,
+					maxcpulines, maxgpulines,
+					maxdsklines, maxmddlines,
+		                        maxlvmlines, maxintlines,
+					maxifblines, maxnfslines,
+					maxcontlines);
 
 		/*
  		** if system-wide statistics do not fit,
@@ -327,11 +336,12 @@ generic_samp(double curtime, double nsecs,
 			move(curline, 0);
 
 			limitedlines();
-			
+
 			curline = prisyst(sstat, curline, nsecs, avgval,
 					fixedhead,  &syssel, &autoorder,
 					maxcpulines, maxdsklines, maxmddlines,
-		                        maxlvmlines, maxintlines, maxnfslines,
+					maxmddlines, maxlvmlines,
+		                        maxintlines, maxifblines, maxnfslines,
 		                        maxcontlines);
 
 			/*
@@ -438,7 +448,7 @@ generic_samp(double curtime, double nsecs,
 			** one entry per user (list has worst-case size)
 			*/
 			tucumlist = calloc(sizeof(struct tstat),    nproc);
-			ucumlist  = malloc(sizeof(struct tstat *) * nproc);
+			ucumlist  = calloc(sizeof(struct tstat *),  nproc);
 
 			ptrverify(tucumlist,
 			        "Malloc failed for %d ucum procs\n", nproc);
@@ -481,7 +491,7 @@ generic_samp(double curtime, double nsecs,
 			** one entry per program (list has worst-case size)
 			*/
 			tpcumlist = calloc(sizeof(struct tstat),    nproc);
-			pcumlist  = malloc(sizeof(struct tstat *) * nproc);
+			pcumlist  = calloc(sizeof(struct tstat *),  nproc);
 
 			ptrverify(tpcumlist,
 			        "Malloc failed for %d pcum procs\n", nproc);
@@ -524,7 +534,7 @@ generic_samp(double curtime, double nsecs,
 			** one entry per user (list has worst-case size)
 			*/
 			tccumlist = calloc(sizeof(struct tstat),    nproc);
-			ccumlist  = malloc(sizeof(struct tstat *) * nproc);
+			ccumlist  = calloc(sizeof(struct tstat *),  nproc);
 
 			ptrverify(tccumlist,
 			        "Malloc failed for %d ccum procs\n", nproc);
@@ -580,7 +590,7 @@ generic_samp(double curtime, double nsecs,
 			if (sellist)	// remove previous list if needed
 				free(sellist);
 
-			sellist = malloc(sizeof(struct tstat *) * ncurlist);
+			sellist = calloc(sizeof(struct tstat *), ncurlist);
 
 			ptrverify(sellist,
 			       "Malloc failed for %d select ptrs\n", ncurlist);
@@ -662,8 +672,8 @@ generic_samp(double curtime, double nsecs,
 					if (tsklist)
 						free(tsklist);	// remove current
 
-					tsklist = malloc(sizeof(struct tstat *)
-								    * ntotal);
+					tsklist = calloc(sizeof(struct tstat *),
+								    ntotal);
 
 					ptrverify(tsklist,
 				             "Malloc failed for %d taskptrs\n",
@@ -788,7 +798,7 @@ generic_samp(double curtime, double nsecs,
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
-				return lastchar;	
+				return '\0';	
 
 			   /*
 			   ** stop it
@@ -843,18 +853,18 @@ generic_samp(double curtime, double nsecs,
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
+				/*
+				** back up two steps, advancing again shortly
+				*/
+				pmtimevalDec(&curtime, &interval);
+				pmtimevalDec(&curtime, &interval);
+
 				return lastchar;
 
                            /*
 			   ** branch to certain time stamp
                            */
                            case MSAMPBRANCH:
-#if 1
-				statmsg = "Not yet supported in this atop!";
-				beep();
-				break;
-#else
-				char hhmm[16];
                                 if (!rawreadflag)
                                 {
                                         statmsg = "Only allowed when viewing "
@@ -869,22 +879,33 @@ generic_samp(double curtime, double nsecs,
                                 echo();
                                 move(statline, 0);
                                 clrtoeol();
-                                printw("Enter new time (format hh:mm): ");
+                                printw("Enter new time "
+				       "(format [YYYYMMDD]hh:mm:ss): ");
 
-                                hhmm[0] = '\0';
-                                scanw("%15s\n", hhmm);
+                                branchtime[0] = '\0';
+                                scanw("%31s\n", branchtime);
                                 noecho();
 
-                                if ( !hhmm2secs(hhmm, &begintime) )
+				timeval = curtime;
+
+                                if ( !getbranchtime(branchtime, &timeval) )
                                 {
                                         move(statline, 0);
                                         clrtoeol();
                                         statmsg = "Wrong time format!";
                                         beep();
-                                        begintime = 0;
                                         break;
                                 }
 
+				/*
+				** back up one step before the branch time,
+				** to advance onto it in the next sample
+				*/
+				curtime = timeval;
+				pmtimevalDec(&curtime, &interval);
+				if (time_less_than(&curtime, &start))
+				    curtime = start;
+
 				if (tpcumlist) free(tpcumlist);
 				if (pcumlist)  free(pcumlist);
 				if (tucumlist) free(tucumlist);
@@ -895,7 +916,6 @@ generic_samp(double curtime, double nsecs,
 				if (sellist)   free(sellist);
 
                                 return lastchar;
-#endif
 
 			   /*
 			   ** sort order automatically depending on
@@ -940,20 +960,29 @@ generic_samp(double curtime, double nsecs,
 			   ** sort in network-activity order
 			   */
 			   case MSORTNET:
-#if 1
-				statmsg = "Not yet supported in this atop!";
-				beep();
-#else
 				if ( !(supportflags & NETATOP) )
 				{
-					statmsg = "Kernel module 'netatop' not "
-					          "active or no root privs; "
+					statmsg = "BCC PMDA not active or "
+					          "'netproc' module not enabled; "
 					          "request ignored!";
 					break;
 				}
 				showorder = MSORTNET;
 				firstproc = 0;
-#endif
+				break;
+
+			   /*
+			   ** sort in gpu-activity order
+			   */
+			   case MSORTGPU:
+				if ( !(supportflags & GPUSTAT) )
+				{
+					statmsg = "No GPU activity figures "
+					          "available; request ignored!";
+					break;
+				}
+				showorder = MSORTGPU;
+				firstproc = 0;
 				break;
 
 			   /*
@@ -1003,14 +1032,10 @@ generic_samp(double curtime, double nsecs,
 			   ** network-specific figures per process
 			   */
 			   case MPROCNET:
-#if 1
-				statmsg = "Not yet supported in this atop!";
-				beep();
-#else
 				if ( !(supportflags & NETATOP) )
 				{
-					statmsg = "Kernel module 'netatop' not "
-					          "active or no root privs; "
+					statmsg = "BCC PMDA not active or "
+					          "'netproc' module not enabled; "
 					          "request ignored!";
 					break;
 				}
@@ -1021,7 +1046,27 @@ generic_samp(double curtime, double nsecs,
 					showorder = MSORTNET;
 
 				firstproc = 0;
-#endif
+				break;
+
+			   /*
+			   ** GPU-specific figures per process
+			   */
+			   case MPROCGPU:
+				if ( !(supportflags & GPUSTAT) )
+				{
+					statmsg = "No GPU activity figures "
+					          "available (pmdanvidia might "
+					          "not be running); "
+					          "request ignored!";
+					break;
+				}
+
+				showtype  = MPROCGPU;
+
+				if (showorder != MSORTAUTO)
+					showorder = MSORTGPU;
+
+				firstproc = 0;
 				break;
 
 			   /*
@@ -1126,17 +1171,20 @@ generic_samp(double curtime, double nsecs,
 			   ** send signal to process
 			   */
 			   case MKILLPROC:
-#if 1
-				statmsg = "Not supported in this atop!";
-				beep();
-#else
-				if (rawreadflag)	// TODO: or via remote pmcd, or...?
+				if (rawreadflag)
 				{
 					statmsg = "Not possible when viewing "
 					          "raw file!";
 					beep();
 					break;
 				}
+				if (!localhost)
+				{
+					statmsg = "Not possible when viewing "
+					          "values from a remote host!";
+					beep();
+					break;
+				}
 
 				setalarm2(0, 0);	/* stop the clock */
 
@@ -1172,22 +1220,12 @@ generic_samp(double curtime, double nsecs,
 					setalarm2(3, 0); /* set short timer */
 
 				firstproc = 0;
-#endif
 				break;
 
 			   /*
 			   ** change interval timeout
 			   */
 			   case MINTERVAL:
-				/* TODO: this is possible using PMAPI ... (pmSetMode); */
-				/* but, need different sampling vs reporting intervals */
-				if (rawreadflag)
-				{
-					statmsg = "Not yet possible when viewing "
-					          "raw file!";
-					beep();
-					break;
-				}
 
 				setalarm2(0, 0);	/* stop the clock */
 
@@ -1200,6 +1238,7 @@ generic_samp(double curtime, double nsecs,
 				{
 					if (!paused)
 						setalarm2(3, 0); /*  set short timer */
+					setup_step_mode(0);
 				}
 				else
 				{
@@ -1376,7 +1415,15 @@ generic_samp(double curtime, double nsecs,
 					break;
 
 				   case 12:	// container id
-					(void)strtol(procsel.container, &p, 16);
+					/*
+					 * Don't even ask ... we want to
+					 * scan the input buffer and are only
+					 * interested in a possible error ...
+					 * the (void)(...+1) babble is the
+					 * only way to silence new "smart" C
+					 * compilers!
+					 */
+					(void)(strtol(procsel.container, &p, 16)+1);
 
 					if (*p)
 					{
@@ -1786,6 +1833,11 @@ generic_samp(double curtime, double nsecs,
 				            "statistics (now %d): ",
 				            maxcpulines, statline);
 
+				maxgpulines =
+				  getnumval("Maximum lines for per-gpu "
+				            "statistics (now %d): ",
+				            maxgpulines, statline);
+
 				if (sstat->dsk.nlvm > 0)
 				{
 					maxlvmlines =
@@ -1812,6 +1864,11 @@ generic_samp(double curtime, double nsecs,
 				            "statistics (now %d): ",
 					    maxintlines, statline);
 
+				maxifblines =
+				  getnumval("Maximum lines for infiniband "
+				            "port statistics (now %d): ",
+					    maxifblines, statline);
+
 				maxnfslines =
 				  getnumval("Maximum lines for NFS mount "
 				            "statistics (now %d): ",
@@ -1968,6 +2025,9 @@ cumusers(struct tstat **curprocs, struct tstat *curusers, int numprocs)
 	*/
 	for (numusers=i=0; i < numprocs; i++, curprocs++)
 	{
+	        if (*curprocs == NULL)
+		        continue;
+		
 		if (procsuppress(*curprocs, &procsel))
 			continue;
 
@@ -2013,6 +2073,9 @@ cumprogs(struct tstat **curprocs, struct tstat *curprogs, int numprocs)
 	*/
 	for (numprogs=i=0; i < numprocs; i++, curprocs++)
 	{
+	        if (*curprocs == NULL)
+		        continue;
+		
 		if (procsuppress(*curprocs, &procsel))
 			continue;
 
@@ -2056,6 +2119,9 @@ cumconts(struct tstat **curprocs, struct tstat *curconts, int numprocs)
 	*/
 	for (numconts=i=0; i < numprocs; i++, curprocs++)
 	{
+	        if (*curprocs == NULL)
+		        continue;
+		
 		if (procsuppress(*curprocs, &procsel))
 			continue;
 
@@ -2139,6 +2205,33 @@ accumulate(struct tstat *curproc, struct tstat *curstat)
 		curstat->mem.vswap  += curproc->mem.vswap;
 		curstat->mem.rgrow  += curproc->mem.rgrow;
 		curstat->mem.vgrow  += curproc->mem.vgrow;
+
+		if (curproc->gpu.state)		// GPU is use?
+		{
+			int i;
+
+			curstat->gpu.state = 'A';
+
+			if (curproc->gpu.gpubusy == -1)
+				curstat->gpu.gpubusy  = -1;
+			else
+				curstat->gpu.gpubusy += curproc->gpu.gpubusy;
+
+			if (curproc->gpu.membusy == -1)
+				curstat->gpu.membusy  = -1;
+			else
+				curstat->gpu.membusy += curproc->gpu.membusy;
+
+			curstat->gpu.memnow  += curproc->gpu.memnow;
+			curstat->gpu.gpulist |= curproc->gpu.gpulist;
+			curstat->gpu.nrgpus   = 0;
+
+			for (i=0; i < hinv_nrgpus; i++)
+			{
+				if (curstat->gpu.gpulist & 1<<i)
+					curstat->gpu.nrgpus++;
+			}
+		}
 	}
 }
 
@@ -2241,13 +2334,32 @@ procsuppress(struct tstat *curstat, struct pselection *sel)
 static void
 limitedlines(void)
 {
-	maxcpulines  = 0;
-	maxdsklines  = 3;
-	maxmddlines  = 3;
-	maxlvmlines  = 4;
-	maxintlines  = 2;
-	maxnfslines  = 2;
-	maxcontlines = 0;
+	if (maxcpulines == 999)		// default?
+		maxcpulines  = 0;
+
+	if (maxgpulines == 999)		// default?
+		maxgpulines  = 2;
+
+	if (maxdsklines == 999)		// default?
+		maxdsklines  = 3;
+
+	if (maxmddlines == 999)		// default?
+		maxmddlines  = 3;
+
+	if (maxlvmlines == 999)		// default?
+		maxlvmlines  = 4;
+
+	if (maxintlines == 999)		// default?
+		maxintlines  = 2;
+
+	if (maxifblines == 999)		// default?
+		maxifblines  = 2;
+
+	if (maxnfslines == 999)		// default?
+		maxnfslines  = 2;
+
+	if (maxcontlines == 999)	// default?
+		maxcontlines = 1;
 }
 
 /*
@@ -2306,7 +2418,7 @@ printg(const char *format, ...)
 	va_start(args, format);
 
 	if (screen)
-		vwprintw(stdscr, (char *) format, args);
+		vw_printw(stdscr, (char *) format, args);
 	else
 		vprintf(format, args);
 
@@ -2337,6 +2449,10 @@ generic_init(void)
 			showorder = MSORTCPU;
 			break;
 
+		   case MSORTGPU:
+			showorder = MSORTGPU;
+			break;
+
 		   case MSORTMEM:
 			showorder = MSORTMEM;
 			break;
@@ -2354,6 +2470,11 @@ generic_init(void)
 			showorder = MSORTCPU;
 			break;
 
+		   case MPROCGPU:
+			showtype  = MPROCGPU;
+			showorder = MSORTGPU;
+			break;
+
 		   case MPROCMEM:
 			showtype  = MPROCMEM;
 			showorder = MSORTMEM;
@@ -2381,8 +2502,9 @@ generic_init(void)
 		   case MPROCNET:
 			if ( !(supportflags & NETATOP) )
 			{
-				fprintf(stderr, "Kernel module 'netatop' not "
-					          "active; request ignored!");
+				fprintf(stderr, "BCC PMDA not active or "
+					        "'netproc' module not enabled; "
+					        "request ignored!");
 				sleep(3);
 				break;
 			}
@@ -2544,6 +2666,7 @@ static struct helptext {
 	{"\t'%c'  - memory details\n",				MPROCMEM},
 	{"\t'%c'  - disk details\n",				MPROCDSK},
 	{"\t'%c'  - network details\n",				MPROCNET},
+	{"\t'%c'  - GPU details\n",				MPROCGPU},
 	{"\t'%c'  - scheduling and thread-group info\n",	MPROCSCH},
 	{"\t'%c'  - various info (ppid, user/group, date/time, status, "
 	 "exitcode)\n",	MPROCVAR},
@@ -2555,6 +2678,7 @@ static struct helptext {
 	{"\t'%c'  - memory consumption\n",			MSORTMEM},
 	{"\t'%c'  - disk activity\n",				MSORTDSK},
 	{"\t'%c'  - network activity\n",			MSORTNET},
+	{"\t'%c'  - GPU activity\n",				MSORTGPU},
 	{"\t'%c'  - most active system resource (auto mode)\n",	MSORTAUTO},
 	{"\n",							' '},
 	{"Accumulated figures:\n",				' '},
@@ -2710,6 +2834,17 @@ generic_end(void)
 	endwin();
 }
 
+void
+generic_prep(void)
+{
+}
+
+int
+generic_next(void)
+{
+    return -1;
+}
+
 /*
 ** function to be called when usage-info is required
 */
@@ -2752,15 +2887,17 @@ generic_usage(void)
 			MCUMPROC);
 	printf("\t  -%c  show cumulated process-info per container\n\n",
 			MCUMCONT);
-	printf("\t  -%c  sort processes in order of cpu-consumption "
+	printf("\t  -%c  sort processes in order of cpu consumption "
 	                "(default)\n",
 			MSORTCPU);
-	printf("\t  -%c  sort processes in order of memory-consumption\n",
+	printf("\t  -%c  sort processes in order of memory consumption\n",
 			MSORTMEM);
-	printf("\t  -%c  sort processes in order of disk-activity\n",
+	printf("\t  -%c  sort processes in order of disk activity\n",
 			MSORTDSK);
-	printf("\t  -%c  sort processes in order of network-activity\n",
+	printf("\t  -%c  sort processes in order of network activity\n",
 			MSORTNET);
+	printf("\t  -%c  sort processes in order of GPU activity\n",
+			MSORTGPU);
 	printf("\t  -%c  sort processes in order of most active resource "
                         "(auto mode)\n",
 			MSORTAUTO);
@@ -2857,6 +2994,12 @@ do_maxcpu(char *name, char *val)
 	maxcpulines = get_posval(name, val);
 }
 
+void
+do_maxgpu(char *name, char *val)
+{
+	maxgpulines = get_posval(name, val);
+}
+
 void
 do_maxdisk(char *name, char *val)
 {
@@ -2881,6 +3024,12 @@ do_maxintf(char *name, char *val)
 	maxintlines = get_posval(name, val);
 }
 
+void
+do_maxifb(char *name, char *val)
+{
+	maxifblines = get_posval(name, val);
+}
+
 void
 do_maxnfsm(char *name, char *val)
 {
@@ -2969,6 +3118,10 @@ do_flags(char *name, char *val)
 			showorder = MSORTCPU;
 			break;
 
+		   case MSORTGPU:
+			showorder = MSORTGPU;
+			break;
+
 		   case MSORTMEM:
 			showorder = MSORTMEM;
 			break;
@@ -2990,6 +3143,11 @@ do_flags(char *name, char *val)
 			showorder = MSORTCPU;
 			break;
 
+		   case MPROCGPU:
+			showtype  = MPROCGPU;
+			showorder = MSORTGPU;
+			break;
+
 		   case MPROCMEM:
 			showtype  = MPROCMEM;
 			showorder = MSORTMEM;
