@@ -1,6 +1,6 @@
 #!/usr/bin/env pmpython
 #
-# Copyright (C) 2018-2020 Red Hat.
+# Copyright (C) 2018-2021 Red Hat.
 # Copyright (C) 2004-2016 Dag Wieers <dag@wieers.com>
 #
 # This program is free software; you can redistribute it and/or modify it
@@ -30,7 +30,8 @@ import termios, struct, atexit, fcntl, sched, errno, time, re, sys, os
 from pcp import pmapi, pmconfig
 from cpmapi import PM_CONTEXT_ARCHIVE, PM_CONTEXT_HOST, PM_CONTEXT_LOCAL
 from cpmapi import PM_TYPE_32, PM_TYPE_U32, PM_TYPE_64, PM_TYPE_U64
-from cpmapi import PM_TYPE_DOUBLE, PM_TYPE_FLOAT
+from cpmapi import PM_TYPE_DOUBLE, PM_TYPE_FLOAT, PM_TIME_MIN, PM_TIME_HOUR
+from cpmapi import PM_TIME_NSEC, PM_TIME_USEC, PM_TIME_MSEC
 from cpmapi import PM_ERR_EOL, PM_IN_NULL, pmUsageMessage
 
 if sys.version >= '3':
@@ -556,7 +557,7 @@ class DstatTool(object):
             pidfile.write(str(os.getpid()))
             pidfile.close()
         except Exception as e:
-            sys.stderr.write('Failed to create pidfile %s\n' % self.pidfile, e)
+            sys.stderr.write('Failed to create pidfile %s\n%s\n' % (self.pidfile, e))
             self.pidfile = False
 
     def create_time_plugins(self):
@@ -704,7 +705,7 @@ class DstatTool(object):
                 sys.exit(1)
         if len(operands) > 1:
             try:
-                self.samples = int(operands[1])
+                self.samples = int(operands[1]) + 1
             except:
                 sys.stderr.write("Invalid sample count '%s'\n" % operands[1])
                 sys.exit(1)
@@ -1265,8 +1266,6 @@ class DstatTool(object):
 
     @staticmethod
     def instance_match(inst, plugin):
-        if plugin.cullinsts is not None and re.match(plugin.cullinsts, inst):
-            return False
         if plugin.instances and inst in plugin.instances:
             return True
         return plugin.grouptype == 1
@@ -1333,6 +1332,8 @@ class DstatTool(object):
                 label = metric[0]
 
                 for instid, instname, _ in result:
+                    if plugin.cullinsts is not None and re.match(plugin.cullinsts, instname):
+                        continue
                     key = self.mgetkey(label, instid)
                     valueset = self.mlookup(valuesets, key)
                     totals[i] += self.maverage(valueset, key)
@@ -1388,6 +1389,8 @@ class DstatTool(object):
                 values = 0
                 result = results[name]
                 for _, instname, val in result:
+                    if plugin.cullinsts is not None and re.match(plugin.cullinsts, instname):
+                        continue
                     totals[i] += val
                     values += 1
             if values == 0:
@@ -1402,6 +1405,30 @@ class DstatTool(object):
         #sys.stderr.write("gshowcsv result line:\n%s%s\n" % (line, THEME['default']))
         return line
 
+    @staticmethod
+    def scale_time(value, scale):
+        """ convert to canonical time units of seconds """
+        div = 1.0
+        mul = 1.0
+        if scale == PM_TIME_NSEC:
+            div = 1000000000.0
+        elif scale == PM_TIME_USEC:
+            div = 1000000.0
+        elif scale == PM_TIME_MSEC:
+            div = 1000.0
+        elif scale == PM_TIME_MIN:
+            mul = 60.0
+        elif scale == PM_TIME_HOUR:
+            mul = 3600.0
+        return (float(value) / div) * mul
+
+    @staticmethod
+    def scale_space(value, scale):
+        """ convert to canonical space units of bytes """
+        if scale == 0:
+            return value
+        return value * pow(1024, scale)
+
     def cprintlist(self, values, units, prtype, pmtype, width, colorstep):
         """Return all columns color printed"""
         ret = sep = ''
@@ -1431,12 +1458,12 @@ class DstatTool(object):
             else:
                 printtype = 's'
         base = 1000
-        if units.dimTime:
-            value *= base * units.scaleTime
-        if units.dimSpace and units.scaleSpace:
+        if units.dimTime and printtype != 's':
+            value = self.scale_time(value, units.scaleTime)
+        if units.dimSpace and printtype != 's':
             base = 1024
-            value *= base * units.scaleSpace
-        if units.dimCount and units.scaleCount:
+            value = self.scale_space(value, units.scaleSpace)
+        if units.dimCount and units.scaleCount and printtype != 's':
             value *= units.scaleCount
 
         ### Display units when base is exact 1000 or 1024
@@ -1686,7 +1713,7 @@ class DstatTool(object):
                 else:
                     sep = CHAR['sep']
                 if plugin in self.timelist:
-                    oline = oline + sep + self.tshow(plugin, self.pmfg_ts)
+                    oline = oline + sep + self.tshowcsv(plugin, self.pmfg_ts)
                 elif plugin.grouptype is None:
                     for m, name in enumerate(plugin.mgroup):
                         oline = oline + sep + self.mshowcsv(plugin, m, results[name])
