@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014,2017-2018 Red Hat.
+ * Copyright (c) 2012-2014,2017-2020 Red Hat.
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -65,10 +65,12 @@ DoText(ClientInfo *cp, __pmPDU* pb)
     if ((sts = __pmDecodeTextReq(pb, &ident, &type)) < 0)
 	return sts;
 
-    if ((ap = FindDomainAgent(((__pmID_int *)&ident)->domain)) == NULL)
+    if ((ap = pmcd_agent(((__pmID_int *)&ident)->domain)) == NULL)
 	return PM_ERR_PMID;
-    else if (!ap->status.connected)
+    if (!ap->status.connected)
 	return PM_ERR_NOAGENT;
+    if (ap->status.fenced)
+	return PM_ERR_PMDAFENCED;
 
     if (ap->ipcType == AGENT_DSO) {
 	if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
@@ -177,10 +179,12 @@ DoDesc(ClientInfo *cp, __pmPDU *pb)
     if ((sts = __pmDecodeDescReq(pb, &pmid)) < 0)
 	return sts;
 
-    if ((ap = FindDomainAgent(((__pmID_int *)&pmid)->domain)) == NULL)
+    if ((ap = pmcd_agent(((__pmID_int *)&pmid)->domain)) == NULL)
 	return PM_ERR_PMID;
-    else if (!ap->status.connected)
+    if (!ap->status.connected)
 	return PM_ERR_NOAGENT;
+    if (ap->status.fenced)
+	return PM_ERR_PMDAFENCED;
 
     if (ap->ipcType == AGENT_DSO) {
 	if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
@@ -258,14 +262,18 @@ DoInstance(ClientInfo *cp, __pmPDU *pb)
 	if (name != NULL) free(name);
 	return PM_ERR_IPC;
     }
-    if ((ap = FindDomainAgent(((__pmInDom_int *)&indom)->domain)) == NULL) {
+    if ((ap = pmcd_agent(((__pmInDom_int *)&indom)->domain)) == NULL) {
 	if (name != NULL) free(name);
 	return PM_ERR_INDOM;
     }
-    else if (!ap->status.connected) {
+    if (!ap->status.connected) {
 	if (name != NULL) free(name);
 	return PM_ERR_NOAGENT;
     }
+    if (ap->status.fenced) {
+	if (name != NULL) free(name);
+	return PM_ERR_PMDAFENCED;
+    }
 
     if (ap->ipcType == AGENT_DSO) {
 	if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
@@ -371,46 +379,17 @@ GetContextLabels(ClientInfo *cp, pmLabelSet **sets)
     const char		*userid;
     const char		*groupid;
     const char		*container;
-    static const char	func[] = "GetContextLabels";
     static char		host[MAXHOSTNAMELEN];
     static char		domain[MAXDOMAINNAMELEN];
     static char		machineid[MAXMACHINEIDLEN];
+    static const char	*host_label, *domain_label, *machineid_label;
     char		buf[PM_MAXLABELJSONLEN];
-    char		*hostname;
-    int			sts;
+    int			sts, flags;
 
     if ((sts = GetChangedContextLabels(sets, &labelChanged)) >= 0) {
-	if ((hostname = pmcd_hostname) == NULL) {
-	    if ((sts = gethostname(host, MAXHOSTNAMELEN)) < 0) {
-		if (pmDebugOptions.labels)
-		    fprintf(stderr, "%s: gethostname() -> %d (%s)\n",
-			    func, sts, pmErrStr(sts));
-		host[0] = '\0';
-	    }
-	    if (host[0] == '\0')
-		pmsprintf(host, sizeof(host), "localhost");
-	    hostname = pmcd_hostname = host;
-	}
-	if (domain[0] == '\0') {
-	    if ((sts = getdomainname(domain, MAXDOMAINNAMELEN)) < 0) {
-		if (pmDebugOptions.labels)
-		    fprintf(stderr, "%s: getdomainname() -> %d (%s)\n",
-			    func, sts, pmErrStr(sts));
-		domain[0] = '\0';
-	    }
-	    if (domain[0] == '\0' || strcmp(domain, "(none)") == 0)
-		pmsprintf(domain, sizeof(domain), "localdomain");
-	}
-	if (machineid[0] == '\0') {
-	    if ((sts = getmachineid(machineid, MAXMACHINEIDLEN)) < 0) {
-		if (pmDebugOptions.labels)
-		    fprintf(stderr, "%s: getmachineid() -> %d (%s)\n",
-			    func, sts, pmErrStr(sts));
-		machineid[0] = '\0';
-	    }
-	    if (machineid[0] == '\0')
-		pmsprintf(machineid, sizeof(machineid), "localmachine");
-	}
+	host_label = __pmGetLabelConfigHostName(host, sizeof(host));
+	domain_label = __pmGetLabelConfigDomainName(domain, sizeof(domain));
+	machineid_label = __pmGetLabelConfigMachineID(machineid, sizeof(machineid));
 	userid = ((node = __pmHashSearch(PCP_ATTR_USERID, &cp->attrs)) ?
 			(const char *)node->data : NULL);
 	groupid = ((node = __pmHashSearch(PCP_ATTR_GROUPID, &cp->attrs)) ?
@@ -418,25 +397,28 @@ GetContextLabels(ClientInfo *cp, pmLabelSet **sets)
 	container = ((node = __pmHashSearch(PCP_ATTR_CONTAINER, &cp->attrs)) ?
 			(const char *)node->data : NULL);
 
-	sts = pmsprintf(buf, sizeof(buf), "{\"hostname\":\"%s\"", hostname);
-	if (domain[0] != '\0')
-	    sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"domainname\":\"%s\"",
-			    domain);
-	if (machineid[0] != '\0')
-	    sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"machineid\":\"%s\"",
-			    machineid);
-	if (userid)
-	    sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"userid\":%s",
-			    userid);
-	if (groupid)
-	    sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"groupid\":%s",
-			    groupid);
+	sts = pmsprintf(buf, sizeof(buf), "{\"%s\":\"%s\"",
+			host_label, host);
 	if (container)
-	    sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"container\":%s",
+	    sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"container\":\"%s\"",
 			    container);
-	pmsprintf(buf+sts, sizeof(buf)-sts, "}");
-	if ((sts = __pmAddLabels(sets, buf, PM_LABEL_CONTEXT)) > 0)
-	    return 1;
+	sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"%s\":\"%s\"",
+			 domain_label, domain);
+	sts += pmsprintf(buf+sts, sizeof(buf)-sts, ",\"%s\":\"%s\"}",
+			 machineid_label, machineid);
+
+	flags = PM_LABEL_CONTEXT;
+	if ((sts = __pmAddLabels(sets, buf, flags)) <= 0)
+	    return sts;
+
+	flags |= PM_LABEL_OPTIONAL;
+	if (userid && groupid) {
+	    pmsprintf(buf, sizeof(buf), "{\"groupid\":%s,\"userid\":%s}",
+					groupid, userid);
+	    if ((sts = __pmAddLabels(sets, buf, flags)) <= 0)
+		return sts;
+	}
+	return 1;
     }
     return sts;
 }
@@ -444,6 +426,10 @@ GetContextLabels(ClientInfo *cp, pmLabelSet **sets)
 int
 DoLabel(ClientInfo *cp, __pmPDU *pb)
 {
+    __pmHashCtl		*hcp;
+    __pmHashNode	*hp;
+    pmProfile		*profile, defprofile = {PM_PROFILE_INCLUDE, 0, NULL};
+    int			ctxnum;
     int			sts, s;
     int			ident, type, nsets = 0;
     pmLabelSet		*sets = NULL;
@@ -459,17 +445,17 @@ DoLabel(ClientInfo *cp, __pmPDU *pb)
 	    nsets = sts = GetContextLabels(cp, &sets);
 	    goto response;
 	case PM_LABEL_DOMAIN:
-	    if (!(ap = FindDomainAgent(ident)))
+	    if (!(ap = pmcd_agent(ident)))
 		return PM_ERR_NOAGENT;
 	    break;
 	case PM_LABEL_INDOM:
-	    if (!(ap = FindDomainAgent(((__pmInDom_int *)&ident)->domain)))
+	case PM_LABEL_INSTANCES:
+	    if (!(ap = pmcd_agent(((__pmInDom_int *)&ident)->domain)))
 		return PM_ERR_INDOM;
 	    break;
 	case PM_LABEL_CLUSTER:
 	case PM_LABEL_ITEM:
-	case PM_LABEL_INSTANCES:
-	    if (!(ap = FindDomainAgent(((__pmID_int *)&ident)->domain)))
+	    if (!(ap = pmcd_agent(((__pmID_int *)&ident)->domain)))
 		return PM_ERR_PMID;
 	    break;
 	default:
@@ -478,6 +464,8 @@ DoLabel(ClientInfo *cp, __pmPDU *pb)
 
     if (!ap->status.connected)
 	return PM_ERR_NOAGENT;
+    if (ap->status.fenced)
+	return PM_ERR_PMDAFENCED;
 
     if (ap->ipcType == AGENT_DSO) {
 	if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
@@ -499,6 +487,42 @@ DoLabel(ClientInfo *cp, __pmPDU *pb)
 	if (ap->status.notReady)
 	    return PM_ERR_AGAIN;
 
+	/* status.madeDsoResult is only used for DSO agents so don't waste time by
+	 * checking that the agent is a DSO first.
+	 */
+	ap->status.madeDsoResult = 0;
+	ctxnum = cp - client;
+
+	if (ap->profClient != cp || ctxnum != ap->profIndex) {
+	    hcp = &cp->profile;
+	    hp = __pmHashSearch(ctxnum, hcp);
+	    if (hp != NULL)
+		profile = (pmProfile *)hp->data;
+	    else
+		profile = &defprofile;
+	    if (ap->ipcType == AGENT_DSO) {
+		if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
+		    ap->ipc.dso.dispatch.version.four.ext->e_context = cp - client;
+		sts = ap->ipc.dso.dispatch.version.any.profile(profile,
+					 ap->ipc.dso.dispatch.version.any.ext);
+	    }
+	    else {
+		if (ap->status.notReady == 0) {
+		    pmcd_trace(TR_XMIT_PDU, ap->inFd, PDU_PROFILE, ctxnum);
+		    if (profile && (sts = __pmSendProfile(ap->inFd, cp - client,
+					       ctxnum, profile)) < 0) {
+			pmcd_trace(TR_XMIT_ERR, ap->inFd, PDU_PROFILE, sts);
+		    }
+		} else {
+		    sts = PM_ERR_AGAIN;
+		}
+	    }
+	    if (sts >= 0) {
+		ap->profClient = cp;
+		ap->profIndex = ctxnum;
+	    }
+	}
+
 	pmcd_trace(TR_XMIT_PDU, ap->inFd, PDU_LABEL_REQ, ident);
 	sts = __pmSendLabelReq(ap->inFd, cp - client, ident, type);
 	if (sts >= 0) {
@@ -577,7 +601,7 @@ DoPMNSIDs(ClientInfo *cp, __pmPDU *pb)
 	 * failure may be a real failure, or could be a metric within a
 	 * dynamic sutree of the PMNS
 	 */
-	if ((ap = FindDomainAgent(((__pmID_int *)&idlist[0])->domain)) == NULL) {
+	if ((ap = pmcd_agent(((__pmID_int *)&idlist[0])->domain)) == NULL) {
 	    sts = PM_ERR_NOAGENT;
 	    goto fail;
 	}
@@ -585,6 +609,10 @@ DoPMNSIDs(ClientInfo *cp, __pmPDU *pb)
 	    sts = PM_ERR_NOAGENT;
 	    goto fail;
 	}
+	if (ap->status.fenced) {
+	    sts = PM_ERR_PMDAFENCED;
+	    goto fail;
+	}
 	if (ap->ipcType == AGENT_DSO) {
 	    if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
 		ap->ipc.dso.dispatch.version.four.ext->e_context = cp - client;
@@ -635,7 +663,7 @@ DoPMNSIDs(ClientInfo *cp, __pmPDU *pb)
     numnames = sts;
 
     pmcd_trace(TR_XMIT_PDU, cp->fd, PDU_PMNS_NAMES, numnames);
-    if ((sts = __pmSendNameList(cp->fd, FROM_ANON, numnames, namelist, NULL)) < 0){
+    if ((sts = __pmSendNameList(cp->fd, FROM_ANON, numnames, (const char **)namelist, NULL)) < 0){
 	pmcd_trace(TR_XMIT_ERR, cp->fd, PDU_PMNS_NAMES, sts);
 	CleanupClient(cp, sts);
     	goto fail;
@@ -675,7 +703,7 @@ DoPMNSNames(ClientInfo *cp, __pmPDU *pb)
 	goto done;
     }
 
-    sts = pmLookupName(numids, namelist, idlist);
+    sts = pmLookupName(numids, (const char **)namelist, idlist);
     /*
      * even if this fails, or looks up fewer than numids, we have to
      * check each PMID looking for dynamic metrics and process them
@@ -692,12 +720,15 @@ DoPMNSNames(ClientInfo *cp, __pmPDU *pb)
 	 * an error via lsts
 	 */
 	idlist[i] = PM_ID_NULL;	/* default case if cannot translate */
-	if ((ap = FindDomainAgent(domain)) == NULL) {
+	if ((ap = pmcd_agent(domain)) == NULL) {
 	    lsts = PM_ERR_NOAGENT;
 	}
 	else if (!ap->status.connected) {
 	    lsts = PM_ERR_NOAGENT;
 	}
+	else if (ap->status.fenced) {
+	    lsts = PM_ERR_PMDAFENCED;
+	}
 	else {
 	    if (ap->ipcType == AGENT_DSO) {
 		if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
@@ -718,7 +749,7 @@ DoPMNSNames(ClientInfo *cp, __pmPDU *pb)
 		    lsts = PM_ERR_AGAIN;
 		else {
 		    pmcd_trace(TR_XMIT_PDU, ap->inFd, PDU_PMNS_NAMES, 1);
-		    lsts = __pmSendNameList(ap->inFd, cp - client, 1, &namelist[i], NULL);
+		    lsts = __pmSendNameList(ap->inFd, cp - client, 1, (const char **)&namelist[i], NULL);
 		    if (lsts >= 0) {
 			int		pinpdu;
 			pinpdu = lsts = __pmGetPDU(ap->outFd, ANY_SIZE, pmcd_timeout, &pb);
@@ -809,11 +840,11 @@ DoPMNSChild(ClientInfo *cp, __pmPDU *pb)
 	goto done;
 
     namelist[0] = name;
-    sts = pmLookupName(1, namelist, idlist);
+    sts = pmLookupName(1, (const char **)namelist, idlist);
     if (sts == 1 && IS_DYNAMIC_ROOT(idlist[0])) {
 	int		domain = pmID_cluster(idlist[0]);
 	AgentInfo	*ap = NULL;
-	if ((ap = FindDomainAgent(domain)) == NULL) {
+	if ((ap = pmcd_agent(domain)) == NULL) {
 	    sts = PM_ERR_NOAGENT;
 	    goto done;
 	}
@@ -821,6 +852,10 @@ DoPMNSChild(ClientInfo *cp, __pmPDU *pb)
 	    sts = PM_ERR_NOAGENT;
 	    goto done;
 	}
+	if (ap->status.fenced) {
+	    sts = PM_ERR_PMDAFENCED;
+	    goto done;
+	}
 	if (ap->ipcType == AGENT_DSO) {
 	    if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
 		ap->ipc.dso.dispatch.version.four.ext->e_context = cp - client;
@@ -901,7 +936,7 @@ DoPMNSChild(ClientInfo *cp, __pmPDU *pb)
 
     numnames = sts;
     pmcd_trace(TR_XMIT_PDU, cp->fd, PDU_PMNS_NAMES, numnames);
-    if ((sts = __pmSendNameList(cp->fd, FROM_ANON, numnames, offspring, statuslist)) < 0) {
+    if ((sts = __pmSendNameList(cp->fd, FROM_ANON, numnames, (const char **)offspring, statuslist)) < 0) {
 	pmcd_trace(TR_XMIT_ERR, cp->fd, PDU_PMNS_NAMES, sts);
 	CleanupClient(cp, sts);
     }
@@ -984,16 +1019,18 @@ traverse_dynamic(ClientInfo *cp, char *start, int *num_names, char ***names)
     for (i = *num_names-1; i >= 0; i--) {
 	offspring = NULL;
 	namelist[0] = (*names)[i];
-	sts = pmLookupName(1, namelist, idlist);
+	sts = pmLookupName(1, (const char **)namelist, idlist);
 	if (sts < 1)
 	    continue;
 	if (IS_DYNAMIC_ROOT(idlist[0])) {
 	    int		domain = pmID_cluster(idlist[0]);
 	    AgentInfo	*ap;
-	    if ((ap = FindDomainAgent(domain)) == NULL)
+	    if ((ap = pmcd_agent(domain)) == NULL)
 		continue;
 	    if (!ap->status.connected)
 		continue;
+	    if (ap->status.fenced)
+		continue;
 	    if (ap->ipcType == AGENT_DSO) {
 		if (ap->ipc.dso.dispatch.comm.pmda_interface >= PMDA_INTERFACE_5)
 		    ap->ipc.dso.dispatch.version.four.ext->e_context = cp - client;
@@ -1192,7 +1229,7 @@ check:
 	goto done;
 
     pmcd_trace(TR_XMIT_PDU, cp->fd, PDU_PMNS_NAMES, travNL_num);
-    if ((sts = __pmSendNameList(cp->fd, FROM_ANON, travNL_num, travNL, NULL)) < 0) {
+    if ((sts = __pmSendNameList(cp->fd, FROM_ANON, travNL_num, (const char **)travNL, NULL)) < 0) {
 	pmcd_trace(TR_XMIT_ERR, cp->fd, PDU_PMNS_NAMES, sts);
 	CleanupClient(cp, sts);
 	goto done;
@@ -1298,7 +1335,8 @@ DoCreds(ClientInfo *cp, __pmPDU *pb)
 			{ PDU_FLAG_SECURE_ACK,	"SECURE_ACK" },
 			{ PDU_FLAG_NO_NSS_INIT,	"NO_NSS_INIT" },
 			{ PDU_FLAG_CONTAINER,	"CONTAINER" },
-			{ PDU_FLAG_LABEL,	"LABEL" },
+			{ PDU_FLAG_BAD_LABEL,	"BAD_LABEL" },
+			{ PDU_FLAG_LABELS,	"LABELS" },
 		    };
 		    int	i;
 		    int	first = 1;
