@@ -673,8 +673,13 @@ CheckNewClient(__pmFdSet * fdset, int rfd, int family)
     ClientInfo	*cp;
 
     if (__pmFD_ISSET(rfd, fdset)) {
-	if ((cp = AcceptNewClient(rfd)) == NULL)
+	if ((cp = AcceptNewClient(rfd)) == NULL) {
+	    if (pmDebugOptions.access) {
+		fprintf(stderr, "CheckNewClient: AcceptNewClient(%d) failed: %s\n",
+		    rfd, pmErrStr(-oserror()));
+	    }
 	    return;	/* Accept failed and no client added */
+	}
 
 	sts = __pmAccAddClient(cp->addr, &cp->denyOps);
 #if defined(HAVE_STRUCT_SOCKADDR_UN)
@@ -1032,6 +1037,8 @@ main(int argc, char *argv[])
     }
 
     if (run_daemon) {
+	/* notify service manager, if any, we are ready */
+	__pmServerNotifyServiceManagerReady(getpid());
 	if (__pmServerCreatePIDFile(PM_SERVER_SERVICE_SPEC, PM_FATAL_ERR) < 0)
 	    DontStart();
 	if (pmSetProcessIdentity(username) < 0)
@@ -1051,7 +1058,10 @@ main(int argc, char *argv[])
     /* all the work is done here */
     ClientLoop();
 
+    /* inform service manager and shutdown cleanly */
+    __pmServerNotifyServiceManagerStopping(pmcd_pid);
     Shutdown();
+
     exit(0);
 }
 
