@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2013 Red Hat.
+ * Copyright (c) 2012-2019 Red Hat.
  * Copyright (c) 1995-2001 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -11,35 +11,35 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef _PMCD_H
 #define _PMCD_H
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include "pmda.h"
 
-#ifdef IS_MINGW
-#ifdef PMCD_INTERNAL
-#define PMCD_INTERN __declspec(dllexport)
-#define PMCD_EXTERN
+/*
+ * Symbol visibility macros for pmcd and libpcp_pmcd MinGW builds
+ */
+#if !defined(IS_MINGW)
+# define PMCD_CALL
+# define PMCD_DATA
 #else
-#define PMCD_INTERN
-#define PMCD_EXTERN __declspec(dllimport)
-#endif
-#else /*!MINGW*/
-#define PMCD_INTERN
-#define PMCD_EXTERN extern
+# if defined(PMCD_INTERNAL)
+#  define PMCD_CALL __declspec(dllexport)
+#  define PMCD_DATA __declspec(dllexport)
+# else
+#  define PMCD_CALL __declspec(dllimport)
+#  define PMCD_DATA __declspec(dllimport)
+# endif
 #endif
 
 #include "client.h"
 
-/* Structures of type-specific info for each kind of domain agent-PMCD 
+/*
+ * Structures of type-specific info for each kind of domain agent-PMCD 
  * connection (DSO, socket, pipe).
  */
 
@@ -56,7 +56,7 @@ typedef struct {
 } DsoInfo;
 
 typedef struct {
-    int	  addrDomain;			/* AF_UNIX or AF_INET */
+    int	  addrDomain;			/* AF_UNIX, AF_INET or AF_INET6 */
     int	  port;				/* Port number if an INET socket */
     char  *name;			/* Port name if supplied for INET */
 					/* or socket name for UNIX */
@@ -87,11 +87,13 @@ typedef struct {
 	    connected : 1,		/* Agent connected to pmcd */
 	    busy : 1,			/* Processing a request */
 	    isChild : 1,		/* Is a child process of pmcd */
+	    isRootChild : 1,		/* Is a child process of pmdaroot */
 	    madeDsoResult : 1,		/* Pmcd made a "bad" pmResult (DSO only) */
 	    restartKeep : 1,		/* Keep agent if set during restart */
 	    notReady : 1,		/* Agent not ready to process PDUs */
 	    startNotReady : 1,		/* Agent starts in non-ready state */
-	    unused : 9,			/* Zero-padded, unused space */
+	    fenced : 1,			/* Agent fenced; no sampling */
+	    unused : 7,			/* Zero-padded, unused space */
 	    flags : 16;			/* Agent-supplied connection flags */
     } status;
     int		reason;			/* if ! connected */
@@ -102,16 +104,17 @@ typedef struct {
     } ipc;
 } AgentInfo;
 
-PMCD_EXTERN AgentInfo	*agent;		/* Array of domain agent structs */
-PMCD_EXTERN int		nAgents;	/* Number of agents in array */
+PMCD_DATA extern AgentInfo	*agent;		/* Array of domain agent structs */
+PMCD_DATA extern int		nAgents;	/* Number of agents in array */
 
 /*
  * DomainId-to-AgentIndex map
  * 9 bits of DomainId, max value is 510 because 511 is special (see
- * DYNAMIC_PMID in impl.h)
+ * DYNAMIC_PMID in libpcp.h)
  */
 #define MAXDOMID	510
 extern int		mapdom[];	/* the map */
+extern int		AgentDied;	/* for updating mapdom[] */
 
 /* Domain agent-PMCD connection types (AgentInfo.ipcType) */
 
@@ -142,16 +145,22 @@ extern int		mapdom[];	/* the map */
 #define REASON_NOSTART	4
 #define REASON_PROTOCOL	8
 
-extern AgentInfo *FindDomainAgent(int);
+PMCD_CALL extern AgentInfo *pmcd_agent(int);
 extern void CleanupAgent(AgentInfo *, int, int);
 extern int HarvestAgents(unsigned int);
 
+/* pmdaroot file descriptor */
+extern int	pmdarootfd;
+
 /* timeout to PMDAs (secs) */
-PMCD_EXTERN int	_pmcd_timeout;
+PMCD_DATA extern int	pmcd_timeout;
 
 /* timeout for credentials */
 extern int	_creds_timeout;
 
+/* flag for context label changes */
+extern int	labelChanged;
+
 /* global PMCD PMDA variables */
 
 /*
@@ -173,8 +182,8 @@ extern int	_creds_timeout;
 /*
  * trace control
  */
-PMCD_EXTERN int		_pmcd_trace_mask;
-PMCD_EXTERN int		_pmcd_trace_nbufs;
+PMCD_DATA extern int	pmcd_trace_mask;
+PMCD_DATA extern int	pmcd_trace_nbufs;
 
 /*
  * trace mask bits
@@ -186,9 +195,9 @@ PMCD_EXTERN int		_pmcd_trace_nbufs;
 /*
  * routines
  */
-extern void pmcd_init_trace(int);
-extern void pmcd_trace(int, int, int, int);
-extern void pmcd_dump_trace(FILE *);
+PMCD_CALL extern void pmcd_init_trace(int);
+PMCD_CALL extern void pmcd_trace(int, int, int, int);
+PMCD_CALL extern void pmcd_dump_trace(FILE *);
 extern int pmcd_load_libpcp_pmda(void);
 
 /*
@@ -197,6 +206,7 @@ extern int pmcd_load_libpcp_pmda(void);
 extern int DoFetch(ClientInfo *, __pmPDU *);
 extern int DoProfile(ClientInfo *, __pmPDU *);
 extern int DoDesc(ClientInfo *, __pmPDU *);
+extern int DoLabel(ClientInfo *, __pmPDU *);
 extern int DoInstance(ClientInfo *, __pmPDU *);
 extern int DoText(ClientInfo *, __pmPDU *);
 extern int DoStore(ClientInfo *, __pmPDU *);
@@ -211,21 +221,42 @@ extern int DoPMNSTraverse(ClientInfo *, __pmPDU *);
  */
 extern void StartDaemon(int, char **);
 extern void Shutdown(void);
-extern int  ParseInitAgents(char *);
+extern int VerifyConfig(char *);
+extern int ParseInitAgents(char *);
 extern void ParseRestartAgents(char *);
 extern void PrintAgentInfo(FILE *);
-extern void ResetBadHosts(void);
-extern void MarkStateChanges(int);
+extern void CheckLabelChange(void);
+extern void MarkStateChanges(unsigned int);
 extern void CleanupClient(ClientInfo *, int);
-extern int ClientsAuthentication(AgentInfo *);
-extern int AgentsAuthentication(int);
+extern int ClientsAttributes(AgentInfo *);
+extern int AgentsAttributes(int);
 extern pmResult **SplitResult(pmResult *);
 
 /*
  * Highest known file descriptor used for a Client or an Agent connection.
  * This is reported in the pmcd.openfds metric.
  */
-PMCD_EXTERN int pmcd_hi_openfds;
-extern void pmcd_openfds_sethi(int fd);
+PMCD_DATA extern int pmcd_hi_openfds;
+PMCD_CALL extern void pmcd_openfds_sethi(int);
+
+/* Explicitly requested hostname (pmcd.hostname metric) */
+PMCD_DATA extern char *pmcd_hostname;
+
+/* Current context label set (pmcd.labels metric) */
+PMCD_DATA extern char *pmcd_labels;
+
+/* Counter of SIGHUPs received and responded to by pmcd */
+PMCD_DATA extern unsigned pmcd_sighups;
+
+/* pmcd's pid */
+PMCD_DATA extern pid_t pmcd_pid;
+
+/* config sequence number ...
+ * incremented each time a PMDA is started or restarted
+ */
+PMCD_DATA extern int pmcd_seqnum;
+
+/* Flag indicating whether agents are currently fenced */
+PMCD_DATA extern int pmcd_fenced;
 
 #endif /* _PMCD_H */
