@@ -224,9 +224,19 @@ get_sample(info_t *ip)
 	    pmExtractValue(crp->vset[FREEMEM]->valfmt, crp->vset[FREEMEM]->vlist,
 		    desclist[FREEMEM].type, &tmp, PM_TYPE_FLOAT);
 	    /* convert from today's units at the collection site to Mbytes */
-	    pmConvScale(PM_TYPE_FLOAT, &tmp, &desclist[FREEMEM].units,
+	    sts = pmConvScale(PM_TYPE_FLOAT, &tmp, &desclist[FREEMEM].units,
 		    &atom, &mbyte_scale);
-	    ip->freemem = atom.f;
+	    if (sts < 0) {
+		/* should never happen */
+		if (pmDebugOptions.value) {
+		    fprintf(stderr, "%s: get_sample: Botch: %s (%s) scale conversion from %s", 
+			pmGetProgname(), pmIDStr(desclist[FREEMEM].pmid), pmclient_sample[FREEMEM], pmUnitsStr(&desclist[FREEMEM].units));
+		    fprintf(stderr, " to %s failed: %s\n", pmUnitsStr(&mbyte_scale), pmErrStr(sts));
+		}
+		ip->freemem = 0;
+	    }
+	    else
+		ip->freemem = atom.f;
 	}
 
 	/* disk IOPS - expect just one value, but need delta */
@@ -294,10 +304,12 @@ main(int argc, char **argv)
 	pmprintf("%s: pause can only be used with archives\n", pmGetProgname());
 	opts.errors++;
     }
-
-    if (opts.errors || opts.optind < argc - 1) {
+    if (opts.optind < argc - 1)
+	opts.errors++;
+    if (opts.errors || (opts.flags & PM_OPTFLAG_EXIT)) {
+	sts = !(opts.flags & PM_OPTFLAG_EXIT);
 	pmUsageMessage(&opts);
-	exit(1);
+	exit(sts);
     }
 
     if (opts.context == PM_CONTEXT_ARCHIVE) {
@@ -353,11 +365,13 @@ main(int argc, char **argv)
 
     while (samples == -1 || samples-- > 0) {
 	if (lines % 15 == 0) {
+	    time_t	time;
 	    if (opts.context == PM_CONTEXT_ARCHIVE)
 		printf("Archive: %s, ", opts.archives[0]);
+	    time = info.timestamp.tv_sec;
 	    printf("Host: %s, %d cpu(s), %s",
 		    host, ncpu,
-		    pmCtime((const time_t *)&info.timestamp.tv_sec, timebuf));
+		    pmCtime(&time, timebuf));
 /* - report format
   CPU  Busy    Busy  Free Mem   Disk     Load Average
  Util   CPU    Util  (Mbytes)   IOPS    1 Min  15 Min
