@@ -2,7 +2,7 @@
  * Configurable Kernel Virtual Machine (KVM) PMDA
  *
  * Copyright (c) 2018 Fujitsu.
- * Copyright (c) 2018 Red Hat.
+ * Copyright (c) 2018,2020 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,11 +27,15 @@
 typedef struct perf_event_attr perf_event_attr_t;
 #endif
 
+#define INTEGRITY		"[integrity]"
+#define INTEGRITY_LEN		(sizeof(INTEGRITY)-1)
+#define CONFIDENTIALITY		"[confidentiality]"
+#define CONFIDENTIALITY_LEN	(sizeof(CONFIDENTIALITY)-1)
+
 static int _isDSO;
 static pmdaNameSpace *pmns;
 static char *username;
 static char helppath[MAXPATHLEN];
-static char sep;
 
 static int ntrace;
 static char **trace_nametab;
@@ -39,6 +43,8 @@ static int ncpus;
 static int *group_fd;
 static char tracefs[MAXPATHLEN];
 static char debugfs[MAXPATHLEN];
+static char lockdown[MAXPATHLEN];
+static int kernel_lockdown;
 static pmdaMetric *tmetrictab;
 
 static pmdaIndom indomtab[] = {
@@ -192,6 +198,9 @@ kvm_debug_refresh(kvm_debug_value_t *kvm)
     char		path[MAXPATHLEN];
     int			i, sts = 0;
 
+    if (kernel_lockdown)
+	return PM_ERR_PERMISSION;
+
     pmsprintf(path, sizeof(path), "%s/kvm", debugfs);
     if ((kvm_dir = opendir(path)) == NULL)
 	return -oserror();
@@ -235,7 +244,7 @@ kvm_trace_refresh(void)
     size_t		bufsize = ksize + sizeof(unsigned long long);
     int			i, sts, changed = 0;
 
-    if (ntrace == 0)
+    if (ntrace == 0 || group_fd == NULL || kernel_lockdown)
 	return;
 
     if (buffer == NULL) {
@@ -291,6 +300,9 @@ perf_event(int ncpus, int *group_fd)
     int			i, fd = 0, cpu, flag, offset = 0, sts = 0;
     char		path[MAXPATHLEN];
 
+    if (kernel_lockdown)
+	return PM_ERR_PERMISSION;
+
     memset(&pe, 0, sizeof(perf_event_attr_t));
     pe.type = PERF_TYPE_TRACEPOINT;
     pe.size = sizeof(struct perf_event_attr);
@@ -385,6 +397,9 @@ kvm_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     char		*name;
     int			sts;
 
+    if (kernel_lockdown)
+	return PM_ERR_PERMISSION;
+
     switch (cluster) {
     case CLUSTER_TRACE:
 	if (item == 0) {
@@ -414,6 +429,29 @@ kvm_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     return 1;
 }
 
+static int
+kvm_text(int ident, int type, char **buf, pmdaExt *pmda)
+{
+    if ((type & PM_TEXT_PMID) == PM_TEXT_PMID) {
+	static char	text[1024];
+	pmID		pmid = (pmID)ident;
+
+	if (pmID_cluster(pmid) == CLUSTER_TRACE) {
+	    if (pmID_item(pmid) == 0)	/* kvm.trace.count */
+		pmsprintf(text, sizeof(text),
+			"Number of KVM trace points from %s/kvm/kvm.conf",
+				pmGetOptionalConfig("PCP_PMDAS_DIR"));
+	    else
+		pmsprintf(text, sizeof(text),
+			"KVM trace point values from %s/events/kvm files",
+				tracefs);
+	    *buf = text;
+	    return 0;
+	}
+    }
+    return pmdaText(ident, type, buf, pmda);
+}
+
 static int
 kvm_pmid(const char *name, pmID *pmid, pmdaExt *pmda)
 {
@@ -479,13 +517,14 @@ kvm_config(void)
 {
     char		*p;
     char		buf[BUFSIZ];
+    char		path[MAXPATHLEN];
     void		*table;
     FILE		*fp;
     enum { STATE_UNKNOWN, STATE_PATHS, STATE_TRACE } state = 0;
 
-    pmsprintf(buf, sizeof(buf), "%s%ckvm%ckvm.conf",
-		pmGetOptionalConfig("PCP_PMDAS_DIR"), sep, sep);
-    if ((fp = fopen(buf, "rt")) == NULL)
+    pmsprintf(buf, sizeof(buf), "%s/kvm/kvm.conf",
+		pmGetOptionalConfig("PCP_PMDAS_DIR"));
+    if ((fp = fopen(buf, "r")) == NULL)
 	return -oserror();
     while (fgets(buf, sizeof(buf), fp) != NULL) {
 	buf[sizeof(buf)-1] = '\0';
@@ -509,10 +548,18 @@ kvm_config(void)
 	}
 
 	if (state == STATE_PATHS) {
-	    if (sscanf(p, "tracefs=%s", tracefs) != 0)
+	    if (sscanf(p, "tracefs=%s", path) == 1) {
+		strcpy(tracefs, path);
 	        continue;
-	    if (sscanf(p, "debugfs=%s", debugfs) != 0)
+	    }
+	    if (sscanf(p, "debugfs=%s", path) == 1) {
+		strcpy(debugfs, path);
 		continue;
+	    }
+	    if (sscanf(p, "lockdown=%s", path) == 1) {
+		strcpy(lockdown, path);
+		continue;
+	    }
 	}
 	if (state == STATE_TRACE) {
 	    if (!(table = realloc(trace_nametab, (ntrace+1) * sizeof(char*)))) {
@@ -528,6 +575,24 @@ kvm_config(void)
 	}
     }
     fclose(fp);
+
+    if (pmDebugOptions.libpmda)
+	fprintf(stderr, "Checking kernel lockdown state in %s\n", lockdown);
+
+    if ((fp = fopen(lockdown, "r")) == NULL)
+	return 0;
+    while (fgets(buf, sizeof(buf), fp) != NULL) {
+	if ((p = strchr(buf, '[')) == NULL)
+	    continue;
+	if (strncmp(p, CONFIDENTIALITY, CONFIDENTIALITY_LEN) == 0 ||
+	    strncmp(p, INTEGRITY, INTEGRITY_LEN) == 0) {
+	    pmNotifyErr(LOG_INFO,
+		    "disabling KVM metrics: kernel running in lockdown mode");
+	    kernel_lockdown = 1;
+	}
+    }
+    fclose(fp);
+
     return 0;
 }
 
@@ -547,9 +612,8 @@ kvm_init(pmdaInterface *dp)
     int			m = 0, sts;
 
     if (_isDSO) {
-	sep = pmPathSeparator();
-	pmsprintf(helppath, sizeof(helppath), "%s%c" "kvm" "%c" "help",
-		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
+	pmsprintf(helppath, sizeof(helppath), "%s/kvm/help",
+		pmGetConfig("PCP_PMDAS_DIR"));
 	pmdaDSO(dp, PMDA_INTERFACE_7, "KVM DSO", helppath);
     } else {
         if (username)
@@ -559,6 +623,25 @@ kvm_init(pmdaInterface *dp)
     if (dp->status != 0)
 	return;
 
+    if ((envpath = getenv("KVM_NCPUS")))
+	ncpus = atoi(envpath);
+    else
+	ncpus = sysconf(_SC_NPROCESSORS_CONF);
+    if (ncpus <= 0)
+	ncpus = 1;
+    if ((envpath = getenv("KVM_DEBUGFS_PATH")))
+	pmsprintf(debugfs, sizeof(debugfs), "%s", envpath);
+    else
+	pmsprintf(debugfs, sizeof(debugfs), "/sys/kernel/debug");
+    if ((envpath = getenv("KVM_TRACEFS_PATH")))
+	pmsprintf(tracefs, sizeof(tracefs), "%s", envpath);
+    else
+	pmsprintf(tracefs, sizeof(tracefs), "/sys/kernel/debug/tracing");
+    if ((envpath = getenv("KVM_LOCKDOWN_PATH")))
+	pmsprintf(lockdown, sizeof(lockdown), "%s", envpath);
+    else
+	pmsprintf(lockdown, sizeof(lockdown), "/sys/kernel/security/lockdown");
+
     /* get paths and any specified KVM trace metrics */ 
     kvm_config();
 
@@ -589,32 +672,19 @@ kvm_init(pmdaInterface *dp)
 	tmetrics = nmetrics;
     }
 
-    if ((envpath = getenv("KVM_NCPUS")))
-	ncpus = atoi(envpath);
-    else
-	ncpus = sysconf(_SC_NPROCESSORS_CONF);
-    if (ncpus <= 0)
-	ncpus = 1;
-    if ((envpath = getenv("KVM_DEBUGFS_PATH")))
-	pmsprintf(debugfs, sizeof(debugfs), "%s", envpath);
-    else
-	pmsprintf(debugfs, sizeof(debugfs), "/sys/kernel/debug");
-    if ((envpath = getenv("KVM_TRACEFS_PATH")))
-	pmsprintf(tracefs, sizeof(tracefs), "%s", envpath);
-    else
-	pmsprintf(tracefs, sizeof(tracefs), "/sys/kernel/debug/tracing");
-
     if (tmetrictab != metrictab) {
 	group_fd = malloc(ncpus * sizeof(int));
 	if ((sts = perf_event(ncpus, group_fd)) < 0) {
 	    pmNotifyErr(LOG_INFO, "disabling perf_event support: %s",
 			pmErrStr(sts));
 	    free(group_fd);
+	    group_fd = NULL;
 	}
     }
 
     dp->version.seven.fetch = kvm_fetch;
     dp->version.seven.label = kvm_label;
+    dp->version.seven.text = kvm_text;
     dp->version.seven.pmid = kvm_pmid;
     dp->version.seven.name = kvm_name;
     dp->version.seven.children = kvm_children;
@@ -654,9 +724,8 @@ main(int argc, char **argv)
     _isDSO = 0;
     pmSetProgname(argv[0]);
 
-    sep = pmPathSeparator();
-    pmsprintf(helppath, sizeof(helppath), "%s%c" "kvm" "%c" "help",
-		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
+    pmsprintf(helppath, sizeof(helppath), "%s/kvm/help",
+		pmGetConfig("PCP_PMDAS_DIR"));
     pmdaDaemon(&dispatch, PMDA_INTERFACE_7, pmGetProgname(), KVM, "kvm.log", helppath);
 
     pmdaGetOptions(argc, argv,  &opts, &dispatch);
