@@ -1,6 +1,7 @@
 /*
  * Linux /proc/cpuinfo metrics cluster
  *
+ * Copyright (c) 2013-2015,2017,2020 Red Hat.
  * Copyright (c) 2001 Gilly Ran (gilly@exanet.com) for the
  * portions of the code supporting the Alpha platform.
  * All rights reserved.
@@ -15,36 +16,8 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifdef  __alpha__
-#define HAVE_ALPHA_LINUX
-#endif
-
-typedef struct {
-    int			cpu_num;
-    char		*sapic;
-    char		*name;
-    char		cpu_char;
-    int			node;
-    float		clock;
-    char		*vendor;
-    char		*model;
-    char		*stepping;
-    unsigned int	cache;
-    float		bogomips;
-} cpuinfo_t;
-
-typedef struct {
-    char		*machine;
-    cpuinfo_t		*cpuinfo;
-    pmdaIndom		*cpuindom;
-    pmdaIndom		*node_indom;
-} proc_cpuinfo_t;
-
-extern int refresh_proc_cpuinfo(proc_cpuinfo_t *);
-extern char *cpu_name(proc_cpuinfo_t *, int);
+extern int refresh_proc_cpuinfo(void);
+extern int refresh_sysfs_online(char *, const char *);
+extern unsigned long refresh_sysfs_thermal_throttle(char *, const char *, const char *);
