@@ -1,7 +1,7 @@
 /*
  * Linux Partitions (disk and disk partition IO stats) Cluster
  *
- * Copyright (c) 2012-2018 Red Hat.
+ * Copyright (c) 2012-2018,2020 Red Hat.
  * Copyright (c) 2015 Intel, Inc.  All Rights Reserved.
  * Copyright (c) 2008,2012 Aconex.  All Rights Reserved.
  * Copyright (c) 2000,2004 Silicon Graphics, Inc.  All Rights Reserved.
@@ -40,6 +40,9 @@ static int _pm_have_kernel_2_6_partition_stats;
  * multi-device (MD) devices are named md[0-9]* and are mapped to optional
  * persistent names using the symlinks in /dev/md.
  *
+ * zram devices (compressed RAM) are named zram[0-9][0-9]*.  There are no
+ * zram sub-partitions.
+ *
  * Mylex raid disks are named e.g. rd/c0d0 or dac960/c0d0
  * Mylex raid partitions are named e.g. rd/c0d0p1 or dac960/c0d0p1
  *
@@ -65,6 +68,12 @@ _pm_isramdisk(char *dname)
     return strncmp(dname, "ram", 3) == 0;
 }
 
+static int
+_pm_iszram(char *dname)
+{
+    return strncmp(dname, "zram", 4) == 0;
+}
+
 static int
 _pm_ismmcdisk(char *dname)
 {
@@ -159,7 +168,7 @@ _pm_ispartition(char *dname)
     }
     else {
 	/*
-	 * default test : partition names end in a digit do not look
+	 * default test: partition names end in a digit and do not look
 	 * like loopback devices.  Handle other special cases here.
 	 */
 	return isdigit((int)dname[m]) &&
@@ -168,6 +177,7 @@ _pm_ispartition(char *dname)
 		!_pm_ismmcdisk(dname) &&
 		!_pm_isnvmedrive(dname) &&
 		!_pm_iscephrados(dname) &&
+		!_pm_iszram(dname) &&
 		!_pm_isnbd(dname) &&
 		!_pm_ismd(dname) &&
 		!_pm_isdm(dname) &&
@@ -185,12 +195,13 @@ _pm_isxvmvol(char *dname)
 }
 
 /*
- * return true is arg is a disk name
+ * return true if arg is a disk name
  */
 static int
 _pm_isdisk(char *dname)
 {
     return (!_pm_isloop(dname) && !_pm_isramdisk(dname) &&
+	    !_pm_iscdrom(dname) && !_pm_iszram(dname) &&
 	    !_pm_ispartition(dname) && !_pm_isxvmvol(dname) &&
 	    !_pm_isdm(dname) && !_pm_ismd(dname));
 }
@@ -198,9 +209,9 @@ _pm_isdisk(char *dname)
 static int
 refresh_mdadm(const char *name)
 {
-    char mdadm[MAXPATHLEN];
-    char args[] = "--detail --test";
-    FILE *pfp;
+    char		mdadm[MAXPATHLEN];
+    char		args[] = "--detail --test";
+    FILE		*pfp;
 
     if (access(linux_mdadm, R_OK) != 0)
     	return -1;
@@ -213,18 +224,106 @@ refresh_mdadm(const char *name)
     return pclose(pfp);
 }
 
+static int
+refresh_zram_io_stat(const char *name, zram_stat_t *zram)
+{
+    zram_io_stat_t	*io;
+    char		buf[MAXPATHLEN];
+    FILE		*fp;
+    int			sts;
+
+    if (zram->uptodate & ZRAM_IO)
+	return 0;
+    pmsprintf(buf, sizeof(buf), "%s/sys/block/%s/io_stat", linux_statspath, name);
+    if ((fp = fopen(buf, "r")) == NULL)
+	return -ENOENT;
+    io = &zram->iostat;
+    sts = fscanf(fp, "%llu %llu %llu %llu", &io->failed_reads,
+		    &io->failed_writes, &io->invalid_io, &io->notify_free);
+    fclose(fp);
+    if (sts != 4)
+	return -ENODATA;
+    zram->uptodate |= ZRAM_IO;
+    return 0;
+}
+
+static int
+refresh_zram_mm_stat(const char *name, zram_stat_t *zram)
+{
+    zram_mm_stat_t	*mm;
+    char		buf[MAXPATHLEN];
+    FILE		*fp;
+    int			sts;
+
+    if (zram->uptodate & ZRAM_MM)
+	return 0;
+    pmsprintf(buf, sizeof(buf), "%s/sys/block/%s/mm_stat", linux_statspath, name);
+    if ((fp = fopen(buf, "r")) == NULL)
+	return -ENOENT;
+    mm = &zram->mmstat;
+    sts = fscanf(fp, "%llu %llu %llu %llu %llu %llu %llu %llu",
+		    &mm->original, &mm->compressed, &mm->mem_used,
+		    &mm->mem_limit, &mm->max_used, &mm->same_pages,
+		    &mm->compacted_pages, &mm->huge_pages);
+    fclose(fp);
+    if (sts != 8)
+	return -ENODATA;
+    zram->uptodate |= ZRAM_MM;
+    return 0;
+}
+
+static int
+refresh_zram_bd_stat(const char *name, zram_stat_t *zram)
+{
+    zram_bd_stat_t	*bd;
+    char		buf[MAXPATHLEN];
+    FILE		*fp;
+    int			sts;
+
+    if (zram->uptodate & ZRAM_BD)
+	return 0;
+    pmsprintf(buf, sizeof(buf), "%s/sys/block/%s/bd_stat", linux_statspath, name);
+    if ((fp = fopen(buf, "r")) == NULL)
+	return -ENOENT;
+    bd = &zram->bdstat;
+    sts = fscanf(fp, "%llu %llu %llu", &bd->count, &bd->reads, &bd->writes);
+    fclose(fp);
+    if (sts != 3)
+	return -ENODATA;
+    zram->uptodate |= ZRAM_BD;
+    return 0;
+}
+
+static int
+refresh_zram_size(const char *name, zram_stat_t *zram, unsigned long long *blocks)
+{
+    char		buf[MAXPATHLEN];
+    FILE		*fp;
+    int			sts;
+
+    if (zram->uptodate & ZRAM_SIZE)
+	return 0;
+    pmsprintf(buf, sizeof(buf), "%s/sys/block/%s/disksize", linux_statspath, name);
+    if ((fp = fopen(buf, "r")) == NULL)
+	return -ENOENT;
+    sts = fscanf(fp, "%llu", blocks);
+    fclose(fp);
+    if (sts != 1)
+	return -ENODATA;
+    *blocks >>= 10;	/* bytes to kilobytes */
+    zram->uptodate |= ZRAM_SIZE;
+    return 0;
+}
+
 static void
 refresh_udev(pmInDom disk_indom, pmInDom partitions_indom)
 {
-    char buf[MAXNAMELEN];
-    char realname[MAXNAMELEN];
-    char *shortname;
-    char *p;
-    char *udevname;
-    FILE *pfp;
-    partitions_entry_t *entry;
-    int indom;
-    int inst;
+    char		buf[MAXNAMELEN];
+    char		realname[MAXNAMELEN];
+    char		*p, *udevname, *shortname;
+    FILE		*pfp;
+    partitions_entry_t	*entry;
+    int			indom, inst;
 
     if (access("/dev/xscsi", R_OK) != 0)
     	return;
@@ -262,13 +361,12 @@ refresh_udev(pmInDom disk_indom, pmInDom partitions_indom)
 static int
 persistent_dm_name(char *namebuf, int namelen, int devmajor, int devminor)
 {
-    int fd;
-    char *p;
-    DIR *dp;
-    int found = 0;
-    struct dirent *dentry;
-    struct stat sb;
-    char path[MAXPATHLEN];
+    char		*p;
+    DIR			*dp;
+    int			fd, found = 0;
+    struct dirent	*dentry;
+    struct stat		sb;
+    char		path[MAXPATHLEN];
 
     pmsprintf(path, sizeof(path), "%s/sys/block/%s/dm/name", linux_statspath, namebuf);
     if ((fd = open(path, O_RDONLY)) >= 0) {
@@ -276,7 +374,8 @@ persistent_dm_name(char *namebuf, int namelen, int devmajor, int devminor)
     	if (read(fd, path, sizeof(path)-1) > 0) {
 	    if ((p = strchr(path, '\n')) != NULL)
 	    	*p = '\0';
-	    strncpy(namebuf, path, MIN((sizeof(path)-1), namelen));
+	    strncpy(namebuf, path, namelen-1);
+	    namebuf[namelen-1] = '\0';	/* buffer overrun guard */
 	    found = 1;
 	}
     	close(fd);
@@ -295,7 +394,8 @@ persistent_dm_name(char *namebuf, int namelen, int devmajor, int devminor)
 		if (stat(path, &sb) != 0 || !S_ISBLK(sb.st_mode))
 		    continue; /* only interested in block devices */
 		if (devmajor == major(sb.st_rdev) && devminor == minor(sb.st_rdev)) {
-		    strncpy(namebuf, dentry->d_name, namelen);
+		    strncpy(namebuf, dentry->d_name, namelen-1);
+		    namebuf[namelen-1] = '\0';	/* buffer overrun guard */
 		    found = 1;
 		    break;
 		}
@@ -315,12 +415,12 @@ persistent_dm_name(char *namebuf, int namelen, int devmajor, int devminor)
 static int
 persistent_md_name(char *namebuf, int namelen)
 {
-    DIR *dp;
-    ssize_t size;
-    int found = 0;
-    struct dirent *dentry;
-    char path[MAXPATHLEN];
-    char name[MAXPATHLEN];
+    DIR			*dp;
+    ssize_t		size;
+    int			found = 0;
+    struct dirent	*dentry;
+    char		path[MAXPATHLEN];
+    char		name[MAXPATHLEN];
 
     pmsprintf(path, sizeof(path), "%s/dev/md", linux_statspath);
     if ((dp = opendir(path)) != NULL) {
@@ -347,7 +447,7 @@ persistent_md_name(char *namebuf, int namelen)
 
 static partitions_entry_t *
 refresh_disk_indom(char *namebuf, size_t namelen, int devmaj, int devmin,
-		pmInDom disk_indom, pmInDom part_indom,
+		pmInDom disk_indom, pmInDom part_indom, pmInDom zram_indom,
 		pmInDom dm_indom, pmInDom md_indom, int *indom_changes)
 {
     int			indom, inst;
@@ -366,6 +466,8 @@ refresh_disk_indom(char *namebuf, size_t namelen, int devmaj, int devmin,
 	indom = part_indom;
     else if (_pm_isdisk(namebuf))
 	indom = disk_indom;
+    else if (_pm_iszram(namebuf))
+	indom = zram_indom;
     else
 	return NULL;
 
@@ -385,12 +487,16 @@ refresh_disk_indom(char *namebuf, size_t namelen, int devmaj, int devmin,
     if (pmdaCacheLookupName(indom, namebuf, &inst, (void **)&p) < 0 || !p) {
 	/* not found: allocate and add a new entry */
 	p = (partitions_entry_t *)calloc(1, sizeof(partitions_entry_t));
+	if (indom == zram_indom)
+	    p->zram = (zram_stat_t *)calloc(1, sizeof(zram_stat_t));
 	*indom_changes += 1;
     } else {
 	if (p->dmname)
 	    free(p->dmname);
 	if (p->mdname)
 	    free(p->mdname);
+	if (p->zram)
+	    p->zram->uptodate = 0;	/* refreshing now required */
 	p->nr_blocks = 0;		/* zero if not read/needed */
     }
 
@@ -416,7 +522,7 @@ refresh_disk_indom(char *namebuf, size_t namelen, int devmaj, int devmin,
 
 static int
 refresh_diskstats(FILE *fp, pmInDom disk_indom, pmInDom part_indom,
-			    pmInDom dm_indom, pmInDom md_indom)
+		pmInDom zram_indom, pmInDom dm_indom, pmInDom md_indom)
 {
     int			indom_changes = 0;
     int			devmin, devmaj, n;
@@ -433,38 +539,52 @@ refresh_diskstats(FILE *fp, pmInDom disk_indom, pmInDom part_indom,
 	    continue;
 
 	if (!(p = refresh_disk_indom(name, sizeof(name), devmaj, devmin,
-		disk_indom, part_indom, dm_indom, md_indom, &indom_changes)))
+			disk_indom, part_indom, zram_indom, dm_indom, md_indom,
+			&indom_changes)))
 	    continue;
 
 	/* 2.6 style /proc/diskstats */
 	name[0] = '\0';
 	/* Linux source: block/genhd.c::diskstats_show(1) */
-	n = sscanf(buf, "%u %u %s %lu %lu %llu %u %lu %lu %llu %u %u %u %u",
+	n = sscanf(buf, "%u %u %s %llu %llu %llu %u %llu %llu %llu %u "
+			"%u %u %u %llu %llu %llu %u %llu %u",
 		&p->major, &p->minor, name,
 		&p->rd_ios, &p->rd_merges, &p->rd_sectors, &p->rd_ticks,
 		&p->wr_ios, &p->wr_merges, &p->wr_sectors, &p->wr_ticks,
-		&p->ios_in_flight, &p->io_ticks, &p->aveq);
-	if (n != 14) {
+		&p->ios_in_flight, &p->io_ticks, &p->aveq,
+		&p->ds_ios, &p->ds_merges, &p->ds_sectors, &p->ds_ticks,
+		&p->fl_ios, &p->fl_ticks);
+	if (n < 14) {
 	    /*
 	     * From 2.6.25 onward, the full set of statistics is
 	     * available again for both partitions and disks.
 	     */
 	    _pm_have_kernel_2_6_partition_stats = 1;
 	    p->rd_merges = p->wr_merges = p->wr_ticks =
-			p->ios_in_flight = p->io_ticks = p->aveq = 0;
+			p->ios_in_flight = p->io_ticks = p->aveq =
+			p->ds_ios = p->ds_merges = p->ds_sectors =
+			p->ds_ticks = p->fl_ios = p->fl_ticks = 0;
 	    /* Linux source: block/genhd.c::diskstats_show(2) */
 	    sscanf(buf, "%u %u %s %u %u %u %u\n",
 		    &p->major, &p->minor, name,
 		    (unsigned int *)&p->rd_ios, (unsigned int *)&p->rd_sectors,
 		    (unsigned int *)&p->wr_ios, (unsigned int *)&p->wr_sectors);
 	}
+	if (n < 18) {
+	    /* Discard statistics are not present */
+	    p->ds_ios = p->ds_merges = p->ds_sectors = p->ds_ticks = 0;
+	}
+	if (n < 20) {
+	    /* Flush statistics are not present */
+	    p->fl_ios = p->fl_ticks = 0;
+	}
     }
     return indom_changes;
 }
 
 static int
 refresh_partitions(FILE *fp, pmInDom disk_indom, pmInDom part_indom,
-			     pmInDom dm_indom, pmInDom md_indom)
+		pmInDom zram_indom, pmInDom dm_indom, pmInDom md_indom)
 {
     int			indom_changes = 0;
     int			devmin, devmaj, n;
@@ -483,12 +603,14 @@ refresh_partitions(FILE *fp, pmInDom disk_indom, pmInDom part_indom,
 	    continue;
 
 	if (!(p = refresh_disk_indom(name, sizeof(name), devmaj, devmin,
-		disk_indom, part_indom, dm_indom, md_indom, &indom_changes)))
+			disk_indom, part_indom, zram_indom, dm_indom, md_indom,
+			&indom_changes)))
 	    continue;
 
 	/* 2.4 format /proc/partitions (distro patched) */
 	name[0] = '\0';
-	sscanf(buf, "%u %u %llu %s %lu %lu %llu %u %lu %lu %llu %u %u %u %u",
+	sscanf(buf,
+		"%u %u %llu %s %llu %llu %llu %u %llu %llu %llu %u %u %u %u",
 		&p->major, &p->minor, &p->nr_blocks, name,
 		&p->rd_ios, &p->rd_merges, &p->rd_sectors,
 		&p->rd_ticks, &p->wr_ios, &p->wr_merges,
@@ -500,7 +622,7 @@ refresh_partitions(FILE *fp, pmInDom disk_indom, pmInDom part_indom,
 
 int
 refresh_proc_partitions(pmInDom disk_indom, pmInDom part_indom,
-			pmInDom dm_indom, pmInDom md_indom,
+			pmInDom zram_indom, pmInDom dm_indom, pmInDom md_indom,
 			int need_diskstats, int need_partitions)
 {
     FILE	*fp;
@@ -512,6 +634,7 @@ refresh_proc_partitions(pmInDom disk_indom, pmInDom part_indom,
 	/* initialize the instance domain caches */
 	pmdaCacheOp(disk_indom, PMDA_CACHE_LOAD);
 	pmdaCacheOp(part_indom, PMDA_CACHE_LOAD);
+	pmdaCacheOp(zram_indom, PMDA_CACHE_LOAD);
 	pmdaCacheOp(dm_indom, PMDA_CACHE_LOAD);
 	pmdaCacheOp(md_indom, PMDA_CACHE_LOAD);
 	indom_changes = 1;
@@ -520,6 +643,7 @@ refresh_proc_partitions(pmInDom disk_indom, pmInDom part_indom,
 
     pmdaCacheOp(disk_indom, PMDA_CACHE_INACTIVE);
     pmdaCacheOp(part_indom, PMDA_CACHE_INACTIVE);
+    pmdaCacheOp(zram_indom, PMDA_CACHE_INACTIVE);
     pmdaCacheOp(dm_indom, PMDA_CACHE_INACTIVE);
     pmdaCacheOp(md_indom, PMDA_CACHE_INACTIVE);
 
@@ -527,7 +651,7 @@ refresh_proc_partitions(pmInDom disk_indom, pmInDom part_indom,
     if (need_diskstats) {
 	if ((fp = linux_statsfile("/proc/diskstats", buf, sizeof(buf)))) {
 	    indom_changes += refresh_diskstats(fp, disk_indom, part_indom,
-						dm_indom, md_indom);
+						zram_indom, dm_indom, md_indom);
 	    fclose(fp);
 	} else {
 	    need_partitions = 1;
@@ -538,7 +662,7 @@ refresh_proc_partitions(pmInDom disk_indom, pmInDom part_indom,
     if (need_partitions) {
 	if ((fp = linux_statsfile("/proc/partitions", buf, sizeof(buf)))) {
 	    indom_changes += refresh_partitions(fp, disk_indom, part_indom,
-						dm_indom, md_indom);
+						zram_indom, dm_indom, md_indom);
 	    fclose(fp);
 	}
     }
@@ -555,6 +679,7 @@ refresh_proc_partitions(pmInDom disk_indom, pmInDom part_indom,
 	refresh_udev(disk_indom, part_indom);
 	pmdaCacheOp(disk_indom, PMDA_CACHE_SAVE);
 	pmdaCacheOp(part_indom, PMDA_CACHE_SAVE);
+	pmdaCacheOp(zram_indom, PMDA_CACHE_SAVE);
 	pmdaCacheOp(dm_indom, PMDA_CACHE_SAVE);
 	pmdaCacheOp(md_indom, PMDA_CACHE_SAVE);
     }
@@ -586,6 +711,14 @@ static pmID disk_metric_table[] = {
     /* disk.dev.write_rawactive	*/   PMDA_PMID(CLUSTER_STAT,73),
     /* disk.dev.total_rawactive	*/   PMDA_PMID(CLUSTER_STAT,79),
     /* disk.dev.capacity */	     PMDA_PMID(CLUSTER_STAT,87),
+    /* disk.dev.discard */	     PMDA_PMID(CLUSTER_STAT,88),
+    /* disk.dev.blkdiscard */	     PMDA_PMID(CLUSTER_STAT,89),
+    /* disk.dev.discard_bytes */     PMDA_PMID(CLUSTER_STAT,90),
+    /* disk.dev.discard_merge */     PMDA_PMID(CLUSTER_STAT,91),
+    /* disk.dev.discard_rawactive */ PMDA_PMID(CLUSTER_STAT,92),
+    /* disk.dev.flush */	     PMDA_PMID(CLUSTER_STAT,93),
+    /* disk.dev.blkflush */	     PMDA_PMID(CLUSTER_STAT,94),
+    /* disk.dev.flush_rawactive */   PMDA_PMID(CLUSTER_STAT,95),
 
     /* disk.all.read */		     PMDA_PMID(CLUSTER_STAT,24),
     /* disk.all.write */	     PMDA_PMID(CLUSTER_STAT,25),
@@ -603,6 +736,14 @@ static pmID disk_metric_table[] = {
     /* disk.all.read_rawactive */    PMDA_PMID(CLUSTER_STAT,74),
     /* disk.all.write_rawactive	*/   PMDA_PMID(CLUSTER_STAT,75),
     /* disk.all.total_rawactive	*/   PMDA_PMID(CLUSTER_STAT,80),
+    /* disk.all.discard */	     PMDA_PMID(CLUSTER_STAT,96),
+    /* disk.all.blkdiscard */	     PMDA_PMID(CLUSTER_STAT,97),
+    /* disk.all.discard_bytes */     PMDA_PMID(CLUSTER_STAT,98),
+    /* disk.all.discard_merge */     PMDA_PMID(CLUSTER_STAT,99),
+    /* disk.all.discard_rawactive */ PMDA_PMID(CLUSTER_STAT,100),
+    /* disk.all.flush */	     PMDA_PMID(CLUSTER_STAT,101),
+    /* disk.all.blkflush */	     PMDA_PMID(CLUSTER_STAT,102),
+    /* disk.all.flush_rawactive */   PMDA_PMID(CLUSTER_STAT,103),
 
     /* disk.partitions.read */	     PMDA_PMID(CLUSTER_PARTITIONS,0),
     /* disk.partitions.write */	     PMDA_PMID(CLUSTER_PARTITIONS,1),
@@ -621,6 +762,14 @@ static pmID disk_metric_table[] = {
     /* disk.partitions.write_rawactive */ PMDA_PMID(CLUSTER_PARTITIONS,14),
     /* disk.partitions.total_rawactive */ PMDA_PMID(CLUSTER_PARTITIONS,15),
     /* disk.partitions.capacity */   PMDA_PMID(CLUSTER_PARTITIONS,16),
+    /* disk.partitions.discard */    PMDA_PMID(CLUSTER_PARTITIONS,17),
+    /* disk.partitions.blkdiscard */ PMDA_PMID(CLUSTER_PARTITIONS,18),
+    /* disk.partitions.discard_bytes */   PMDA_PMID(CLUSTER_PARTITIONS,19),
+    /* disk.partitions.discard_merge */   PMDA_PMID(CLUSTER_PARTITIONS,20),
+    /* disk.partitions.discard_rawactive */ PMDA_PMID(CLUSTER_PARTITIONS,21),
+    /* disk.partitions.flush */      PMDA_PMID(CLUSTER_PARTITIONS,22),
+    /* disk.partitions.blkflush */   PMDA_PMID(CLUSTER_PARTITIONS,23),
+    /* disk.partitions.flush_rawactive */ PMDA_PMID(CLUSTER_PARTITIONS,24),
 
     /* disk.dm.read */               PMDA_PMID(CLUSTER_DM,0),
     /* disk.dm.write */		     PMDA_PMID(CLUSTER_DM,1),
@@ -640,6 +789,14 @@ static pmID disk_metric_table[] = {
     /* disk.dm.write_rawactive */    PMDA_PMID(CLUSTER_DM,15),
     /* disk.dm.total_rawactive */    PMDA_PMID(CLUSTER_DM,16),
     /* disk.dm.capacity */	     PMDA_PMID(CLUSTER_DM,17),
+    /* disk.dm.discard */	     PMDA_PMID(CLUSTER_DM,18),
+    /* disk.dm.blkdiscard */	     PMDA_PMID(CLUSTER_DM,19),
+    /* disk.dm.discard_bytes */      PMDA_PMID(CLUSTER_DM,20),
+    /* disk.dm.discard_merge */      PMDA_PMID(CLUSTER_DM,21),
+    /* disk.dm.discard_rawactive */  PMDA_PMID(CLUSTER_DM,22),
+    /* disk.dm.flush */	             PMDA_PMID(CLUSTER_DM,23),
+    /* disk.dm.blkflush */	     PMDA_PMID(CLUSTER_DM,24),
+    /* disk.dm.flush_rawactive */    PMDA_PMID(CLUSTER_DM,25),
 
     /* disk.md.read */               PMDA_PMID(CLUSTER_MD,0),
     /* disk.md.write */		     PMDA_PMID(CLUSTER_MD,1),
@@ -659,6 +816,40 @@ static pmID disk_metric_table[] = {
     /* disk.md.write_rawactive */    PMDA_PMID(CLUSTER_MD,15),
     /* disk.md.total_rawactive */    PMDA_PMID(CLUSTER_MD,16),
     /* disk.md.capacity */	     PMDA_PMID(CLUSTER_MD,17),
+    /* disk.md.discard */	     PMDA_PMID(CLUSTER_MD,18),
+    /* disk.md.blkdiscard */	     PMDA_PMID(CLUSTER_MD,19),
+    /* disk.md.discard_bytes */      PMDA_PMID(CLUSTER_MD,20),
+    /* disk.md.discard_merge */      PMDA_PMID(CLUSTER_MD,21),
+    /* disk.md.discard_rawactive */  PMDA_PMID(CLUSTER_MD,22),
+    /* disk.md.flush */	             PMDA_PMID(CLUSTER_MD,23),
+    /* disk.md.blkflush */	     PMDA_PMID(CLUSTER_MD,24),
+    /* disk.md.flush_rawactive */    PMDA_PMID(CLUSTER_MD,25),
+
+    /* zram.read */	             PMDA_PMID(CLUSTER_ZRAM_DEVICES,0),
+    /* zram.write */     	     PMDA_PMID(CLUSTER_ZRAM_DEVICES,1),
+    /* zram.total */	             PMDA_PMID(CLUSTER_ZRAM_DEVICES,2),
+    /* zram.blkread */               PMDA_PMID(CLUSTER_ZRAM_DEVICES,3),
+    /* zram.blkwrite */              PMDA_PMID(CLUSTER_ZRAM_DEVICES,4),
+    /* zram.blktotal */              PMDA_PMID(CLUSTER_ZRAM_DEVICES,5),
+    /* zram.read_bytes */            PMDA_PMID(CLUSTER_ZRAM_DEVICES,6),
+    /* zram.write_bytes */           PMDA_PMID(CLUSTER_ZRAM_DEVICES,7),
+    /* zram.total_bytes */           PMDA_PMID(CLUSTER_ZRAM_DEVICES,8),
+    /* zram.read_merge */            PMDA_PMID(CLUSTER_ZRAM_DEVICES,9),
+    /* zram.write_merge */           PMDA_PMID(CLUSTER_ZRAM_DEVICES,10),
+    /* zram.avactive */              PMDA_PMID(CLUSTER_ZRAM_DEVICES,11),
+    /* zram.aveq */                  PMDA_PMID(CLUSTER_ZRAM_DEVICES,12),
+    /* zram.read_rawactive */        PMDA_PMID(CLUSTER_ZRAM_DEVICES,13),
+    /* zram.write_rawactive */       PMDA_PMID(CLUSTER_ZRAM_DEVICES,14),
+    /* zram.total_rawactive */       PMDA_PMID(CLUSTER_ZRAM_DEVICES,15),
+    /* zram.capacity */              PMDA_PMID(CLUSTER_ZRAM_DEVICES,16),
+    /* zram.discard */               PMDA_PMID(CLUSTER_ZRAM_DEVICES,17),
+    /* zram.blkdiscard */            PMDA_PMID(CLUSTER_ZRAM_DEVICES,18),
+    /* zram.discard_bytes */         PMDA_PMID(CLUSTER_ZRAM_DEVICES,19),
+    /* zram.discard_merge */         PMDA_PMID(CLUSTER_ZRAM_DEVICES,20),
+    /* zram.discard_rawactive */     PMDA_PMID(CLUSTER_ZRAM_DEVICES,21),
+    /* zram.flush */                 PMDA_PMID(CLUSTER_ZRAM_DEVICES,22),
+    /* zram.blkflush */              PMDA_PMID(CLUSTER_ZRAM_DEVICES,23),
+    /* zram.flush_rawactive */       PMDA_PMID(CLUSTER_ZRAM_DEVICES,24),
 };
 
 int
@@ -778,7 +969,7 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	/*
 	 * disk.{dev,all} remain in CLUSTER_STAT for backward compatibility
 	 */
-	switch(item) {
+	switch (item) {
 	case 4: /* disk.dev.read */
 	    if (p == NULL)
 		return PM_ERR_INST;
@@ -812,17 +1003,17 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	case 38: /* disk.dev.read_bytes */
 	    if (p == NULL)
 		return PM_ERR_INST;
-	    atom->ul = p->rd_sectors / 2;
+	    _pm_assign_ulong(atom, p->rd_sectors / 2);
 	    break;
 	case 39: /* disk.dev.write_bytes */
 	    if (p == NULL)
 		return PM_ERR_INST;
-	    atom->ul = p->wr_sectors / 2;
+	    _pm_assign_ulong(atom, p->wr_sectors / 2);
 	    break;
 	case 40: /* disk.dev.total_bytes */
 	    if (p == NULL)
 		return PM_ERR_INST;
-	    atom->ul = (p->rd_sectors + p->wr_sectors) / 2;
+	    _pm_assign_ulong(atom, (p->rd_sectors + p->wr_sectors) / 2);
 	    break;
 	case 46: /* disk.dev.avactive ... already msec from /proc/diskstats */
 	    if (p == NULL)
@@ -871,6 +1062,41 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		return 0;
 	    atom->ull = p->nr_blocks;
 	    break;
+	case 88: /* disk.dev.discard */
+	    if (p == NULL)
+		return PM_ERR_INST;
+	    _pm_assign_ulong(atom, p->ds_ios);
+	    break;
+	case 89: /* disk.dev.blkdiscard */
+	    if (p == NULL)
+		return PM_ERR_INST;
+	    _pm_assign_ulong(atom, p->ds_sectors);
+	    break;
+	case 90: /* disk.dev.discard_bytes */
+	    if (p == NULL)
+		return PM_ERR_INST;
+	    _pm_assign_ulong(atom, p->ds_sectors / 2);
+	    break;
+	case 91: /* disk.dev.discard_merge */
+	    if (p == NULL)
+		return PM_ERR_INST;
+	    _pm_assign_ulong(atom, p->ds_merges);
+	    break;
+	case 92: /* disk.dev.discard_rawactive already ms from /proc/diskstats */
+	    if (p == NULL)
+		return PM_ERR_INST;
+	    atom->ul = p->ds_ticks;
+	    break;
+	case 93: /* disk.dev.flush */
+	    if (p == NULL)
+		return PM_ERR_INST;
+	    _pm_assign_ulong(atom, p->fl_ios);
+	    break;
+	case 94: /* disk.dev.flush_rawactive already ms from /proc/diskstats */
+	    if (p == NULL)
+		return PM_ERR_INST;
+	    atom->ul = p->fl_ticks;
+	    break;
 	default:
 	    /* disk.all.* is a singular instance domain */
 	    atom->ull = 0;
@@ -899,13 +1125,13 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		    atom->ull += p->rd_sectors + p->wr_sectors;
 		    break;
 		case 41: /* disk.all.read_bytes */
-		    atom->ul += p->rd_sectors / 2;
+		    _pm_append_ulong(atom, p->rd_sectors / 2);
 		    break;
 		case 42: /* disk.all.write_bytes */
-		    atom->ul += p->wr_sectors / 2;
+		    _pm_append_ulong(atom, p->wr_sectors / 2);
 		    break;
 		case 43: /* disk.all.total_bytes */
-		    atom->ul += (p->rd_sectors + p->wr_sectors) / 2;
+		    _pm_append_ulong(atom, (p->rd_sectors + p->wr_sectors) / 2);
 		    break;
 		case 44: /* disk.all.avactive ... already msec from /proc/diskstats */
 		    atom->ull += p->io_ticks;
@@ -928,6 +1154,27 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		case 80: /* disk.all.total_rawactive ... already msec from /proc/diskstats */
 		    atom->ull += p->rd_ticks + p->wr_ticks;
 		    break;
+		case 96: /* disk.all.discard */
+		    _pm_append_ulong(atom, p->ds_ios);
+		    break;
+		case 97: /* disk.all.blkdiscard */
+		    _pm_append_ulong(atom, p->ds_sectors);
+		    break;
+		case 98: /* disk.all.discard_bytes */
+		    _pm_append_ulong(atom, p->ds_sectors / 2);
+		    break;
+		case 99: /* disk.all.discard_merge */
+		    _pm_append_ulong(atom, p->ds_merges);
+		    break;
+		case 100: /* disk.all.discard_rawactive ... already msec from /proc/diskstats */
+		    atom->ul += p->ds_ticks;
+		    break;
+		case 101: /* disk.all.flush */
+		    _pm_append_ulong(atom, p->fl_ios);
+		    break;
+		case 102: /* disk.all.flush_rawactive ... already msec from /proc/diskstats */
+		    atom->ul += p->fl_ticks;
+		    break;
 		default:
 		    return PM_ERR_PMID;
 		}
@@ -936,85 +1183,100 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	break;
 
     case CLUSTER_PARTITIONS:
+    case CLUSTER_ZRAM_DEVICES:
 	if (p == NULL)
 	    return PM_ERR_INST;
-	switch(item) {
-	    /* disk.partitions */
-	    case 0: /* disk.partitions.read */
+	switch (item) {
+	    case 0: /* {disk.partitions,zram}.read */
 		_pm_assign_ulong(atom, p->rd_ios);
 		break;
-	    case 1: /* disk.partitions.write */
+	    case 1: /* {disk.partitions,zram}.write */
 		_pm_assign_ulong(atom, p->wr_ios);
 		break;
-	    case 2: /* disk.partitions.total */
-		atom->ul = p->wr_ios + p->rd_ios;
+	    case 2: /* {disk.partitions,zram}.total */
+		_pm_assign_ulong(atom, p->wr_ios + p->rd_ios);
 		break;
-	    case 3: /* disk.partitions.blkread */
+	    case 3: /* {disk.partitions,zram}.blkread */
 		_pm_assign_ulong(atom, p->rd_sectors);
 		break;
-	    case 4: /* disk.partitions.blkwrite */
+	    case 4: /* {disk.partitions,zram}.blkwrite */
 		_pm_assign_ulong(atom, p->wr_sectors);
 		break;
-	    case 5: /* disk.partitions.blktotal */
-		atom->ul = p->rd_sectors + p->wr_sectors;
+	    case 5: /* {disk.partitions,zram}.blktotal */
+		_pm_assign_ulong(atom, p->rd_sectors + p->wr_sectors);
 		break;
-	    case 6: /* disk.partitions.read_bytes */
-		atom->ul = p->rd_sectors / 2;
+	    case 6: /* {disk.partitions,zram}.read_bytes */
+		_pm_assign_ulong(atom, p->rd_sectors / 2);
 		break;
-	    case 7: /* disk.partitions.write_bytes */
-		atom->ul = p->wr_sectors / 2;
+	    case 7: /* {disk.partitions,zram}.write_bytes */
+		_pm_assign_ulong(atom, p->wr_sectors / 2);
 		break;
-	    case 8: /* disk.partitions.total_bytes */
-		atom->ul = (p->rd_sectors +
-			   p->wr_sectors) / 2;
+	    case 8: /* {disk.partitions,zram}.total_bytes */
+		_pm_assign_ulong(atom, (p->rd_sectors + p->wr_sectors) / 2);
 		break;
-            case 9: /* disk.partitions.read_merge */
+            case 9: /* {disk.partitions,zram}.read_merge */
                 if (_pm_have_kernel_2_6_partition_stats)
                     return PM_ERR_APPVERSION; /* no read_merge for partition in 2.6 */
-                else
-                    _pm_assign_ulong(atom, p->rd_merges);
+                _pm_assign_ulong(atom, p->rd_merges);
                 break;
-            case 10: /* disk.partitions.write_merge */
+            case 10: /* {disk.partitions,zram}.write_merge */
                 if (_pm_have_kernel_2_6_partition_stats)
                     return PM_ERR_APPVERSION; /* no write_merge for partition in 2.6 */
-                else
-                    _pm_assign_ulong(atom, p->wr_merges);
+                _pm_assign_ulong(atom, p->wr_merges);
                 break;
-            case 11: /* disk.partitions.avactive */
+            case 11: /* {disk.partitions,zram}.avactive */
                 if (_pm_have_kernel_2_6_partition_stats)
                     return PM_ERR_APPVERSION; /* no avactive for partition in 2.6 */
-                else
-                    atom->ul = p->io_ticks;
+                atom->ul = p->io_ticks;
                 break;
-            case 12: /* disk.partitions.aveq */
+            case 12: /* {disk.partitions,zram}.aveq */
                 if (_pm_have_kernel_2_6_partition_stats)
                     return PM_ERR_APPVERSION; /* no aveq for partition in 2.6 */
-                else
-                    atom->ul = p->aveq;
+                atom->ul = p->aveq;
                 break;
-            case 13: /* disk.partitions.read_rawactive */
+            case 13: /* {disk.partitions,zram}.read_rawactive */
                 if (_pm_have_kernel_2_6_partition_stats)
                     return PM_ERR_APPVERSION; /* no read_rawactive for partition in 2.6 */
-                else
-                    atom->ul = p->rd_ticks;
+                atom->ul = p->rd_ticks;
                 break;
-            case 14: /* disk.partitions.write_rawactive */
+            case 14: /* {disk.partitions,zram}.write_rawactive */
                 if (_pm_have_kernel_2_6_partition_stats)
                     return PM_ERR_APPVERSION; /* no write_rawactive for partition in 2.6 */
-                else
-                    atom->ul = p->wr_ticks;
+                atom->ul = p->wr_ticks;
                 break;
-            case 15: /* disk.partitions.total_rawactive */
+            case 15: /* {disk.partitions,zram}.total_rawactive */
                 if (_pm_have_kernel_2_6_partition_stats)
                     return PM_ERR_APPVERSION; /* no read_rawactive or write_rawactive for partition in 2.6 */
-                else
-                    atom->ul = p->rd_ticks + p->wr_ticks;
+                atom->ul = p->rd_ticks + p->wr_ticks;
                 break;
-	    case 16: /* disk.partitions.capacity */
-		if (!p->nr_blocks)
+	    case 16: /* {disk.partitions,zram}.capacity */
+		if (cluster == CLUSTER_ZRAM_DEVICES)
+		    refresh_zram_size(p->namebuf, p->zram, &p->nr_blocks);
+		else if (!p->nr_blocks)
 		    return 0;
 		atom->ull = p->nr_blocks;
 		break;
+	    case 17: /* {disk.partitions,zram}.discard */
+		_pm_assign_ulong(atom, p->ds_ios);
+		break;
+	    case 18: /* {disk.partitions,zram}.blkdiscard */
+		_pm_assign_ulong(atom, p->ds_sectors);
+		break;
+	    case 19: /* {disk.partitions,zram}.discard_bytes */
+		_pm_assign_ulong(atom, p->ds_sectors / 2);
+		break;
+	    case 20: /* {disk.partitions,zram}.discard_merge */
+		_pm_assign_ulong(atom, p->ds_merges);
+		break;
+	    case 21: /* {disk.partitions,zram}.discard_rawactive */
+		atom->ul = p->ds_ticks;
+		break;
+	    case 22: /* {disk.partitions,zram}.flush */
+		_pm_assign_ulong(atom, p->fl_ios);
+		break;
+	    case 23: /* {disk.partitions,zram}.flush_rawactive */
+		atom->ul = p->fl_ticks;
+		break;
 	    default:
 		return PM_ERR_PMID;
 	}
@@ -1024,7 +1286,7 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     case CLUSTER_MD:
 	if (p == NULL)
 	    return PM_ERR_INST;
-	switch(item) {
+	switch (item) {
 	case 0: /* disk.{dm,md}.read */
 	    _pm_assign_ulong(atom, p->rd_ios);
 	    break;
@@ -1081,6 +1343,27 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		return 0;
 	    atom->ull = p->nr_blocks;
 	    break;
+	case 18: /* disk.{dm,md}.discard */
+	    _pm_assign_ulong(atom, p->ds_ios);
+	    break;
+	case 19: /* disk.{dm,md}.blkdiscard */
+	    _pm_assign_ulong(atom, p->ds_sectors);
+	    break;
+	case 20: /* disk.{dm,md}.discard_bytes */
+	    _pm_assign_ulong(atom, p->ds_sectors / 2);
+	    break;
+	case 21: /* disk.{dm,md}.discard_merge */
+	    _pm_assign_ulong(atom, p->ds_merges);
+	    break;
+	case 22: /* disk.{dm,md}.discard_rawactive */
+	    atom->ul = p->ds_ticks;
+	    break;
+	case 23: /* disk.{dm,md}.flush */
+	    _pm_assign_ulong(atom, p->fl_ios);
+	    break;
+	case 24: /* disk.{dm,md}.flush_rawactive */
+	    atom->ul = p->fl_ticks;
+	    break;
 	default:
 	    return PM_ERR_PMID;
 	}
@@ -1089,7 +1372,7 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     case CLUSTER_MDADM:
 	if (p == NULL)
 	    return PM_ERR_INST;
-	switch(item) {
+	switch (item) {
 	case 0: /* disk.md.status */
 	    atom->l = refresh_mdadm(p->namebuf);
 	    break;
@@ -1098,6 +1381,85 @@ proc_partitions_fetch(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	}
 	break;
 
+    case CLUSTER_ZRAM_IO_STAT:
+	if (p == NULL)
+	    return PM_ERR_INST;
+	if (refresh_zram_io_stat(p->namebuf, p->zram) < 0)
+	    return 0;
+	switch (item) {
+	case 0: /* zram.io_stat.failed.reads */
+	    atom->ull = p->zram->iostat.failed_reads;
+	    break;
+	case 1: /* zram.io_stat.failed.writes */
+	    atom->ull = p->zram->iostat.failed_writes;
+	    break;
+	case 2: /* zram.io_stat.invalid */
+	    atom->ull = p->zram->iostat.invalid_io;
+	    break;
+	case 3: /* zram.io_stat.notify_free */
+	    atom->ull = p->zram->iostat.notify_free;
+	    break;
+	default:
+	    return PM_ERR_PMID;
+	}
+	break;
+
+    case CLUSTER_ZRAM_MM_STAT:
+	if (p == NULL)
+	    return PM_ERR_INST;
+	if (refresh_zram_mm_stat(p->namebuf, p->zram) < 0)
+	    return 0;
+	switch (item) {
+	case 0: /* zram.mm_stat.data_size.original */
+	    atom->ull = p->zram->mmstat.original >> 10;		/* bytes to KB */
+	    break;
+	case 1: /* zram.mm_stat.data_size.compressed */
+	    atom->ull = p->zram->mmstat.compressed >> 10;	/* bytes to KB */
+	    break;
+	case 2: /* zram.mm_stat.mem.used_total */
+	    atom->ull = p->zram->mmstat.mem_used >> 10;		/* bytes to KB */
+	    break;
+	case 3: /* zram.mm_stat.mem.limit */
+	    atom->ull = p->zram->mmstat.mem_limit >> 10;	/* bytes to KB */
+	    break;
+	case 4: /* zram.mm_stat.mem.max_used */
+	    atom->ull = p->zram->mmstat.max_used >> 10;		/* bytes to KB */
+	    break;
+	case 5: /* zram.mm_stat.pages.same */
+	    atom->ull = p->zram->mmstat.same_pages << 2;	/* 4k to 1k */
+	    break;
+	case 6: /* zram.mm_stat.pages.compacted */
+	    atom->ull = p->zram->mmstat.compacted_pages << 2;	/* 4k to 1k */
+	    break;
+	case 7: /* zram.mm_stat.pages.huge */
+	    atom->ull = p->zram->mmstat.huge_pages << 2;	/* 4k to 1k */
+	    break;
+	default:
+	    return PM_ERR_PMID;
+	}
+	break;
+
+    case CLUSTER_ZRAM_BD_STAT:
+	if (p == NULL)
+	    return PM_ERR_INST;
+	if (refresh_zram_bd_stat(p->namebuf, p->zram) < 0)
+	    return 0;
+	switch (item) {
+	case 0: /* zram.bd_stat.count */
+	    atom->ull = p->zram->bdstat.count << 2;		/* 4k to 1k */
+	    break;
+	case 1: /* zram.bd_stat.reads */
+	    atom->ull = p->zram->bdstat.reads << 2;		/* 4k to 1k */
+	    break;
+	case 2: /* zram.bd_stat.writes */
+	    atom->ull = p->zram->bdstat.writes << 2;		/* 4k to 1k */
+	    break;
+	default:
+	    return PM_ERR_PMID;
+	}
+	break;
+
+
     default: /* switch cluster */
 	return PM_ERR_PMID;
     }
