@@ -1,7 +1,7 @@
 /*
  * Linux /proc/vmstat metrics cluster
  *
- * Copyright (c) 2013-2014,2016-2017 Red Hat.
+ * Copyright (c) 2013-2014,2016-2017,2020 Red Hat.
  * Copyright (c) 2007,2011 Aconex.  All Rights Reserved.
  * Copyright (c) 2004 Silicon Graphics, Inc.  All Rights Reserved.
  * 
@@ -26,6 +26,16 @@ static struct {
     /* sorted by name to make maintenance easier */
     { .field = "allocstall",
      .offset = &_pm_proc_vmstat.allocstall },
+    { .field = "allocstall_dma",
+     .offset = &_pm_proc_vmstat.allocstall_dma },
+    { .field = "allocstall_dma32",
+     .offset = &_pm_proc_vmstat.allocstall_dma32 },
+    { .field = "allocstall_high",
+     .offset = &_pm_proc_vmstat.allocstall_high },
+    { .field = "allocstall_movable",
+     .offset = &_pm_proc_vmstat.allocstall_movable },
+    { .field = "allocstall_normal",
+     .offset = &_pm_proc_vmstat.allocstall_normal },
     { .field = "balloon_deflate",
      .offset = &_pm_proc_vmstat.balloon_deflate },
     { .field = "balloon_inflate",
@@ -34,6 +44,10 @@ static struct {
      .offset = &_pm_proc_vmstat.balloon_migrate },
     { .field = "compact_blocks_moved",
      .offset = &_pm_proc_vmstat.compact_blocks_moved },
+    { .field = "compact_daemon_free_scanned",
+     .offset = &_pm_proc_vmstat.compact_daemon_free_scanned },
+    { .field = "compact_daemon_migrate_scanned",
+     .offset = &_pm_proc_vmstat.compact_daemon_migrate_scanned },
     { .field = "compact_daemon_wake",
      .offset = &_pm_proc_vmstat.compact_daemon_wake },
     { .field = "compact_fail",
@@ -51,11 +65,11 @@ static struct {
     { .field = "compact_stall",
      .offset = &_pm_proc_vmstat.compact_stall },
     { .field = "compact_success",
-     .offset = &_pm_proc_vmstat.drop_pagecache },
-    { .field = "drop_pagecache",
-     .offset = &_pm_proc_vmstat.drop_slab },
-    { .field = "drop_slab",
      .offset = &_pm_proc_vmstat.compact_success },
+    { .field = "drop_pagecache",
+     .offset = &_pm_proc_vmstat.drop_pagecache },
+    { .field = "drop_slab",
+     .offset = &_pm_proc_vmstat.drop_slab },
     { .field = "htlb_buddy_alloc_fail",
      .offset = &_pm_proc_vmstat.htlb_buddy_alloc_fail },
     { .field = "htlb_buddy_alloc_success",
@@ -80,16 +94,24 @@ static struct {
      .offset = &_pm_proc_vmstat.nr_anon_transparent_hugepages },
     { .field = "nr_bounce",
      .offset = &_pm_proc_vmstat.nr_bounce },
-    { .field = "nr_dirty",
-     .offset = &_pm_proc_vmstat.nr_dirty },
     { .field = "nr_dirtied",
      .offset = &_pm_proc_vmstat.nr_dirtied },
-    { .field = "nr_dirty_threshold",
-     .offset = &_pm_proc_vmstat.nr_dirty_threshold },
+    { .field = "nr_dirty",
+     .offset = &_pm_proc_vmstat.nr_dirty },
     { .field = "nr_dirty_background_threshold",
      .offset = &_pm_proc_vmstat.nr_dirty_background_threshold },
+    { .field = "nr_dirty_threshold",
+     .offset = &_pm_proc_vmstat.nr_dirty_threshold },
+    { .field = "nr_file_hugepages",
+     .offset = &_pm_proc_vmstat.nr_file_hugepages },
     { .field = "nr_file_pages",
      .offset = &_pm_proc_vmstat.nr_file_pages },
+    { .field = "nr_file_pmdmapped",
+     .offset = &_pm_proc_vmstat.nr_file_pmdmapped },
+    { .field = "nr_foll_pin_acquired",
+     .offset = &_pm_proc_vmstat.nr_foll_pin_acquired },
+    { .field = "nr_foll_pin_released",
+     .offset = &_pm_proc_vmstat.nr_foll_pin_released },
     { .field = "nr_free_cma",
      .offset = &_pm_proc_vmstat.nr_free_cma },
     { .field = "nr_free_pages",
@@ -102,6 +124,8 @@ static struct {
      .offset = &_pm_proc_vmstat.nr_isolated_anon },
     { .field = "nr_isolated_file",
      .offset = &_pm_proc_vmstat.nr_isolated_file },
+    { .field = "nr_kernel_misc_reclaimable",
+     .offset = &_pm_proc_vmstat.nr_kernel_misc_reclaimable },
     { .field = "nr_kernel_stack",
      .offset = &_pm_proc_vmstat.nr_kernel_stack },
     { .field = "nr_mapped",
@@ -138,14 +162,14 @@ static struct {
      .offset = &_pm_proc_vmstat.nr_writeback_temp },
     { .field = "nr_written",
      .offset = &_pm_proc_vmstat.nr_written },
-    { .field = "nr_zone_inactive_anon",
-     .offset = &_pm_proc_vmstat.nr_zone_inactive_anon },
     { .field = "nr_zone_active_anon",
      .offset = &_pm_proc_vmstat.nr_zone_active_anon },
-    { .field = "nr_zone_inactive_file",
-     .offset = &_pm_proc_vmstat.nr_zone_inactive_file },
     { .field = "nr_zone_active_file",
      .offset = &_pm_proc_vmstat.nr_zone_active_file },
+    { .field = "nr_zone_inactive_anon",
+     .offset = &_pm_proc_vmstat.nr_zone_inactive_anon },
+    { .field = "nr_zone_inactive_file",
+     .offset = &_pm_proc_vmstat.nr_zone_inactive_file },
     { .field = "nr_zone_unevictable",
      .offset = &_pm_proc_vmstat.nr_zone_unevictable },
     { .field = "nr_zone_write_pending",
@@ -174,6 +198,8 @@ static struct {
      .offset = &_pm_proc_vmstat.numa_pages_migrated },
     { .field = "numa_pte_updates",
      .offset = &_pm_proc_vmstat.numa_pte_updates },
+    { .field = "oom_kill",
+     .offset = &_pm_proc_vmstat.oom_kill },
     { .field = "pageoutrun",
      .offset = &_pm_proc_vmstat.pageoutrun },
     { .field = "pgactivate",
@@ -260,6 +286,8 @@ static struct {
      .offset = &_pm_proc_vmstat.pgsteal_dma },
     { .field = "pgsteal_dma32",
      .offset = &_pm_proc_vmstat.pgsteal_dma32 },
+    { .field = "pgsteal_file",
+     .offset = &_pm_proc_vmstat.pgsteal_file },
     { .field = "pgsteal_high",
      .offset = &_pm_proc_vmstat.pgsteal_high },
     { .field = "pgsteal_kswapd",
@@ -282,6 +310,10 @@ static struct {
      .offset = &_pm_proc_vmstat.pswpout },
     { .field = "slabs_scanned",
      .offset = &_pm_proc_vmstat.slabs_scanned },
+    { .field = "swap_ra",
+     .offset = &_pm_proc_vmstat.swap_ra },
+    { .field = "swap_ra_hit",
+     .offset = &_pm_proc_vmstat.swap_ra_hit },
     { .field = "thp_collapse_alloc",
      .offset = &_pm_proc_vmstat.thp_collapse_alloc },
     { .field = "thp_collapse_alloc_failed",
@@ -292,6 +324,8 @@ static struct {
      .offset = &_pm_proc_vmstat.thp_fault_alloc },
     { .field = "thp_fault_fallback",
      .offset = &_pm_proc_vmstat.thp_fault_fallback },
+    { .field = "thp_fault_fallback_charge",
+     .offset = &_pm_proc_vmstat.thp_fault_fallback_charge },
     { .field = "thp_file_alloc",
      .offset = &_pm_proc_vmstat.thp_file_alloc },
     { .field = "thp_file_mapped",
@@ -304,6 +338,12 @@ static struct {
      .offset = &_pm_proc_vmstat.thp_split_page_failed },
     { .field = "thp_split_pmd",
      .offset = &_pm_proc_vmstat.thp_split_pmd },
+    { .field = "thp_split_pud",
+     .offset = &_pm_proc_vmstat.thp_split_pud },
+    { .field = "thp_swpout",
+     .offset = &_pm_proc_vmstat.thp_swpout },
+    { .field = "thp_swpout_fallback",
+     .offset = &_pm_proc_vmstat.thp_swpout_fallback },
     { .field = "thp_zero_page_alloc",
      .offset = &_pm_proc_vmstat.thp_zero_page_alloc },
     { .field = "thp_zero_page_alloc_failed",
@@ -328,8 +368,12 @@ static struct {
      .offset = &_pm_proc_vmstat.workingset_activate },
     { .field = "workingset_nodereclaim",
      .offset = &_pm_proc_vmstat.workingset_nodereclaim },
+    { .field = "workingset_nodes",
+     .offset = &_pm_proc_vmstat.workingset_nodes },
     { .field = "workingset_refault",
      .offset = &_pm_proc_vmstat.workingset_refault },
+    { .field = "workingset_restore",
+     .offset = &_pm_proc_vmstat.workingset_restore },
     { .field = "zone_reclaim_failed",
      .offset = &_pm_proc_vmstat.zone_reclaim_failed },
 
