@@ -81,7 +81,7 @@ static pmdaMetric metrictab[] = {
       { PMDA_PMID(CL_SYSCTL,0), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
 	PMDA_PMUNITS(0,0,0,0,0,0) } },
     { (void *)"hinv.physmem",
-      { PMDA_PMID(CL_SYSCTL,1), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_DISCRETE,
+      { PMDA_PMID(CL_SYSCTL,1), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_MBYTE,0,0) } },
     { NULL,	/* kernel.all.load */
       { PMDA_PMID(CL_SPECIAL,2), PM_TYPE_FLOAT, LOADAV_INDOM, PM_SEM_INSTANT,
@@ -101,6 +101,9 @@ static pmdaMetric metrictab[] = {
     { (void *)"kernel.all.cpu.idle",
       { PMDA_PMID(CL_SYSCTL,7), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER, 
 	PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) } },
+    { (void *)"kernel.all.cpu.spin",
+      { PMDA_PMID(CL_SYSCTL,8), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER, 
+	PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) } },
     { NULL, 	/* kernel.percpu.cpu.user */
       { PMDA_PMID(CL_CPUTIME,3), PM_TYPE_U64, CPU_INDOM, PM_SEM_COUNTER, 
 	PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) } },
@@ -116,6 +119,9 @@ static pmdaMetric metrictab[] = {
     { NULL, 	/* kernel.percpu.cpu.idle */
       { PMDA_PMID(CL_CPUTIME,7), PM_TYPE_U64, CPU_INDOM, PM_SEM_COUNTER, 
 	PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) } },
+    { NULL, 	/* kernel.percpu.cpu.spin */
+      { PMDA_PMID(CL_CPUTIME,8), PM_TYPE_U64, CPU_INDOM, PM_SEM_COUNTER, 
+	PMDA_PMUNITS(0,1,0,0,PM_TIME_MSEC,0) } },
     { (void *)"kernel.all.hz",
       { PMDA_PMID(CL_SYSCTL,13), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
 	PMDA_PMUNITS(0,1,0,0,PM_TIME_USEC,0) } },
@@ -185,26 +191,23 @@ static pmdaMetric metrictab[] = {
     { NULL, 	/* mem.util.inactive */
       { PMDA_PMID(CL_VM_UVMEXP,11), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
-    { NULL, 	/* mem.util.avail */
+    { NULL,	/* mem.util.zeropages */
       { PMDA_PMID(CL_VM_UVMEXP,12), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
-    { NULL,	/* mem.util.zeropages */
+    { NULL,	/* mem.util.pagedaemonpages */
       { PMDA_PMID(CL_VM_UVMEXP,13), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
-    { NULL,	/* mem.util.pagedaemonpages */
+    { NULL,	/* mem.util.kernelpages */
       { PMDA_PMID(CL_VM_UVMEXP,14), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
-    { NULL,	/* mem.util.kernelpages */
-      { PMDA_PMID(CL_VM_UVMEXP,15), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
-    { NULL,	/* mem.util.anonpages */
-      { PMDA_PMID(CL_VM_UVMEXP,16), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
+    { NULL,	/* mem.util.anonpages ... has gone away */
+      { PMDA_PMID(CL_VM_UVMEXP,15), PM_TYPE_NOSUPPORT, PM_INDOM_NULL, PM_SEM_INSTANT,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* mem.util.vnodepages */
-      { PMDA_PMID(CL_VM_UVMEXP,17), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
+      { PMDA_PMID(CL_VM_UVMEXP,16), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* mem.util.vtextpages */
-      { PMDA_PMID(CL_VM_UVMEXP,18), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
+      { PMDA_PMID(CL_VM_UVMEXP,17), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT,
 	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
 
     { NULL,	/* disk.dev.read */
@@ -218,13 +221,13 @@ static pmdaMetric metrictab[] = {
 	PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
     { NULL,	/* disk.dev.read_bytes */
       { PMDA_PMID(CL_DISK,3), PM_TYPE_U64, DISK_INDOM, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.dev.write_bytes */
       { PMDA_PMID(CL_DISK,4), PM_TYPE_U64, DISK_INDOM, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.dev.total_bytes */
       { PMDA_PMID(CL_DISK,5), PM_TYPE_U64, DISK_INDOM, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.all.read */
       { PMDA_PMID(CL_DISK,6), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
 	PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
@@ -236,13 +239,13 @@ static pmdaMetric metrictab[] = {
 	PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) } },
     { NULL,	/* disk.all.read_bytes */
       { PMDA_PMID(CL_DISK,9), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.all.write_bytes */
       { PMDA_PMID(CL_DISK,10), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
     { NULL,	/* disk.all.total_bytes */
       { PMDA_PMID(CL_DISK,11), PM_TYPE_U64, PM_INDOM_NULL, PM_SEM_COUNTER,
-	PMDA_PMUNITS(1,0,0,PM_SPACE_BYTE,0,0) } },
+	PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) } },
 
     { NULL,	/* network.interface.mtu */
       { PMDA_PMID(CL_NETIF,0), PM_TYPE_U64, NETIF_INDOM, PM_SEM_INSTANT,
@@ -351,6 +354,9 @@ static pmdaMetric metrictab[] = {
     { NULL,	/* kernel.uname.nodename */
       { PMDA_PMID(CL_SPECIAL,18), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE, 
       PMDA_PMUNITS(0,0,0,0,0,0) } },
+    { NULL,	/* kernel.all.uptime */
+      { PMDA_PMID(CL_SPECIAL,19), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_INSTANT, 
+      PMDA_PMUNITS(0,1,0,0,PM_TIME_SEC,0) } },
     { NULL,	/* pmda.uname */
       { PMDA_PMID(CL_SPECIAL,20), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE, 
       PMDA_PMUNITS(0,0,0,0,0,0) } },
@@ -443,16 +449,20 @@ sysctlnametomib(const char *name, int *mibp, size_t *sizep)
  * Fetch values from sysctl()
  *
  * Expect the result to be xpect bytes to match the PCP data size or
- * anticipated structure size, unless xpect is ==0 in which case the
+ * anticipated structure size, unless xpect is == 0 in which case the
  * size test is skipped.
  */
 static int
 do_sysctl(mib_t *mp, size_t xpect)
 {
     /*
-     * Note zero trip if mp->m_data and mp->datalen are already valid
+     * Note zero trip if mp->m_data and mp->m_datalen are already valid
      * and current
      */
+    if (pmDebugOptions.appl0 && pmDebugOptions.desperate) {
+	fprintf(stderr, "do_sysctl(%s, %d) m_data=%p m_datalen=%d m_fetched=%d\n", 
+	    mp->m_name, (int)xpect, mp->m_data, (int)mp->m_datalen, mp->m_fetched);
+    }
     for ( ; mp->m_fetched == 0; ) {
 	int	sts;
 	sts = sysctl(mp->m_mib, (u_int)mp->m_miblen, mp->m_data, &mp->m_datalen, NULL, 0);
@@ -476,57 +486,11 @@ do_sysctl(mib_t *mp, size_t xpect)
     }
     if (xpect > 0 && mp->m_datalen != xpect) {
 	fprintf(stderr, "Error: %s: sysctl(%s) datalen=%d not %d!\n", mp->m_pcpname, mp->m_name, (int)mp->m_datalen, (int)xpect);
-	return 0;
+	return -1;
     }
     return mp->m_datalen;
 }
 
-/*
- * kernel memory reader setup
- */
-struct nlist	symbols[] = {
-	{ .n_name = "_ifnet" },
-	{ .n_name = NULL }
-};
-kvm_t	*kvmp;
-
-static void
-kmemread_init(void)
-{
-    int		sts;
-    int		i;
-    char	errmsg[_POSIX2_LINE_MAX];
-
-    /*
-     * If we're running as a daemon PMDA, assume we're setgid kmem,
-     * so we can open /dev/kmem, and downgrade privileges after the
-     * kvm_open().
-     * For a DSO PMDA, we have to assume pmcd has the required
-     * privileges and don't dink with them.
-     */
-    kvmp = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errmsg);
-    if (!isDSO)
-	setgid(getgid());
-    if (kvmp == NULL) {
-	fprintf(stderr, "kmemread_init: kvm_openfiles failed: %s\n", errmsg);
-	return;
-    }
-
-    sts = kvm_nlist(kvmp, symbols);
-    if (sts < 0) {
-	fprintf(stderr, "kmemread_init: kvm_nlist failed: %s\n", pmErrStr(-errno));
-	for (i = 0; i < sizeof(symbols)/sizeof(symbols[0])-1; i++)
-	    symbols[i].n_value = 0;
-	return;
-    }
-    if (pmDebugOptions.appl0) {
-	for (i = 0; i < sizeof(symbols)/sizeof(symbols[0])-1; i++) {
-	    fprintf(stderr, "Info: kernel symbol %s found at 0x%08lx\n", symbols[i].n_name, symbols[i].n_value);
-	}
-    }
-
-}
-
 /*
  * Callback provided to pmdaFetch ... come here once per metric-instance
  * pair in each pmFetch().
@@ -539,6 +503,8 @@ openbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     unsigned int	item = pmID_item(mdesc->m_desc.pmid);
     mib_t		*mp;
     int			i;
+    int			pick;
+    struct timespec	now;
 
     mp = (mib_t *)mdesc->m_user;
     if (cluster == CL_SYSCTL) {
@@ -558,8 +524,7 @@ openbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		sts = do_sysctl(mp, sizeof(atom->ull));
 		if (sts > 0) {
 		    /* sysctl returns bytes, convert to Mbytes */
-		    atom->ull = *((__uint64_t *)mp->m_data);
-		    atom->ull /= 1024*1024;
+		    atom->ul = (*((long *)mp->m_data))/(1024*1024);
 		    sts = 1;
 		}
 		break;
@@ -576,22 +541,47 @@ openbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 
 	    /* structs and aggregates */
 	    case 3:		/* kernel.all.cpu.user */
+		pick = CP_USER;
+		goto xtract;
 	    case 4:		/* kernel.all.cpu.nice */
+		pick = CP_NICE;
+		goto xtract;
 	    case 5:		/* kernel.all.cpu.sys */
+		pick = CP_SYS;
+		goto xtract;
 	    case 6:		/* kernel.all.cpu.intr */
+		pick = CP_INTR;
+		goto xtract;
 	    case 7:		/* kernel.all.cpu.idle */
-		sts = do_sysctl(mp, CPUSTATES*sizeof(atom->ull));
-		if (sts > 0) {
-		    /*
-		     * PMID assignment is important in the "-3" below so
-		     * that metrics map to consecutive elements of the
-		     * returned value in the order defined for CPUSTATES,
-		     * i.e. CP_USER, CP_NICE, CP_SYS, CP_INTR and
-		     * CP_IDLE
-		     */
-		    atom->ull = 1000*((__uint64_t *)mp->m_data)[item-3]/cpuhz;
+		pick = CP_IDLE;
+		goto xtract;
+	    case 8:		/* kernel.all.cpu.spin */
+#ifdef CP_SPIN
+		pick = CP_SPIN;
+		goto xtract;
+#else
+		sts = 0;
+		break;
+#endif
+
+xtract:
+		sts = do_sysctl(mp, 0);
+		if (sts == CPUSTATES*sizeof(__uint64_t)) {
+		    /* 64-bit values from sysctl() */
+		    atom->ull = ncpu*1000*((__uint64_t)((__uint64_t *)mp->m_data)[pick])/cpuhz;
 		    sts = 1;
 		}
+		else if (sts == CPUSTATES*sizeof(__uint32_t)) {
+		    /* 32-bit values from sysctl() */
+		    atom->ull = ncpu*1000*((__uint64_t)((__uint32_t *)mp->m_data)[pick])/cpuhz;
+		    sts = 1;
+		}
+		else {
+		    fprintf(stderr, "Error: %s: sysctl(%s) datalen=%d not %d (long) or %d (int)!\n",
+			mp->m_pcpname, mp->m_name, sts, (int)(CPUSTATES*sizeof(__uint64_t)), 
+			(int)(CPUSTATES*sizeof(__uint32_t))); 
+		    sts = 0;
+		}
 		break;
 
 	    case 13:		/* kernel.all.hz */
@@ -607,8 +597,8 @@ openbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
     }
     else if (cluster == CL_SPECIAL) {
 	/* special cases */
-	double	loadavg[3];
-	char 	uname_string[sizeof(kernel_uname)];
+	double		loadavg[3];
+	static char 	uname_string[sizeof(kernel_uname)];
 
 	switch (item) {
 	    case 0:	/* hinv.ndisk */
@@ -665,6 +655,12 @@ openbsd_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		sts = 1;
 		break;
 
+	    case 19:	/* kernel.all.uptime */
+		clock_gettime(CLOCK_UPTIME, &now);
+		atom->ul = now.tv_sec;
+		sts = 1;
+		break;
+
 	    case 20: /* pmda.uname */
 		pmsprintf(uname_string, sizeof(uname_string), "%s %s %s %s %s",
 		    kernel_uname.sysname, 
@@ -894,12 +890,12 @@ openbsd_init(pmdaInterface *dp)
 		}
 		if (pmDebugOptions.appl0) {
 		    int	p;
-		    fprintf(stderr, "Info: %s (%s): sysctl metric \"%s\" -> ", (char *)metrictab[m].m_user, pmIDStr(metrictab[m].m_desc.pmid), map[i].m_name);
+		    fprintf(stderr, "Info: %s (%s): sysctl metric \"%s\" -> mib ", (char *)metrictab[m].m_user, pmIDStr(metrictab[m].m_desc.pmid), map[i].m_name);
 		    for (p = 0; p < map[i].m_miblen; p++) {
 			if (p > 0) fputc('.', stderr);
 			fprintf(stderr, "%d", map[i].m_mib[p]);
 		    }
-		    fputc('\n', stderr);
+		    fprintf(stderr, " (len=%d)\n", (int)map[i].m_miblen);
 		}
 		metrictab[m].m_user = (void *)&map[i];
 		break;
@@ -968,8 +964,6 @@ openbsd_init(pmdaInterface *dp)
 	    /*NOTREACHED*/
 	}
     }
-
-    kmemread_init();
 }
 
 static void
