@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2012 Red Hat.
  * Copyright (c) 1995-2003 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -14,7 +15,8 @@
 
 #include <ctype.h>
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
+#include "deprecated.h"
 #include "pmda.h"
 #include "shping.h"
 #include "domain.h"
@@ -119,6 +121,7 @@ logmessage(int priority, const char *format, ...)
     char        *level;
     char        *p;
     time_t      now;
+    int		bytes;
 
     buffer[0] = '\0';
     time(&now);
@@ -154,15 +157,19 @@ logmessage(int priority, const char *format, ...)
             break;
     }
 
-    va_start (arglist, format);
-    vsnprintf (buffer, sizeof(buffer), format, arglist);
+    va_start(arglist, format);
+    bytes = vsnprintf(buffer, sizeof(buffer), format, arglist);
+    va_end(arglist);
+    if (bytes >= sizeof(buffer))
+	buffer[sizeof(buffer)-1] = '\0';
+    else if (bytes < 0)
+	buffer[0] = '\0';
 
     /* strip unwanted '\n' at end of text so's not to double up */
     for (p = buffer; *p; p++);
     if (*(--p) == '\n') *p = '\0';
 
-    fprintf (stderr, "[%.19s] %s(%" FMT_PID ") %s: %s\n", ctime(&now), pmProgname, getpid(), level, buffer) ;
-    va_end (arglist) ;
+    fprintf(stderr, "[%.19s] %s(%" FMT_PID ") %s: %s\n", ctime(&now), pmGetProgname(), (pid_t)getpid(), level, buffer) ;
 }
 
 
@@ -182,10 +189,8 @@ onalarm(int dummy)
 	kill(-shpid, SIGKILL);
     }
     signal(SIGALRM, onalarm);
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL1)
+    if (pmDebugOptions.appl1)
 	fprintf(stderr, "Timeout!\n");
-#endif
 }
 
 /*
@@ -209,10 +214,13 @@ refresh(void *dummy)
     prctl(PR_TERMCHILD);          	/* SIGHUP when the parent dies */
 #elif defined (PR_SET_PDEATHSIG)
     prctl(PR_SET_PDEATHSIG, SIGTERM);
-#elif defined(IS_SOLARIS) || defined(IS_DARWIN) || defined(IS_MINGW) || defined(IS_AIX) || defined(IS_FREEBSD)
+#elif defined(IS_SOLARIS) || defined(IS_DARWIN) || defined(IS_MINGW) || defined(IS_AIX) || defined(IS_FREEBSD) || defined(IS_GNU) || defined(IS_NETBSD) || defined(IS_OPENBSD)
     /* no signals here for child exit */
 #else
 !bozo: cant decide between PR_TERMCHILD and PR_SET_PDEATHSIG
+#endif
+#ifndef NOFILE
+#define NOFILE 7
 #endif
 
     signal(SIGALRM, onalarm);
@@ -227,18 +235,14 @@ refresh(void *dummy)
 
     for ( ; ; ) {
 	cycles++;
-	__pmtimevalNow(&startcycle);
-#ifdef PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL1)
+	pmtimevalNow(&startcycle);
+	if (pmDebugOptions.appl1)
 	    fprintf(stderr, "\nStart cycle @ %s", ctime(&startcycle.tv_sec));
-#endif
 	for (i = 0; i < numcmd; i++) {
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL1)
+	    if (pmDebugOptions.appl1)
 		fprintf(stderr, "[%s] %s ->\n", cmdlist[i].tag, cmdlist[i].cmd);
-#endif
 	    getrusage(RUSAGE_CHILDREN, &cpu_then);
-	    __pmtimevalNow(&then);
+	    pmtimevalNow(&then);
 	    fflush(stderr);
 	    fflush(stdout);
 	    shpid = fork();
@@ -249,8 +253,7 @@ refresh(void *dummy)
 		    close(j);
 		open("/dev/null", O_RDONLY, 0);
 		sts = -1;
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL2) {
+		if (pmDebugOptions.appl2) {
 		    FILE	*f;
 		    if ((f = fopen("shping.out", "a")) != NULL) {
 			fprintf(f, "\n[%s] %s\n", cmdlist[i].tag, cmdlist[i].cmd);
@@ -258,7 +261,6 @@ refresh(void *dummy)
 		    }
 		    sts = open("shping.out", O_WRONLY|O_APPEND|O_CREAT, 0644);
 		}
-#endif
 		if (sts == -1)
 		    open("/dev/null", O_WRONLY, 0);
 		if (dup(1) == -1) {
@@ -279,7 +281,7 @@ refresh(void *dummy)
 	    timedout = 0;
 	    alarm(timeout);
 	    waitpid(shpid, &sts, 0);
-	    __pmtimevalNow(&now);
+	    pmtimevalNow(&now);
 	    getrusage(RUSAGE_CHILDREN, &cpu_now);
 	    alarm(0);
 
@@ -287,10 +289,8 @@ refresh(void *dummy)
 		cmdlist[i].error = PM_ERR_TIMEOUT;
 		cmdlist[i].status = STATUS_TIMEOUT;
 		cmdlist[i].real = cmdlist[i].usr = cmdlist[i].sys = -1;
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL0)
+		if (pmDebugOptions.appl0)
 		    logmessage(LOG_INFO, "[%s] timeout\n", cmdlist[i].tag);
-#endif
 	    }
 	    else {
 		if (WIFEXITED(sts)) {
@@ -299,31 +299,25 @@ refresh(void *dummy)
 			cmdlist[i].status = STATUS_OK;
 		    else {
 			cmdlist[i].status = STATUS_EXIT;
-#ifdef PCP_DEBUG
-			if (pmDebug & DBG_TRACE_APPL0)
+			if (pmDebugOptions.appl0)
 			    logmessage(LOG_INFO, "[%s] exit status: %d\n",
 				    cmdlist[i].tag, cmdlist[i].error);
-#endif
 		    }
 		}
 		else if (WIFSIGNALED(sts)) {
 		    cmdlist[i].error = WTERMSIG(sts);
 		    cmdlist[i].status = STATUS_SIG;
-#ifdef PCP_DEBUG
-		    if (pmDebug & DBG_TRACE_APPL0)
+		    if (pmDebugOptions.appl0)
 			logmessage(LOG_INFO, "[%s] killed signal: %d\n",
 				cmdlist[i].tag, cmdlist[i].error);
-#endif
 		}
 		else {
 		    /* assume WIFSTOPPED(sts) */
 		    cmdlist[i].error = WSTOPSIG(sts);
 		    cmdlist[i].status = STATUS_SIG;
-#ifdef PCP_DEBUG
-		    if (pmDebug & DBG_TRACE_APPL0)
+		    if (pmDebugOptions.appl0)
 			logmessage(LOG_INFO, "[%s] stopped signal: %d\n",
 				cmdlist[i].tag, cmdlist[i].error);
-#endif
 		}
 		cmdlist[i].real = 1000 * (float)(now.tv_sec - then.tv_sec) + 
 			    (float)(now.tv_usec - then.tv_usec) / 1000;
@@ -333,12 +327,10 @@ refresh(void *dummy)
 			    (float)(cpu_now.ru_stime.tv_usec - cpu_then.ru_stime.tv_usec) / 1000;
 	    }
 
-#ifdef PCP_DEBUG
-	    if (pmDebug & DBG_TRACE_APPL1)
+	    if (pmDebugOptions.appl1)
 		fprintf(stderr, "status: %d error: %d real: %.3f usr: %.3f sys: %.3f\n\n",
 		    cmdlist[i].status, cmdlist[i].error,
 		    cmdlist[i].real, cmdlist[i].usr, cmdlist[i].sys);
-#endif
 	}
 
 	/*
@@ -351,7 +343,7 @@ refresh(void *dummy)
 		break;
 	 }
 
-	__pmtimevalNow(&now);
+	pmtimevalNow(&now);
 	if (cycletime) {
 	    waittime = (int)cycletime - now.tv_sec + startcycle.tv_sec;
 	    if (waittime < 0) {
@@ -378,7 +370,6 @@ shping_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *ext)
     static pmResult	*res = NULL;
     static int		maxnpmids = 0;
     pmValueSet		*vset;
-    __pmID_int		*pmidp;
     pmAtomValue		atom;
     pmDesc		*dp = NULL;
     int			type;
@@ -417,16 +408,15 @@ shping_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *ext)
 
     for (i = 0; i < numpmid; i++) {
 
-    	pmidp = (__pmID_int*)&pmidlist[i];
 	dp = NULL;
 
 	if (ext->e_direct) {
- 	    if (pmidp->cluster == 0 && pmidp->item < nummetric)
-		dp = &metrics[pmidp->item].m_desc;
+ 	    if (pmID_cluster(pmidlist[i]) == 0 && pmID_item(pmidlist[i]) < nummetric)
+		dp = &metrics[pmID_item(pmidlist[i])].m_desc;
 	}
 	else {
 	    for (j = 1; j<nummetric; j++) {
-		if (pmidp->cluster == 0 && 
+		if (pmID_cluster(pmidlist[i]) == 0 && 
 		    metrics[j].m_desc.pmid == pmidlist[i]) {
 		    dp = &metrics[j].m_desc;
 		    break;
@@ -477,7 +467,6 @@ shping_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *ext)
 	}
 
 	type = dp->type;
-	pmidp = (__pmID_int *)&pmidlist[i];
 	j = 0;
 
 	do {
@@ -496,8 +485,8 @@ shping_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *ext)
 	    }
 	    vset->vlist[j].inst = inst;
 
-	    if (pmidp->cluster == 0) {
-		switch (pmidp->item) {
+	    if (pmID_cluster(pmidlist[i]) == 0) {
+		switch (pmID_item(pmidlist[i])) {
 
 		    case 0:	/* shping.time.real PMID: ...0.0 */
 			atom.f = cmdlist[inst].real;
@@ -574,13 +563,11 @@ shping_store(pmResult *result, pmdaExt *ext)
     pmValueSet	*vsp;
     int		sts = 0;
     int		ival;
-    __pmID_int	*pmidp;
 
     for (i = 0; i < result->numpmid; i++) {
 	vsp = result->vset[i];
-	pmidp = (__pmID_int *)&vsp->pmid;
-	if (pmidp->cluster == 0) {
-	    switch (pmidp->item) {
+	if (pmID_cluster(vsp->pmid) == 0) {
+	    switch (pmID_item(vsp->pmid)) {
 		case 4:	/* shping.control.cycletime PMID: ...0.4 */
 		    ival = vsp->vlist[0].value.lval;
 		    if (ival < 0) {
@@ -605,7 +592,8 @@ shping_store(pmResult *result, pmdaExt *ext)
 			sts = PM_ERR_SIGN;
 			break;
 		    }
-		    pmDebug = ival;
+		    pmClearDebug("all");
+		    __pmSetDebugBits(ival);
 		    break;
 
 		default:
@@ -651,8 +639,12 @@ shping_init(pmdaInterface *dp)
     }
     else {
 	dp->status = 0;
-        logmessage(LOG_INFO, "Started sproc (spid=%" FMT_PID ")\n", sprocpid);
+        logmessage(LOG_INFO, "Started sproc (spid=%" FMT_PID ")\n", (pid_t)sprocpid);
     }
+
+    /* we're talking to pmcd ... no timeout's for us thanks */
+    signal(SIGALRM, SIG_IGN);
+
 #elif defined (HAVE_PTHREAD_H)
     {
 	int err = pthread_create(&sprocpid, NULL, (void (*))refresh, NULL);
@@ -662,15 +654,10 @@ shping_init(pmdaInterface *dp)
 	    dp->status = err;
 	} else {
 	    dp->status = 0;
-	    logmessage (LOG_INFO, "Started thread %" FMT_PTHREAD "\n", sprocpid);
+	    logmessage (LOG_INFO, "Started refresh thread\n");
 	}
     }
 #else
 #error "Need pthreads or sproc"
 #endif
-
-    /* we're talking to pmcd ... no timeout's for us thanks */
-    signal(SIGALRM, SIG_IGN);
-
-    return;
 }
