@@ -10,10 +10,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -31,9 +27,8 @@ extern char	*helpfile;
 extern int	cmdpipe;
 extern pid_t	clientPID;
 
-extern void summaryMainLoop(char *, int, int, pmdaInterface *);
+extern void summaryMainLoop(char *, int, pmdaInterface *);
 extern void mainLoopFreeResultCallback(void (*)(pmResult *));
 extern void service_client(__pmPDU *);
-extern void service_config(__pmPDU *);
 
 
