@@ -29,6 +29,11 @@ PMIECONF="$PCP_BIN_DIR/pmieconf"
 #
 unset PCP_STDERR
 
+# ensure mere mortals cannot write any configuration files,
+# but that the unprivileged PCP_USER account has read access
+#
+umask 022
+
 # added to handle problem when /var/log/pcp is a symlink, as first
 # reported by Micah_Altman@harvard.edu in Nov 2001
 #
@@ -67,7 +72,7 @@ _cleanup()
     $USE_SYSLOG && [ $status -ne 0 ] && \
     $PCP_SYSLOG_PROG -p daemon.error "$prog failed - see $PROGLOG"
     lockfile=`cat $tmp/lock 2>/dev/null`
-    rm -f "$lockfile"
+    [ -n "$lockfile" ] && rm -f "$lockfile"
     rm -rf $tmp
     $VERY_VERBOSE && echo "End: `date '+%F %T.%N'`"
 }
@@ -206,9 +211,16 @@ then
     echo "Start: `date '+%F %T.%N'`"
     if which pstree >/dev/null 2>&1
     then
-	echo "Called from:"
-	pstree -spa $$
-	echo "--- end of pstree output ---"
+	if pstree -spa $$ >$tmp/tmp 2>&1
+	then
+	    echo "Called from:"
+	    cat $tmp/tmp
+	    echo "--- end of pstree output ---"
+	else
+	    # pstree not functional for us ... -s not supported in older
+	    # versions
+	    :
+	fi
     fi
 fi
 _error()
@@ -414,7 +426,7 @@ _configure_pmie()
 	    cp "$configfile" "$tmpconfig"
 	    if $PMIECONF -f "$tmpconfig" -cF >$tmp/diag 2>&1
 	    then
-		[ $isprimary = y ] && $PMIECONF -f "$tmpconfig" modify primary enabled yes
+		[ $isprimary = y ] && $PMIECONF -f "$tmpconfig" -c modify primary enabled yes
 		grep -v "generated by pmieconf" "$configfile" >$tmp/old
 		grep -v "generated by pmieconf" "$tmpconfig" >$tmp/new
 		if ! diff $tmp/old $tmp/new >/dev/null
@@ -435,6 +447,7 @@ _configure_pmie()
 		cat "$tmpconfig"
 		echo "=== end pmieconf file ==="
 	    fi
+	    rm -f "$tmpconfig"
 	fi
     elif [ ! -e "$configfile" ]
     then
@@ -450,7 +463,7 @@ _configure_pmie()
 	    cat "$configfile"
 	    echo "=== end pmieconf file ==="
 	else
-	    [ $isprimary = y ] && $PMIECONF -f "$configfile" modify primary enabled yes
+	    [ $isprimary = y ] && $PMIECONF -f "$configfile" -c modify primary enabled yes
             chown $PCP_USER:$PCP_GROUP "$configfile" >/dev/null 2>&1
 	fi
     fi
