@@ -34,6 +34,19 @@ PROGLOG=$PCP_LOG_DIR/pmlogger/$prog.log
 MYPROGLOG=$PROGLOG.$$
 USE_SYSLOG=true
 
+# optional begin logging to $PCP_LOG_DIR/NOTICES
+#
+if $PCP_LOG_RC_SCRIPTS
+then
+    logmsg="begin pid:$$ $prog args:$*"
+    if which pstree >/dev/null 2>&1
+    then
+	logmsg="$logmsg [`pstree -lps $$`]"
+	logmsg="`echo "$logmsg" | sed -e 's/---pstree([^)]*)//'`"
+    fi
+    $PCP_BINADM_DIR/pmpost "$logmsg"
+fi
+
 _cleanup()
 {
     if [ -s "$MYPROGLOG" ]
@@ -45,8 +58,14 @@ _cleanup()
     fi
     $USE_SYSLOG && [ $status -ne 0 ] && \
     $PCP_SYSLOG_PROG -p daemon.error "$prog failed - see $PROGLOG"
+    if $SHOWME
+    then
+	: no pid file created
+    else
+	rm -f "$PCP_RUN_DIR/pmlogger_daily.pid"
+    fi
     lockfile=`cat $tmp/lock 2>/dev/null`
-    rm -f "$lockfile" "$PCP_RUN_DIR/pmlogger_daily.pid"
+    [ -n "$lockfile" ] && rm -f "$lockfile"
     rm -rf $tmp
     $VERY_VERBOSE && echo >&2 "End: `date '+%F %T.%N'`"
 }
@@ -534,8 +553,15 @@ then
     echo >&2 "Start: `date '+%F %T.%N'`"
     if which pstree >/dev/null 2>&1
     then
-	echo >&2 "Called from:"
-	pstree >&2 -spa $$
+	if pstree -spa $$ >$tmp/tmp 2>&1
+	then
+	    echo >&2 "Called from:"
+	    cat >&2 $tmp/tmp
+	else
+	    # pstree not functional for us ... -s not supported in older
+	    # versions
+	    :
+	fi
     fi
 fi
 
@@ -1456,8 +1482,11 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp/list
 		    i=1
 		    while true
 		    do
+			# pmlc may race with pmlogger starting up here - a timeout is required
+			# to avoid pmlc blocking forever and hanging pmlogger_daily. RHBZ#1892326
+			[ -z "$PMLOGGER_REQUEST_TIMEOUT" ] && export PMLOGGER_REQUEST_TIMEOUT=2
 			echo status | pmlc $pid >$tmp/out 2>&1
-			if egrep "Connection refused|Transport endpoint is not connected" <$tmp/out >/dev/null
+			if egrep "Connection refused|Transport endpoint is not connected|Timeout, closed connection" <$tmp/out >/dev/null
 			then
 			    [ $i -eq 20 ] && break
 			    i=`expr $i + 1`
@@ -1631,7 +1660,7 @@ p
 # only be done once, e.g. after software upgrade with new pmlogrewrite(1)
 # configuration files
 #
-if [ -f $PCP_LOG_DIR/pmlogger/.NeedRewrite ]
+if ! $COMPRESSONLY && [ -f $PCP_LOG_DIR/pmlogger/.NeedRewrite ]
 then
     REWRITEALL=true
     $VERBOSE && echo "Info: found .NeedRewrite => rewrite all archives"
@@ -1654,9 +1683,16 @@ fi
 # if need be, remove .NeedRewrite so we don't trigger -R processing
 # next time
 #
-if [ -f $PCP_LOG_DIR/pmlogger/.NeedRewrite ]
+if ! $COMPRESSONLY && [ -f $PCP_LOG_DIR/pmlogger/.NeedRewrite ]
 then
     rm -f $PCP_LOG_DIR/pmlogger/.NeedRewrite
 fi
 
+# optional end logging to $PCP_LOG_DIR/NOTICES
+#
+if $PCP_LOG_RC_SCRIPTS
+then
+    $PCP_BINADM_DIR/pmpost "end pid:$$ $prog status=$status"
+fi
+
 exit
