@@ -33,6 +33,19 @@
 status=0
 prog=`basename $0`
 
+# optional begin logging to $PCP_LOG_DIR/NOTICES
+#
+if $PCP_LOG_RC_SCRIPTS
+then
+    logmsg="begin pid:$$ $prog args:$*"
+    if which pstree >/dev/null 2>&1
+    then
+	logmsg="$logmsg [`pstree -lps $$`]"
+	logmsg="`echo "$logmsg" | sed -e 's/---pstree([^)]*)//'`"
+    fi
+    $PCP_BINADM_DIR/pmpost "$logmsg"
+fi
+
 # error messages should go to stderr, not the GUI notifiers
 unset PCP_STDERR
 
@@ -303,4 +316,12 @@ _report :numa-per-node-cpu '# NUMA per-node CPU statistics'
 _report :numa-pgmigrate-per-node '# NUMA per-node page migration statistics' 
 
 [ -f $tmp/err ] && status=1
+
+# optional end logging to $PCP_LOG_DIR/NOTICES
+#
+if $PCP_LOG_RC_SCRIPTS
+then
+    $PCP_BINADM_DIR/pmpost "end pid:$$ $prog status=$status"
+fi
+
 exit
