@@ -33,17 +33,23 @@ int		vol_samples_counter;     /* Counts samples - reset for new vol*/
 int		vol_switch_afid = -1;    /* afid of event for vol switch */
 int		vol_switch_flag;         /* sighup received - switch vol now */
 int		vol_switch_alarm;	 /* vol_switch_callback() called */
+int		log_switch_flag;         /* SIGUSR2 received to re-exec / log-roll */
+int		argc_saved;		 /* saved for execv when switching logs */
+char		**argv_saved;		 /* saved for re-exec when switching logs */
 int		run_done_alarm;		 /* run_done_callback() called */
 int		log_alarm;	 	 /* log_callback() called */
 int		parse_done;
 int		primary;		/* Non-zero for primary pmlogger */
-char	    	*archBase;		/* base name for log files */
+char	    	*archBase;		/* template base name for archive */
+char		*archName;		/* real base name for archive */
 char		*pmcd_host;
 char		*pmcd_host_conn;
 char		*pmcd_host_label;
 int		host_context = PM_CONTEXT_HOST;	 /* pmcd / local context mode */
 int		archive_version = PM_LOG_VERS02; /* Type of archive to create */
 int		linger = 0;		/* linger with no tasks/events */
+int		notify_service_mgr = 0;	/* notify service manager when we're ready (daemon mode only) */
+int		pmlogger_reexec = 0;	/* set when PMLOGGER_REEXEC is set in the environment */
 int		rflag;			/* report sizes */
 int		Cflag;			/* parse config and exit */
 struct timeval	epoch;
@@ -67,9 +73,9 @@ static int	sep;
 void
 run_done(int sts, char *msg)
 {
-    int	lsts;
+    int	i, lsts;
 
-    if (pmDebugOptions.log && pmDebugOptions.desperate) {
+    if (pmDebugOptions.services || (pmDebugOptions.log && pmDebugOptions.desperate)) {
 	fprintf(stderr, "run_done(%d, %s) last_log_offset=%d last_stamp=",
 		sts, msg, last_log_offset);
 	pmPrintStamp(stderr, &last_stamp);
@@ -81,9 +87,9 @@ run_done(int sts, char *msg)
 	    pmErrStr(lsts));
 
     if (msg != NULL)
-    	fprintf(stderr, "pmlogger: %s, exiting\n", msg);
+    	fprintf(stderr, "pmlogger: %s, %s\n", msg, log_switch_flag ? "reexec" : "exiting");
     else
-    	fprintf(stderr, "pmlogger: End of run time, exiting\n");
+    	fprintf(stderr, "pmlogger: End of run time, %s\n", log_switch_flag ? "reexec" : "exiting");
 
     /*
      * write the last last temporal index entry with the time stamp
@@ -101,6 +107,39 @@ run_done(int sts, char *msg)
 	__pmLogPutIndex(&archctl, &tmp);
     }
 
+    /*
+     * close the archive
+     */
+    __pmFclose(archctl.ac_mfp);
+    __pmFclose(archctl.ac_log->l_tifp);
+    __pmFclose(archctl.ac_log->l_mdfp);
+
+    if (log_switch_flag) {
+    	/*
+	 * re-exec using saved args, see save_args().
+	 */
+	fprintf(stderr, "\n=== pmlogger: reexec cmdlne:");
+	for (i=0; i < argc_saved; i++)
+	    fprintf(stderr, " %s", argv_saved[i]);
+	fprintf(stderr, "\n");
+
+	/* close/remove all control fds and sockets */
+	cleanup();
+
+	/* this tells the next pmlogger it has been re-exec'd */
+	putenv("PMLOGGER_REEXEC=1");
+
+	execvp(argv_saved[0], argv_saved);
+	perror("Error: execvp returned unexpectedly");
+    }
+
+    /*
+     * Notify service manager, if any, we are stopping. Do this even
+     * if we've been reexec'd because the original pid is exiting.
+     */
+    if (notify_service_mgr)
+	__pmServerNotifyServiceManagerStopping(getpid());
+
     exit(sts);
 }
 
@@ -523,6 +562,7 @@ static pmLongOptions longopts[] = {
     PMOPT_SPECLOCAL,
     { "local-PMDA", 0, 'o', 0, "metrics sourced without connecting to pmcd" },
     PMOPT_NAMESPACE,
+    { "notify", 0, 'N', 0, "notify service manager (if any) when started and ready" },
     { "PID", 1, 'p', "PID", "Log specified metric for the lifetime of the pid" },
     { "primary", 0, 'P', 0, "execute as primary logger instance" },
     { "report", 0, 'r', 0, "report record sizes and archive growth rate" },
@@ -540,7 +580,7 @@ static pmLongOptions longopts[] = {
 };
 
 static pmOptions opts = {
-    .short_options = "c:CD:h:H:l:K:Lm:n:op:Prs:T:t:uU:v:V:x:y?",
+    .short_options = "c:CD:fh:H:l:K:Lm:Nn:op:Prs:T:t:uU:v:V:x:y?",
     .long_options = longopts,
     .short_usage = "[options] archive",
 };
@@ -594,6 +634,121 @@ do_pmcpp(char *configfile)
     return f;
 }
 
+/*
+ * Save original args for re-exec on SIGUSR2. Final arg is the
+ * archive name, which may contain strftime(3) meta chars.
+ */
+static void
+save_args(int argc, char **argv)
+{
+    int i, mflag = -1;
+
+    /*
+     * saved argv needs room for the sentinal NULL that
+     * terminates the argv array passed to execv(2) and
+     * also to insert -m reexec (if -m is not present).
+     */
+    argc_saved = argc;
+    argv_saved = (char **)malloc((argc + 3) * sizeof(char *));
+    if (argv_saved == NULL) {
+    	pmNoMem("save_args", argc * sizeof(char *), PM_FATAL_ERR);
+	/* NOTREACHED */
+    }
+
+    memset(argv_saved, 0, (argc + 3) * sizeof(char *)); /* for covscan */
+    for (i=0; i < argc; i++) {
+	if (strncmp(argv[i], "-m", 2) == 0)
+	    mflag = i;
+    	if ((argv_saved[i] = strdup(argv[i])) == NULL) {
+	    pmNoMem("save_args", strlen(argv[i]) + 1, PM_FATAL_ERR);
+	    /* NOTREACHED */
+	}
+    }
+
+    /*
+     * Replace "-m *" with "-m reexec"
+     * note: this may be one or two args
+     */
+    if (mflag > 0) {
+	if (strlen(argv_saved[mflag]) > 2) {
+	    /* remove one arg, e.g. -mpmlogger_check */
+	    free(argv_saved[mflag]);
+	    for (i=mflag; i < argc_saved; i++)
+	    	argv_saved[i] = argv_saved[i+1];
+	    argc_saved -= 1;
+	}
+	else {
+	    /* remove two args, e.g. -m pmlogger_check */
+	    if (argv_saved[mflag])
+		free(argv_saved[mflag]); /* covscan */
+	    if (argv_saved[mflag+1])
+		free(argv_saved[mflag+1]);
+	    for (i=mflag; i < argc_saved; i++)
+	    	argv_saved[i] = argv_saved[i+2];
+	    argc_saved -= 2;
+	}
+    }
+
+    /* insert -mreexec (as one arg) */
+    argv_saved[argc_saved] = argv_saved[argc_saved-1]; /* archive base name */
+    if ((argv_saved[argc_saved-1] = strdup("-mreexec")) == NULL) {
+	pmNoMem("save_args", strlen("-mreexec") + 1, PM_FATAL_ERR);
+	/* NOTREACHED */
+    }
+    argc_saved++;
+    argv_saved[argc_saved] = NULL; /* sentinal for execvp(3) */
+}
+
+static void
+updateLatestFolio(const char *host, const char *base)
+{
+    FILE *fp;
+    time_t now;
+    char date[26];
+    char dir[MAXPATHLEN];
+    char *logdir = pmGetConfig("PCP_ARCHIVE_DIR");
+    char thishost[MAXHOSTNAMELEN];
+
+    /*
+     * Only write the "Latest" folio if we're a pmlogger service daemon,
+     * i.e. pmlogger current dir is below $PCP_ARCHIVE_DIR
+     */
+    if (getcwd(dir, sizeof(dir)) == NULL) {
+	if (pmDebugOptions.services)
+	    fprintf(stderr, "Info: updateLatestFolio: getcwd() failed for host %s: %s", host, strerror(errno));
+	return;
+    }
+    if (strncmp(dir, logdir, strlen(logdir)) != 0) {
+	if (pmDebugOptions.services)
+	    fprintf(stderr, "Info: not creating \"Latest\" archive folio for host %s: %s", host, strerror(errno));
+    	return;
+    }
+
+    gethostname(thishost, MAXHOSTNAMELEN);
+    thishost[MAXHOSTNAMELEN-1] = '\0';
+
+    if ((fp = fopen("Latest", "w")) == NULL) {
+    	fprintf(stderr, "Warning: failed to create \"Latest\" archive folio for host %s: %s", host, strerror(errno));
+	return;
+    }
+    time(&now);
+    ctime_r(&now, date);
+
+    fprintf(fp,
+	"PCPFolio\n"
+	"Version: 1\n"
+	"# use pmafm(1) to process this PCP archive folio\n"
+	"#\n"
+	"Created: on %s at %s"
+	"Creator: pmlogger\n"
+	"%-15s %-23s %s\n"
+	"%-15s %-23s %s\n",
+	    thishost, date,
+	    "#", "Host", "Basename",
+	    "Archive:", host, base);
+    fclose(fp);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -607,6 +762,8 @@ main(int argc, char **argv)
 				    /* default log (not archive) file name */
     char		*endnum;
     int			i;
+    int			suff;		/* for -NN */
+    int			make_uniq = 0;	/* set if -NN suffix regime is in play */
     task_t		*tp;
     optcost_t		ocp;
     __pmFdSet		readyfds;
@@ -618,18 +775,16 @@ main(int argc, char **argv)
     pid_t               target_pid = 0;
     int			exit_code = 0;
     char		*exit_msg;
+    const char		*name = "pmcd.timezone";
+    pmID		pmid;
+    pmResult		*resp;
 
+    save_args(argc, argv);
     pmGetUsername(&username);
     sep = pmPathSeparator();
     if ((endnum = getenv("PMLOGGER_INTERVAL")) != NULL)
 	delta.tv_sec = atoi(endnum);
 
-    /*
-     * Warning:
-     *		If any of the pmlogger options change, make sure the
-     *		corresponding changes are made to pmnewlog when pmlogger
-     *		options are passed through from the control file
-     */
     while ((c = pmgetopt_r(argc, argv, &opts)) != EOF) {
 	switch (c) {
 
@@ -692,8 +847,13 @@ main(int argc, char **argv)
 
 	case 'm':		/* note for port map file */
 	    note = opts.optarg;
-	    isdaemon = ((strcmp(note, "pmlogger_check") == 0) ||
-			(strcmp(note, "pmlogger_daily") == 0));
+	    isdaemon = ((strncmp(note, "pmlogger_check", 14) == 0) ||
+			(strncmp(note, "pmlogger_daily", 14) == 0) ||
+			(strncmp(note, "reexec", 6) == 0));
+	    break;
+
+	case 'N':		/* notify service manager (even if not primary) */
+	    notify_service_mgr = 1;
 	    break;
 
 	case 'n':		/* alternative name space file */
@@ -830,16 +990,34 @@ main(int argc, char **argv)
 	opts.errors++;
     }
 
-    if (!opts.errors && opts.optind != argc - 1) {
+    if (!opts.errors && ((Cflag == 0 && opts.optind > argc - 1) ||
+			 (Cflag == 1 && opts.optind > argc))) {
 	pmprintf("%s: insufficient arguments\n", pmGetProgname());
 	opts.errors++;
     }
 
+    if (!opts.errors && ((Cflag == 0 && opts.optind < argc - 1) ||
+			 (Cflag == 1 && opts.optind < argc))) {
+	pmprintf("%s: too many arguments\n", pmGetProgname());
+	opts.errors++;
+    }
+
     if (opts.errors) {
 	pmUsageMessage(&opts);
 	exit(1);
     }
 
+    if (getenv("PMLOGGER_REEXEC") != NULL) {
+	/*
+	 * We have been re-exec'd. See run_done(). This flag indicates
+	 * not to daemonize, do not notify the service manager, do
+	 * not (re)create the PID file and keep using the same log file.
+	 * We still create and bind to various sockets because these
+	 * are cleaned up by the outgoing pmlogger prior to exec.
+	 */
+	pmlogger_reexec = 1;
+    }
+
     if (rsc_fd != -1 && note == NULL) {
 	/* add default note to indicate running with -x */
 	static char	xnote[10];
@@ -852,18 +1030,52 @@ main(int argc, char **argv)
 	pmSetProcessIdentity(username);
 
     if (Cflag == 0) {
-	pmOpenLog("pmlogger", logfile, stderr, &sts);
-	if (sts != 1) {
-	    fprintf(stderr, "%s: Warning: log file (%s) creation failed\n", pmGetProgname(), logfile);
-	    /* continue on ... writing to stderr */
+	/* only open a new log if we are NOT reexec'd */
+	if (pmlogger_reexec) {
+	    if (pmDebugOptions.services)
+		fprintf(stderr, "existing pmlogger.log remains open after re-exec\n");
+	} else {
+	    pmOpenLog("pmlogger", logfile, stderr, &sts);
+	    if (sts != 1) {
+		fprintf(stderr, "%s: Warning: log file (%s) creation failed\n", pmGetProgname(), logfile);
+		/* continue on ... writing to stderr */
+	    }
 	}
     }
 
-    /* base name for archive is here ... */
-    archBase = strdup(argv[opts.optind]);
-    if (archBase == NULL) {
-	pmNoMem("main", strlen(argv[opts.optind])+1, PM_FATAL_ERR);
-	/* NOTREACHED */
+    if (Cflag == 0) {
+	/* base name for archive is here ... */
+	if ((archName = malloc(MAXPATHLEN+1)) == NULL) {
+	    pmNoMem("main: archName", strlen(argv[opts.optind])+1, PM_FATAL_ERR);
+	    /* NOTREACHED */
+	}
+	if (strchr(argv[opts.optind], '%') == NULL) {
+	    /* no meta chars - go with what we have been given */
+	    if ((archBase = strdup(argv[opts.optind])) == NULL) {
+		pmNoMem("main: strdup archBase", strlen(argv[opts.optind])+1, PM_FATAL_ERR);
+		/* NOTREACHED */
+	    }
+	} else {
+	    /*
+	     * strftime(3) meta char substitution.
+	     */
+	    time_t	now;
+	    struct tm	*arch_tm;
+
+	    if ((archBase = malloc(MAXPATHLEN+1)) == NULL) {
+		pmNoMem("main malloc archBase", MAXPATHLEN+1, PM_FATAL_ERR);
+		/* NOTREACHED */
+	    }
+	    time(&now);
+	    arch_tm = localtime(&now);
+	    if (strftime(archBase, MAXPATHLEN, argv[opts.optind], arch_tm) == 0) {
+		fprintf(stderr, "Error: strftime failed on \"%s\"\n", argv[opts.optind]);
+		exit(1);
+	    }
+	    if (pmDebugOptions.services)
+		fprintf(stderr, "archBase after strftime substitutions: \"%s\"\n", archBase);
+	    make_uniq = 1;
+	}
     }
 
     /* initialise access control */
@@ -955,6 +1167,21 @@ main(int argc, char **argv)
 	    __pmOptFetchDump(stderr, tp->t_fetch);
 	}
     }
+    if (pmDebugOptions.optfetch) {
+	int	j = 0;
+	for (tp = tasklist; tp != NULL; tp = tp->t_next) {
+	    fetchctl_t	*fcp = tp->t_fetch;
+	    int		fg = 0;
+	    fprintf(stderr, "Fetch task[%d] delta: %ld usec numpmid: %d\n",
+		j, (long)(1000000 * tp->t_delta.tv_sec + tp->t_delta.tv_usec),
+		tp->t_numpmid);
+	    while (fcp != NULL) {
+		fprintf(stderr, "  Fetch group[%d][%d] %d metrics\n", j, fg++, fcp->f_numpmid);
+		fcp = fcp->f_next;
+	    }
+	    j++;
+	}
+    }
 
     if (Cflag)
 	exit(0);
@@ -972,41 +1199,60 @@ main(int argc, char **argv)
     }
 
     archctl.ac_log = &logctl;
-    if ((sts = __pmLogCreate(pmcd_host, archBase, archive_version, &archctl)) < 0) {
-	fprintf(stderr, "__pmLogCreate: %s\n", pmErrStr(sts));
+
+    /*
+     * If we reexec quickly then archBase will be the same as the
+     * previous iteration, and if this happens use a -NN suffix to make
+     * archName different, ... but only if make_uniq is set
+     */
+    for (suff = -1; suff < 99; suff++) { /* limit of 100 retries */
+	if (suff == -1)
+	    memcpy(archName, archBase, strlen(archBase)+1);
+	else
+	    snprintf(archName, MAXPATHLEN, "%s-%02d", archBase, suff);
+
+	if ((sts = __pmLogCreate(pmcd_host, archName, archive_version, &archctl)) < 0) {
+	    if (make_uniq)
+		continue;	/* try the next -NN */
+	    /* otherwise this is fatal */
+	    break;
+	}
+	/* success */
+	if (pmDebugOptions.services)
+	    fprintf(stderr, "archName after __pmLogCreate: \"%s\"\n", archName);
+	break;
+    }
+    if (sts < 0) {
+	fprintf(stderr, "__pmLogCreate(%s, %s, ...): %s\n", pmcd_host, archName, pmErrStr(sts));
 	exit(1);
     }
-    else {
-	/*
-	 * try and establish $TZ from the remote PMCD ...
-	 * Note the label record has been set up, but not written yet
-	 */
-	char		*name = "pmcd.timezone";
-	pmID		pmid;
-	pmResult	*resp;
 
-	pmtimevalNow(&epoch);
-	sts = pmUseContext(ctx);
+    /*
+     * try and establish $TZ from the remote PMCD ...
+     * Note the label record has been set up, but not written yet
+     */
 
-	if (sts >= 0)
-	    sts = pmLookupName(1, &name, &pmid);
-	if (sts >= 0)
-	    sts = pmFetch(1, &pmid, &resp);
-	if (sts >= 0) {
-	    if (resp->vset[0]->numval > 0) { /* pmcd.timezone present */
-		strcpy(logctl.l_label.ill_tz, resp->vset[0]->vlist[0].value.pval->vbuf);
-		/* prefer to use remote time to avoid clock drift problems */
-		epoch = resp->timestamp;		/* struct assignment */
-		if (! use_localtime)
-		    pmNewZone(logctl.l_label.ill_tz);
-	    }
-	    else if (pmDebugOptions.log) {
-		fprintf(stderr,
-			"main: Could not get timezone from host %s\n",
-			pmcd_host);
-	    }
-	    pmFreeResult(resp);
+    pmtimevalNow(&epoch);
+    sts = pmUseContext(ctx);
+
+    if (sts >= 0)
+	sts = pmLookupName(1, &name, &pmid);
+    if (sts >= 0)
+	sts = pmFetch(1, &pmid, &resp);
+    if (sts >= 0) {
+	if (resp->vset[0]->numval > 0) { /* pmcd.timezone present */
+	    strcpy(logctl.l_label.ill_tz, resp->vset[0]->vlist[0].value.pval->vbuf);
+	    /* prefer to use remote time to avoid clock drift problems */
+	    epoch = resp->timestamp;		/* struct assignment */
+	    if (! use_localtime)
+		pmNewZone(logctl.l_label.ill_tz);
 	}
+	else if (pmDebugOptions.log) {
+	    fprintf(stderr,
+		    "main: Could not get timezone from host %s\n",
+		    pmcd_host);
+	}
+	pmFreeResult(resp);
     }
 
     /* do ParseTimeWindow stuff for -T */
@@ -1039,15 +1285,25 @@ main(int argc, char **argv)
         last_stamp = res_end;
     }
 
-    fprintf(stderr, "Archive basename: %s\n", archBase);
+    fprintf(stderr, "Archive basename: %s\n", archName);
 
+    if (notify_service_mgr && !pmlogger_reexec) {
+	/*
+	 * If we haven't been reexec'd, notify service manager (if any),
+	 * that we are ready.
+	 */
+	__pmServerNotifyServiceManagerReady(getpid());
+    }
+
+    if (isdaemon) {
 #ifndef IS_MINGW
-    /* detach yourself from the launching process */
-    if (isdaemon)
+	/* detach yourself from the launching process */
         setpgid(getpid(), 0);
 #endif
+	__pmServerCreatePIDFile(pmGetProgname(), 0);
+    }
 
-    /* set up control port */
+    /* set up control ports and signal handlers */
     init_ports();
     __pmFD_ZERO(&fds);
     for (i = 0; i < CFD_NUM; ++i) {
@@ -1071,6 +1327,11 @@ main(int argc, char **argv)
     parse_done = 1;	/* enable callback processing */
     __pmAFunblock();
 
+    /* create the Latest folio */
+    if (isdaemon) {
+	updateLatestFolio(pmcd_host, archName);
+    }
+
     for ( ; ; ) {
 	int		nready;
 
@@ -1302,7 +1563,7 @@ newvolume(int vol_switch_type)
                                    vol_switch_callback);
     }
 
-    if ((newfp = __pmLogNewFile(archBase, nextvol)) != NULL) {
+    if ((newfp = __pmLogNewFile(archName, nextvol)) != NULL) {
 	if (logctl.l_state == PM_LOG_STATE_NEW) {
 	    /*
 	     * nothing has been logged as yet, force out the label records
