@@ -45,20 +45,20 @@ doscan(struct timeval *end)
 	 */
 	if ((ictx_b = pmNewContext(PM_CONTEXT_ARCHIVE, iname)) < 0) {
 	    fprintf(stderr, "%s: Error: cannot open archive \"%s\" (ctx_b): %s\n",
-		    pmProgname, iname, pmErrStr(ictx_b));
+		    pmGetProgname(), iname, pmErrStr(ictx_b));
 	    exit(1);
 	}
 
 	if ((sts = pmSetMode(PM_MODE_FORW, NULL, 0)) < 0) {
 	    fprintf(stderr,
-		"%s: Error: pmSetMode (ictx_b) failed: %s\n", pmProgname, pmErrStr(sts));
+		"%s: Error: pmSetMode (ictx_b) failed: %s\n", pmGetProgname(), pmErrStr(sts));
 	    exit(1);
 	}
     }
 
     if ((sts = pmUseContext(ictx_b)) < 0) {
 	fprintf(stderr, "%s: doscan: Error: cannot use context: %s\n",
-		pmProgname, pmErrStr(sts));
+		pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
 
@@ -75,18 +75,16 @@ doscan(struct timeval *end)
 	    if (sts == PM_ERR_EOL)
 		break;
 	    fprintf(stderr,
-		"%s: doscan: Error: pmFetch failed: %s\n", pmProgname, pmErrStr(sts));
+		"%s: doscan: Error: pmFetch failed: %s\n", pmGetProgname(), pmErrStr(sts));
 	    exit(1);
 	}
-#if PCP_DEBUG
-	if (pmDebug & DBG_TRACE_APPL2) {
+	if (pmDebugOptions.appl2) {
 	    if (nr == 0) {
 		fprintf(stderr, "scan starts at ");
-		__pmPrintStamp(stderr, &rp->timestamp);
+		pmPrintStamp(stderr, &rp->timestamp);
 		fprintf(stderr, "\n");
 	    }
 	}
-#endif
 
 	if (rp->numpmid == 0) {
 	    /*
@@ -99,17 +97,22 @@ doscan(struct timeval *end)
 		__pmPDU		len;
 		__pmPDU		type;
 		__pmPDU		from;
-		__pmTimeval	timestamp;
+		pmTimeval	timestamp;
 		int		numpmid;	/* zero PMIDs to follow */
+		__pmPDU		trailer;
 	    } markrec;
-	    markrec.len = sizeof(markrec);
+	    /*
+	     * add space for, but don't bump length for, trailer so
+	     * __pmLogPutResult2() has space for trailer in the buffer
+	     */
+	    markrec.len = sizeof(markrec) - sizeof(__pmPDU);
 	    markrec.type = markrec.from = 0;
 	    markrec.timestamp.tv_sec = htonl(rp->timestamp.tv_sec);
 	    markrec.timestamp.tv_usec = htonl(rp->timestamp.tv_usec);
 	    markrec.numpmid = 0;
-	    if ((sts = __pmLogPutResult(&logctl, (__pmPDU *)&markrec)) < 0) {
-		fprintf(stderr, "%s: Error: __pmLogPutResult: mark record write: %s\n",
-			pmProgname, pmErrStr(sts));
+	    if ((sts = __pmLogPutResult2(&archctl, (__pmPDU *)&markrec)) < 0) {
+		fprintf(stderr, "%s: Error: __pmLogPutResult2: mark record write: %s\n",
+			pmGetProgname(), pmErrStr(sts));
 		exit(1);
 	    }
 	    /*
@@ -146,7 +149,7 @@ doscan(struct timeval *end)
 	    if (i == numpmid) {
 		fprintf(stderr,
 		    "%s: scan: Arrgh, cannot find pid %s in pidlist[]\n",
-			pmProgname, pmIDStr(vsp->pmid));
+			pmGetProgname(), pmIDStr(vsp->pmid));
 		exit(1);
 	    }
 	    mp = &metriclist[i];
@@ -164,7 +167,7 @@ doscan(struct timeval *end)
 		    vp = (value_t *)malloc(sizeof(value_t));
 		    if (vp == NULL) {
 			fprintf(stderr,
-			    "%s: rewrite: Arrgh, cannot malloc value_t\n", pmProgname);
+			    "%s: rewrite: Arrgh, cannot malloc value_t\n", pmGetProgname());
 			exit(1);
 		    }
 		    if (lvp == NULL)
@@ -175,14 +178,12 @@ doscan(struct timeval *end)
 		    vp->nobs = vp->nwrap = 0;
 		    vp->control = V_INIT;
 		    vp->next = NULL;
-#if PCP_DEBUG && DESPERATE
 
-		    if (pmDebug & DBG_TRACE_APPL0) {
+		    if (pmDebugOptions.appl1) {
 			fprintf(stderr,
 			    "add value_t for %s (%s) inst %d\n",
 			    namelist[i], pmIDStr(pmidlist[i]), vsp->vlist[j].inst);
 		    }
-#endif
 		}
 		/* TODO ... hard part goes here 8^) */
 		if (mp->idesc.sem == PM_SEM_COUNTER) {
@@ -192,34 +193,30 @@ doscan(struct timeval *end)
 		     */
 		    ;
 		}
-#if PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL1) {
-		    __pmPrintStamp(stderr, &rp->timestamp);
+		if (pmDebugOptions.appl1) {
+		    pmPrintStamp(stderr, &rp->timestamp);
 		    fprintf(stderr, ": seen %s (%s) inst %d\n",
 			namelist[i], pmIDStr(pmidlist[i]),
 			vsp->vlist[j].inst);
 		}
-#endif
 		vp->control |= V_SEEN;
 	    }
 	}
 
 	pmFreeResult(rp);
     }
-#if PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL2) {
+    if (pmDebugOptions.appl2) {
 	fprintf(stderr, "scan ends at ");
-	__pmPrintStamp(stderr, &last_tv);
+	pmPrintStamp(stderr, &last_tv);
 	if (sts == PM_ERR_EOL)
 	    fprintf(stderr, " [EOL]");
 	fprintf(stderr, " (%d records)\n", nr);
     }
-#endif
 
-    if ((sts == pmSetMode(PM_MODE_FORW, &last_tv, 0)) < 0) {
+    if ((sts = pmSetMode(PM_MODE_FORW, &last_tv, 0)) < 0) {
 	fprintf(stderr,
-	    "%s: doscan: Error: pmSetMode (ictx_b) time=", pmProgname);
-	__pmPrintStamp(stderr, &last_tv);
+	    "%s: doscan: Error: pmSetMode (ictx_b) time=", pmGetProgname());
+	pmPrintStamp(stderr, &last_tv);
 	fprintf(stderr,
 	    " failed: %s\n", pmErrStr(sts));
 	exit(1);
