@@ -35,12 +35,13 @@ static pmLongOptions longopts[] = {
     { "maximum", 0, 'M', 0, "also print maximum value" },
     PMOPT_NAMESPACE,
     { "", 0, 'N', 0, "suppress warnings from individual archive fetches (default)" },
-    { "precision", 0, 'p', 0, "number of digits to display after the decimal point" },
+    { "precision", 1, 'p', "N", "number of digits to display after the decimal point" },
+    { "sum", 0, 's', 0, "only print the sum of all values of each metric" },
     PMOPT_START,
     PMOPT_FINISH,
     { "verbose", 0, 'v', 0, "verbose, enable warnings from individual archive fetches" },
     { "", 0, 'x', 0, "print only stochastic averages for counter metrics" },
-    { "samples", 0, 'y', "print sample count for each metric" },
+    { "samples", 0, 'y', 0, "print sample count for each metric" },
     PMOPT_TIMEZONE,
     PMOPT_HOSTZONE,
     PMOPT_VERSION,
@@ -163,8 +164,10 @@ printstamp(struct timeval *stamp, int delimiter)
     if (dayflag) {
 	char	*ddmm;
 	char	*yr;
+	time_t	time;
 
-	ddmm = pmCtime((const time_t *)&stamp->tv_sec, timebuf);
+	time = stamp->tv_sec;
+	ddmm = pmCtime(&time, timebuf);
 	ddmm[10] = ' ';
 	ddmm[11] = '\0';
 	yr = &ddmm[20];
@@ -185,6 +188,7 @@ printlabel(void)
     char        *ddmm;
     char        *yr;
     int         sts;
+    time_t	time;
 
     if ((sts = pmGetArchiveLabel(&label)) < 0) {
 	fprintf(stderr, "%s: Cannot get archive label record: %s\n",
@@ -195,7 +199,8 @@ printlabel(void)
     printf("Log Label (Log Format Version %d)\n", label.ll_magic & 0xff);
     printf("Performance metrics from host %s\n", label.ll_hostname);
 
-    ddmm = pmCtime((const time_t *)&opts.start.tv_sec, timebuf);
+    time = opts.start.tv_sec;
+    ddmm = pmCtime(&time, timebuf);
     ddmm[10] = '\0';
     yr = &ddmm[20];
     printf("  commencing %s ", ddmm);
@@ -207,7 +212,8 @@ printlabel(void)
 	printf("  ending     UNKNOWN\n");
     }
     else {
-	ddmm = pmCtime((const time_t *)&opts.finish.tv_sec, timebuf);
+	time = opts.finish.tv_sec;
+	ddmm = pmCtime(&time, timebuf);
 	ddmm[10] = '\0';
 	yr = &ddmm[20];
 	printf("  ending     %s ", ddmm);
@@ -257,7 +263,7 @@ printsummary(const char *name)
     struct timeval	metrictimespan;
 
     /* cast away const, pmLookupName should never modify name */
-    if ((sts = pmLookupName(1, (char **)&name, &pmid)) < 0) {
+    if ((sts = pmLookupName(1, &name, &pmid)) < 0) {
 	fprintf(stderr, "%s: failed to lookup metric name (pmid=%s): %s\n",
 		pmGetProgname(), name, pmErrStr(sts));
 	return;
@@ -288,7 +294,7 @@ printsummary(const char *name)
 	    /* counter metric doesn't cover 90% of log */
 	    star = (avedata->desc.sem == PM_SEM_COUNTER && metricspan / logspan <= 0.1);
 
-	    if ((sts = pmNameInDom(avedata->desc.indom, instdata->inst, &str)) < 0) {
+	    if ((sts = pmNameInDomArchive(avedata->desc.indom, instdata->inst, &str)) < 0) {
 		if (msp && msp->ninst > 0 && avedata->desc.indom == PM_INDOM_NULL)
 		    break;
 		if (star)
@@ -860,7 +866,7 @@ calcaverage(pmResult *result)
 				char	*istr = NULL;
 
 				numnames = pmNameAll(avedata->desc.pmid, &names);
-				if (pmNameInDom(avedata->desc.indom,
+				if (pmNameInDomArchive(avedata->desc.indom,
 				    instdata->inst, &istr) < 0)
 				    istr = NULL;
 				if (rate < instdata->min) {
