@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019 Red Hat.
+ * Copyright (c) 2019-2020 Red Hat.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -34,29 +34,39 @@ typedef enum http_flags {
     HTTP_FLAG_JSON	= (1<<0),
     HTTP_FLAG_TEXT	= (1<<1),
     HTTP_FLAG_HTML	= (1<<2),
-    HTTP_FLAG_JS	= (1<<3),
-    HTTP_FLAG_CSS	= (1<<4),
-    HTTP_FLAG_ICO	= (1<<5),
-    HTTP_FLAG_JPG	= (1<<6),
-    HTTP_FLAG_PNG	= (1<<7),
-    HTTP_FLAG_GIF	= (1<<8),
     HTTP_FLAG_UTF8	= (1<<10),
     HTTP_FLAG_UTF16	= (1<<11),
+    HTTP_FLAG_NO_BODY	= (1<<13),
     HTTP_FLAG_COMPRESS	= (1<<14),
     HTTP_FLAG_STREAMING	= (1<<15),
     /* maximum 16 for server.h */
 } http_flags;
 
+typedef enum http_options {
+    HTTP_OPT_GET	= (1 << HTTP_GET),
+    HTTP_OPT_PUT	= (1 << HTTP_PUT),
+    HTTP_OPT_HEAD	= (1 << HTTP_HEAD),
+    HTTP_OPT_POST	= (1 << HTTP_POST),
+    HTTP_OPT_TRACE	= (1 << HTTP_TRACE),
+    HTTP_OPT_OPTIONS	= (1 << HTTP_OPTIONS),
+    /* maximum 16 in command opts fields */
+} http_options;
+
+#define HTTP_COMMON_OPTIONS (HTTP_OPT_HEAD | HTTP_OPT_TRACE | HTTP_OPT_OPTIONS)
+#define HTTP_OPTIONS_GET    (HTTP_COMMON_OPTIONS | HTTP_OPT_GET)
+#define HTTP_OPTIONS_PUT    (HTTP_COMMON_OPTIONS | HTTP_OPT_PUT)
+#define HTTP_OPTIONS_POST   (HTTP_COMMON_OPTIONS | HTTP_OPT_POST)
+#define HTTP_SERVER_OPTIONS (HTTP_OPTIONS_GET | HTTP_OPT_PUT | HTTP_OPT_POST)
+
 typedef unsigned int http_code;
 
 extern void http_transfer(struct client *);
-extern void http_reply(struct client *, sds, http_code, http_flags);
+extern void http_reply(struct client *, sds, http_code, http_flags, http_options);
 extern void http_error(struct client *, http_code, const char *);
 
 extern int http_decode(const char *, size_t, sds);
 extern const char *http_status_mapping(http_code);
 extern const char *http_content_type(http_flags);
-extern http_flags http_suffix_type(const char *);
 
 extern sds http_get_buffer(struct client *);
 extern void http_set_buffer(struct client *, sds, http_flags);
@@ -81,6 +91,7 @@ typedef struct servlet {
     httpReleaseCallBack	on_release;
 } servlet;
 
+extern struct servlet pmsearch_servlet;
 extern struct servlet pmseries_servlet;
 extern struct servlet pmwebapi_servlet;
 
