@@ -1,6 +1,5 @@
 /*
- * Copyright (c) 2018-2019 Red Hat.
- * Copyright (c) 2018 Challa Venkata Naga Prajwal <cvnprajwal at gmail dot com>
+ * Copyright (c) 2018-2020 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -15,19 +14,29 @@
 #include "server.h"
 #include "discover.h"
 
+static int search_queries;
 static int series_queries;
 static int redis_protocol;
 static int archive_discovery;
 
+static pmDiscoverCallBacks redis_series = {
+    .on_source		= pmSeriesDiscoverSource,
+    .on_closed		= pmSeriesDiscoverClosed,
+    .on_labels		= pmSeriesDiscoverLabels,
+    .on_metric		= pmSeriesDiscoverMetric,
+    .on_values		= pmSeriesDiscoverValues,
+    .on_indom		= pmSeriesDiscoverInDom,
+    .on_text		= pmSeriesDiscoverText,
+};
+
+static pmDiscoverCallBacks redis_search = {
+    .on_metric		= pmSearchDiscoverMetric,
+    .on_indom		= pmSearchDiscoverInDom,
+    .on_text		= pmSearchDiscoverText,
+};
+
 static pmDiscoverSettings redis_discover = {
-    .callbacks.on_source	= pmSeriesDiscoverSource,
-    .callbacks.on_closed	= pmSeriesDiscoverClosed,
-    .callbacks.on_labels	= pmSeriesDiscoverLabels,
-    .callbacks.on_metric	= pmSeriesDiscoverMetric,
-    .callbacks.on_values	= pmSeriesDiscoverValues,
-    .callbacks.on_indom		= pmSeriesDiscoverInDom,
-    .callbacks.on_text		= pmSeriesDiscoverText,
-    .module.on_info		= proxylog,
+    .module.on_info	= proxylog,
 };
 
 static sds
@@ -41,20 +50,42 @@ redisfmt(redisReply *reply)
 
     switch (reply->type) {
     case REDIS_REPLY_STRING:
-	return sdscatfmt(command, "$%U\r\n%s\r\n", reply->len, reply->str);
+	command = sdscatfmt(command, "$%U\r\n", (uint64_t)reply->len);
+	command = sdscatlen(command, reply->str, reply->len);
+	return sdscatlen(command, "\r\n", 2);
     case REDIS_REPLY_ARRAY:
+    case REDIS_REPLY_MAP:
+    case REDIS_REPLY_SET:
 	c = sdsempty();
-	for (i = 0; i < reply->elements; i++)
-	    c = sdscat(c, redisfmt(reply->element[i]));
-	command = sdscatfmt(command, "*%u\r\n%S", reply->elements, c);
+	for (i = 0; i < reply->elements; i++) {
+	    sds e = redisfmt(reply->element[i]);
+	    c = sdscatsds(c, e);
+	    sdsfree(e);
+	}
+	if (reply->type == REDIS_REPLY_ARRAY)
+	    command = sdscatfmt(command, "*%U\r\n%S", (uint64_t)reply->elements, c);
+	else if (reply->type == REDIS_REPLY_MAP)
+	    command = sdscatfmt(command, "%%%U\r\n%S", (uint64_t)reply->elements, c);
+	else /* (reply->type == REDIS_REPLY_SET) */
+	    command = sdscatfmt(command, "~%U\r\n%S", (uint64_t)reply->elements, c);
 	sdsfree(c);
 	return command;
     case REDIS_REPLY_INTEGER:
 	return sdscatfmt(command, ":%I\r\n", reply->integer);
+    case REDIS_REPLY_DOUBLE:
+	command = sdscatlen(command, ",", 1);
+	command = sdscatlen(command, reply->str, reply->len);
+	return sdscatlen(command, "\r\n", 2);
     case REDIS_REPLY_STATUS:
-	return sdscatfmt(command, "+%s\r\n", reply->str);
+	command = sdscatlen(command, "+", 1);
+	command = sdscatlen(command, reply->str, reply->len);
+	return sdscatlen(command, "\r\n", 2);
     case REDIS_REPLY_ERROR:
-	return sdscatfmt(command, "-%s\r\n", reply->str);
+	command = sdscatlen(command, "-", 1);
+	command = sdscatlen(command, reply->str, reply->len);
+	return sdscatlen(command, "\r\n", 2);
+    case REDIS_REPLY_BOOL:
+	return sdscatfmt(command, "#%s\r\n", reply->integer ? "t" : "f");
     case REDIS_REPLY_NIL:
     default:
 	break;
@@ -80,7 +111,7 @@ on_redis_client_read(struct proxy *proxy, struct client *client,
     if (pmDebugOptions.pdu)
 	fprintf(stderr, "%s: client %p\n", "on_redis_client_read", client);
 
-    if (redis_protocol == 0 ||
+    if (redis_protocol == 0 || proxy->redisetup == 0 ||
 	redisSlotsProxyConnect(proxy->slots,
 		proxylog, &client->u.redis.reader,
 		buf->base, nread, on_redis_server_reply, client) < 0) {
@@ -105,17 +136,36 @@ static void
 on_redis_connected(void *arg)
 {
     struct proxy	*proxy = (struct proxy *)arg;
+    mmv_registry_t	*metric_registry = proxymetrics(proxy, METRICS_REDIS);
     sds			message;
 
     message = sdsnew("Redis slots");
     if (redis_protocol)
 	message = sdscat(message, ", command keys");
-    if (archive_discovery || series_queries)
+    if ((search_queries = pmSearchEnabled(proxy->slots)))
+	message = sdscat(message, ", RediSearch");
+    if (series_queries)
 	message = sdscat(message, ", schema version");
     pmNotifyErr(LOG_INFO, "%s setup\n", message);
     sdsfree(message);
 
-    pmDiscoverSetSlots(&redis_discover.module, proxy->slots);
+    if (series_queries) {
+	if (search_queries)
+	    redis_series.next = &redis_search;
+	redis_discover.callbacks = redis_series;
+    } else if (search_queries) {
+	redis_discover.callbacks = redis_search;
+    }
+
+    if (archive_discovery && (series_queries || search_queries)) {
+	pmDiscoverSetEventLoop(&redis_discover.module, proxy->events);
+	pmDiscoverSetConfiguration(&redis_discover.module, proxy->config);
+	pmDiscoverSetMetricRegistry(&redis_discover.module, metric_registry);
+	pmDiscoverSetup(&redis_discover.module, &redis_discover.callbacks, proxy);
+	pmDiscoverSetSlots(&redis_discover.module, proxy->slots);
+    }
+
+    proxy->redisetup = 1;
 }
 
 /*
@@ -127,33 +177,27 @@ void
 setup_redis_module(struct proxy *proxy)
 {
     redisSlotsFlags	flags = SLOTS_NONE;
-    mmv_registry_t	*metric_registry = proxymetrics(proxy, METRICS_REDIS);
     sds			option;
 
     if ((option = pmIniFileLookup(config, "pmproxy", "redis.enabled")))
 	redis_protocol = (strncmp(option, "true", sdslen(option)) == 0);
     if ((option = pmIniFileLookup(config, "pmseries", "enabled")))
 	series_queries = (strncmp(option, "true", sdslen(option)) == 0);
+    if ((option = pmIniFileLookup(config, "pmsearch", "enabled")))
+	search_queries = (strncmp(option, "true", sdslen(option)) == 0);
     if ((option = pmIniFileLookup(config, "discover", "enabled")))
 	archive_discovery = (strncmp(option, "true", sdslen(option)) == 0);
 
     if (proxy->slots == NULL) {
 	if (redis_protocol)
 	    flags |= SLOTS_KEYMAP;
-	if (archive_discovery || series_queries)
+	if (series_queries)
 	    flags |= SLOTS_VERSION;
+	if (search_queries)
+	    flags |= SLOTS_SEARCH;
 	proxy->slots = redisSlotsConnect(proxy->config,
 			flags, proxylog, on_redis_connected,
 			proxy, proxy->events, proxy);
-	if (archive_discovery && series_queries)
-	    pmDiscoverSetSlots(&redis_discover.module, proxy->slots);
-    }
-
-    if (archive_discovery && series_queries) {
-	pmDiscoverSetEventLoop(&redis_discover.module, proxy->events);
-	pmDiscoverSetConfiguration(&redis_discover.module, proxy->config);
-	pmDiscoverSetMetricRegistry(&redis_discover.module, metric_registry);
-	pmDiscoverSetup(&redis_discover.module, &redis_discover.callbacks, proxy);
     }
 }
 
