@@ -1,12 +1,12 @@
 /*
  * Copyright (c) 2019 Red Hat.
  *
- * This library is free software; you can redistribute it and/or modify it
+ * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but
+ * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
@@ -29,6 +29,9 @@ remove_connection_from_queue(struct client *client)
 void
 on_secure_client_close(struct client *client)
 {
+    if (pmDebugOptions.auth || pmDebugOptions.http)
+	fprintf(stderr, "%s: client %p\n", "on_secure_client_close", client);
+
     remove_connection_from_queue(client);
     /* client->read and client->write freed by SSL_free */
     SSL_free(client->secure.ssl);
@@ -71,6 +74,13 @@ setup_secure_client(struct proxy *proxy, struct client *client)
     SSL_set_bio(client->secure.ssl, client->secure.read, client->secure.write);
 }
 
+void
+on_secure_client_write(struct client *client)
+{
+    if (pmDebugOptions.auth || pmDebugOptions.http)
+	fprintf(stderr, "%s: client %p\n", "on_secure_client_write", client);
+}
+
 void
 on_secure_client_read(struct proxy *proxy, struct client *client,
 			ssize_t nread, const uv_buf_t *buf)
@@ -78,6 +88,9 @@ on_secure_client_read(struct proxy *proxy, struct client *client,
     size_t		bytes = 0;
     int			sts;
 
+    if (pmDebugOptions.auth || pmDebugOptions.http)
+	fprintf(stderr, "%s: client %p\n", "on_secure_client_read", client);
+
     /* once-off per-client SSL setup first time through */
     if (client->stream.secure == 0) {
 	setup_secure_client(proxy, client);
@@ -97,7 +110,7 @@ on_secure_client_read(struct proxy *proxy, struct client *client,
 	else if (SSL_get_error(client->secure.ssl, sts) == SSL_ERROR_WANT_READ)
 	    maybe_flush_ssl(proxy, client); /* defer to libuv if more to read */
 	else
-	    uv_close((uv_handle_t *)&client->stream, on_client_close);
+	    client_close(client);
 	break;
     } while (1);
 }
@@ -105,14 +118,17 @@ on_secure_client_read(struct proxy *proxy, struct client *client,
 static void
 flush_ssl_buffer(struct client *client)
 {
-    stream_write_baton	*request = calloc(1, sizeof(stream_write_baton));
+    stream_write_baton	*request;
     ssize_t		bytes;
 
     if ((bytes = BIO_pending(client->secure.write)) > 0) {
+	request = calloc(1, sizeof(stream_write_baton));
 	request->buffer[0] = uv_buf_init(sdsnewlen(SDS_NOINIT, bytes), bytes);
+	request->nbuffers = 1;
+	request->writer.data = client;
 	BIO_read(client->secure.write, request->buffer[0].base, bytes);
 	uv_write(&request->writer, (uv_stream_t *)&client->stream,
-			request->buffer, 1, on_client_write);
+			request->buffer, request->nbuffers, on_client_write);
     }
 }
 
@@ -120,8 +136,8 @@ void
 flush_secure_module(struct proxy *proxy)
 {
     struct client	*client, **head = &proxy->pending_writes;
-    size_t		i;
-    int			used, sts;
+    size_t		i, used;
+    int			sts;
 
     while ((client = *head) != NULL) {
 	flush_ssl_buffer(client);
@@ -148,8 +164,8 @@ flush_secure_module(struct proxy *proxy)
 		continue;
 	    }
 	    if (sts != SSL_ERROR_WANT_READ) {
-		uv_close((uv_handle_t *)&client->stream, on_client_close);
 		client->secure.pending.queued = 0;
+		client_close(client);
 		break;
 	    }
 	    /*
@@ -168,23 +184,21 @@ flush_secure_module(struct proxy *proxy)
 	} else {
 	    client->secure.pending.writes_count -= used;
 	    memmove(client->secure.pending.writes_buffer,
-		    client->secure.pending.writes_buffer + sizeof(uv_buf_t) * used,
+		    client->secure.pending.writes_buffer + used,
 		    sizeof(uv_buf_t) * client->secure.pending.writes_count);
 	}
     }
 }
 
 void
-secure_client_write(struct client *client,
-		uv_write_t *writer, unsigned int nbuffers)
+secure_client_write(struct client *client, stream_write_baton *request)
 {
-    stream_write_baton	*request = (stream_write_baton *)writer;
     struct proxy	*proxy = client->proxy;
     uv_buf_t		*dup;
     size_t		count, bytes;
     int			i, sts, defer = 0, maybe = 0;
 
-    for (i = 0; i < nbuffers; i++) {
+    for (i = 0; i < request->nbuffers; i++) {
 	if (defer == 0) {
 	    if ((sts = SSL_write(client->secure.ssl,
 			request->buffer[i].base, request->buffer[i].len)) > 0) {
@@ -200,7 +214,7 @@ secure_client_write(struct client *client,
 		    continue;
 	    }
 	    if (sts != SSL_ERROR_WANT_READ) {
-		on_client_write(writer, 1);	/* fail client */
+		on_client_write(&request->writer, 1);	/* fail client */
 		return;
 	    }
 	}
@@ -216,7 +230,7 @@ secure_client_write(struct client *client,
 	bytes = sizeof(uv_buf_t) * count;
 	if ((client->secure.pending.writes_buffer = realloc(
 			client->secure.pending.writes_buffer, bytes)) == NULL) {
-	    on_client_write(writer, 1);		/* fail client */
+	    on_client_write(&request->writer, 1);	/* fail client */
 	    return;
 	}
 	dup = &client->secure.pending.writes_buffer[count-1];
@@ -224,7 +238,7 @@ secure_client_write(struct client *client,
 	dup->len = request->buffer[i].len;
 	maybe = 1;
     }
-    on_client_write((uv_write_t *)request, 0);	/* successfully written */
+    on_client_write(&request->writer, 0);	/* successfully written */
 
     if (maybe)
 	maybe_flush_ssl(proxy, client);
