@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019 Red Hat.
+ * Copyright (c) 2019-2020 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -15,8 +15,7 @@
 #include <assert.h>
 
 typedef enum pmSeriesRestKey {
-    RESTKEY_NONE	= 0,
-    RESTKEY_SOURCE,
+    RESTKEY_SOURCE	= 1,
     RESTKEY_DESC,
     RESTKEY_INSTS,
     RESTKEY_LABELS,
@@ -29,7 +28,8 @@ typedef enum pmSeriesRestKey {
 
 typedef struct pmSeriesRestCommand {
     const char		*name;
-    unsigned int	size;
+    unsigned int	namelen : 16;
+    unsigned int	options : 16;
     pmSeriesRestKey	key;
 } pmSeriesRestCommand;
 
@@ -39,7 +39,8 @@ typedef struct pmSeriesBaton {
     pmSeriesFlags	flags;
     pmSeriesTimeWindow	window;
     uv_work_t		loading;
-    unsigned int	working;
+    unsigned int	working : 1;
+    unsigned int	options : 16;
     int			nsids;
     pmSID		*sids;
     pmSID		sid;
@@ -55,16 +56,25 @@ typedef struct pmSeriesBaton {
 } pmSeriesBaton;
 
 static pmSeriesRestCommand commands[] = {
-    { .key = RESTKEY_QUERY, .name = "query", .size = sizeof("query")-1 },
-    { .key = RESTKEY_DESC,  .name = "descs",  .size = sizeof("descs")-1 },
-    { .key = RESTKEY_INSTS, .name = "instances", .size = sizeof("instances")-1 },
-    { .key = RESTKEY_LABELS, .name = "labels", .size = sizeof("labels")-1 },
-    { .key = RESTKEY_METRIC, .name = "metrics", .size = sizeof("metrics")-1 },
-    { .key = RESTKEY_SOURCE, .name = "sources", .size = sizeof("sources")-1 },
-    { .key = RESTKEY_VALUES, .name = "values", .size = sizeof("values")-1 },
-    { .key = RESTKEY_LOAD, .name = "load", .size = sizeof("load")-1 },
-    { .key = RESTKEY_PING, .name = "ping", .size = sizeof("ping")-1 },
-    { .key = RESTKEY_NONE }
+    { .key = RESTKEY_QUERY, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "query", .namelen = sizeof("query")-1 },
+    { .key = RESTKEY_DESC, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "descs", .namelen = sizeof("descs")-1 },
+    { .key = RESTKEY_INSTS, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "instances", .namelen = sizeof("instances")-1 },
+    { .key = RESTKEY_LABELS, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "labels", .namelen = sizeof("labels")-1 },
+    { .key = RESTKEY_METRIC, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "metrics", .namelen = sizeof("metrics")-1 },
+    { .key = RESTKEY_SOURCE, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "sources", .namelen = sizeof("sources")-1 },
+    { .key = RESTKEY_VALUES, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "values", .namelen = sizeof("values")-1 },
+    { .key = RESTKEY_LOAD, .options = HTTP_OPTIONS_GET | HTTP_OPTIONS_POST,
+	    .name = "load", .namelen = sizeof("load")-1 },
+    { .key = RESTKEY_PING, .options = HTTP_OPTIONS_GET,
+	    .name = "ping", .namelen = sizeof("ping")-1 },
+    { .name = NULL }	/* sentinel */
 };
 
 /* constant string keys (initialized during servlet setup) */
@@ -75,11 +85,11 @@ static sds PARAM_ALIGN, PARAM_COUNT, PARAM_DELTA, PARAM_OFFSET,
 	   PARAM_BEGIN, PARAM_END, PARAM_RANGE, PARAM_ZONE;
 
 /* constant global strings (read-only) */
-static const char pmseries_success[] = "{\"success\":true}\r\n";
-static const char pmseries_failure[] = "{\"success\":false}\r\n";
+static const char pmseries_success[] = "\"success\":true";
+static const char pmseries_failure[] = "\"success\":false";
 
-static pmSeriesRestKey
-pmseries_lookup_restkey(sds url)
+static pmSeriesRestCommand *
+pmseries_lookup_rest_command(sds url)
 {
     pmSeriesRestCommand	*cp;
     const char		*name;
@@ -88,11 +98,11 @@ pmseries_lookup_restkey(sds url)
 	strncmp(url, "/series/", sizeof("/series/") - 1) == 0) {
 	name = (const char *)url + sizeof("/series/") - 1;
 	for (cp = &commands[0]; cp->name; cp++) {
-	    if (strncmp(cp->name, name, cp->size) == 0)
-		return cp->key;
+	    if (strncmp(cp->name, name, cp->namelen) == 0)
+		return cp;
 	}
     }
-    return RESTKEY_NONE;
+    return NULL;
 }
 
 static void
@@ -167,6 +177,11 @@ on_pmseries_value(pmSID sid, pmSeriesValue *value, void *arg)
     sds			timestamp, series, quoted;
     sds			result = http_get_buffer(baton->client);
 
+    if (pmDebugOptions.query && pmDebugOptions.desperate)
+	fprintf(stderr, "on_pmseries_value: arg=%p %s %s %s\n",
+	    arg, value->timestamp, value->data, value->series);
+
+
     timestamp = value->timestamp;
     series = value->series;
     quoted = sdscatrepr(sdsempty(), value->data, sdslen(value->data));
@@ -230,8 +245,9 @@ on_pmseries_metric(pmSID sid, sds name, void *arg)
     pmSeriesBaton	*baton = (pmSeriesBaton *)arg;
     struct client	*client = baton->client;
     const char		*prefix;
-    sds			s, result = http_get_buffer(client);
+    sds			s, quoted, result = http_get_buffer(client);
 
+    quoted = sdscatrepr(sdsempty(), name, sdslen(name));
     if (sid == NULL) {	/* request for all metric names globally */
 	if (baton->values == 0) {
 	    result = push_client_identifier(baton, result);
@@ -240,7 +256,7 @@ on_pmseries_metric(pmSID sid, sds name, void *arg)
 	} else {
 	    prefix = ",";
 	}
-	result = sdscatfmt(result, "%s\"%S\"", prefix, name);
+	result = sdscatfmt(result, "%s%S", prefix, quoted);
     } else {
 	if ((s = baton->sid) == NULL) {	/* first series seem */
 	    baton->sid = sdsdup(sid);
@@ -258,9 +274,10 @@ on_pmseries_metric(pmSID sid, sds name, void *arg)
 	    }
 	    prefix = ",";
 	}
-	result = sdscatfmt(result, "%s{\"series\":\"%S\",\"name\":\"%S\"}",
-				prefix, sid, name);
+	result = sdscatfmt(result, "%s{\"series\":\"%S\",\"name\":%S}",
+				prefix, sid, quoted);
     }
+    sdsfree(quoted);
     baton->values++;	/* count of names for this series/request */
 
     http_set_buffer(client, result, HTTP_FLAG_JSON);
@@ -518,34 +535,86 @@ on_pmseries_done(int status, void *arg)
 {
     pmSeriesBaton	*baton = (pmSeriesBaton *)arg;
     struct client	*client = baton->client;
+    http_options	options = baton->options;
     http_flags		flags = client->u.http.flags;
     http_code		code;
     sds			msg;
 
+    if (pmDebugOptions.query && pmDebugOptions.desperate)
+	fprintf(stderr, "on_pmseries_done: arg=%p status=%d\n", arg, status);
     if (status == 0) {
 	code = HTTP_STATUS_OK;
 	/* complete current response with JSON suffix if needed */
 	if ((msg = baton->suffix) == NULL) {	/* empty OK response */
-	    if (baton->clientid)
-		msg = sdscatfmt(sdsempty(),
-				"{\"client\":%S,\"success\":%s}\r\n",
-				baton->clientid, "true");
-	    else
-		msg = sdsnewlen(pmseries_success, sizeof(pmseries_success) - 1);
+	    switch (baton->restkey) {
+	    case RESTKEY_LABELS:
+		if (baton->names != NULL) {
+		    /* the label values API method always returns an */
+		    /* object { labelName: [labelValues] } */
+		    if (baton->clientid)
+			msg = sdscatfmt(sdsempty(),
+				    "{\"client\":%S,\"result\":{}}\r\n",
+				    baton->clientid);
+		    else
+			msg = sdsnewlen("{}\r\n", 4);
+		    break;
+		}
+	    case RESTKEY_DESC:
+	    case RESTKEY_INSTS:
+	    case RESTKEY_METRIC:
+	    case RESTKEY_VALUES:
+	    case RESTKEY_SOURCE:
+	    case RESTKEY_QUERY:		/* result is an empty array */
+		if (baton->clientid)
+		    msg = sdscatfmt(sdsempty(),
+				"{\"client\":%S,\"result\":[]}\r\n",
+				baton->clientid);
+		else
+		    msg = sdsnewlen("[]\r\n", 4);
+		break;
+
+	    default:			/* use success:true default */
+		msg = sdsnewlen("{", 1);
+		if (baton->clientid)
+		    msg = sdscatfmt(msg, "\"client\":%S,", baton->clientid);
+		msg = sdscatfmt(msg, "%s}\r\n", pmseries_success);
+		break;
+	    }
 	}
 	baton->suffix = NULL;
     } else {
 	if (((code = client->u.http.parser.status_code)) == 0)
 	    code = HTTP_STATUS_BAD_REQUEST;
+	msg = sdsnewlen("{", 1);
 	if (baton->clientid)
-	    msg = sdscatfmt(sdsempty(),
-				"{\"client\":%S,\"success\":%s}\r\n",
-				baton->clientid, "false");
-	else
-	    msg = sdsnewlen(pmseries_failure, sizeof(pmseries_failure) - 1);
+	    msg = sdscatfmt(msg, "\"client\":%S,", baton->clientid);
+	msg = sdscatfmt(msg, "%s}\r\n", pmseries_failure);
 	flags |= HTTP_FLAG_JSON;
     }
-    http_reply(client, msg, code, flags);
+    http_reply(client, msg, code, flags, options);
+}
+
+static void
+on_pmseries_error(pmLogLevel level, sds message, void *arg)
+{
+    pmSeriesBaton	*baton = (pmSeriesBaton *)arg;
+    struct client	*client = baton->client;
+    http_options	options = baton->options;
+    http_flags		flags = client->u.http.flags | HTTP_FLAG_JSON;
+    http_code		status_code;
+    sds			quoted, msg;
+
+    if (((status_code = client->u.http.parser.status_code)) == 0)
+	status_code = (level > PMLOG_REQUEST) ?
+		HTTP_STATUS_INTERNAL_SERVER_ERROR : HTTP_STATUS_BAD_REQUEST;
+    quoted = sdscatrepr(sdsempty(), message, sdslen(message));
+    msg = sdsnewlen("{", 1);
+    if (baton->clientid)
+	msg = sdscatfmt(msg, "\"client\":%S,", baton->clientid);
+    msg = sdscatfmt(msg, "\"message\":%S,%s}\r\n", quoted, pmseries_failure);
+    sdsfree(quoted);
+
+    http_reply(client, msg, status_code, flags, options);
 }
 
 static void
@@ -555,6 +624,18 @@ pmseries_setup(void *arg)
 	fprintf(stderr, "series module setup (arg=%p)\n", arg);
 }
 
+static void
+pmseries_log(pmLogLevel level, sds message, void *arg)
+{
+    pmSeriesBaton	*baton = (pmSeriesBaton *)arg;
+
+    /* locally log low priority diagnostics or when already responding */
+    if (level <= PMLOG_INFO || baton->suffix)
+	proxylog(level, message, baton->client->proxy);
+    else	/* inform client, complete request */
+	on_pmseries_error(level, message, baton);
+}
+
 static pmSeriesSettings pmseries_settings = {
     .callbacks.on_match		= on_pmseries_match,
     .callbacks.on_desc		= on_pmseries_desc,
@@ -567,7 +648,7 @@ static pmSeriesSettings pmseries_settings = {
     .callbacks.on_label		= on_pmseries_label,
     .callbacks.on_done		= on_pmseries_done,
     .module.on_setup		= pmseries_setup,
-    .module.on_info		= proxylog,
+    .module.on_info		= pmseries_log,
 };
 
 static void
@@ -577,7 +658,7 @@ pmseries_setup_request_parameters(struct client *client,
     enum http_method	method = client->u.http.parser.method;
     dictEntry		*entry;
     size_t		length;
-    sds			series, names;
+    sds			series, names, expr;
 
     if (parameters) {
 	/* allow all APIs to pass(-through) a 'client' parameter */
@@ -619,8 +700,13 @@ pmseries_setup_request_parameters(struct client *client,
 	    client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
 	} else if (parameters != NULL &&
 	    (entry = dictFind(parameters, PARAM_EXPR)) != NULL) {
-	    baton->query = dictGetVal(entry);   /* get sds value */
-	    dictSetVal(parameters, entry, NULL);   /* claim this */
+	    expr = dictGetVal(entry);   /* get sds value */
+	    if (expr == NULL || sdslen(expr) == 0) {
+		client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
+	    } else {
+		dictSetVal(parameters, entry, NULL);   /* claim this */
+		baton->query = expr;
+	    }
 	} else if (method != HTTP_POST) {
 	    client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
 	}
@@ -633,8 +719,10 @@ pmseries_setup_request_parameters(struct client *client,
 	} else if (parameters != NULL &&
 	    (entry = dictFind(parameters, PARAM_SERIES)) != NULL) {
 	    series = dictGetVal(entry);
-	    length = sdslen(series);
-	    baton->sids = sdssplitlen(series, length, ",", 1, &baton->nsids);
+	    if (series == NULL || (length = sdslen(series)) == 0)
+		client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
+	    else
+		baton->sids = sdssplitlen(series, length, ",", 1, &baton->nsids);
 	} else if (method != HTTP_POST) {
 	    client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
 	}
@@ -648,8 +736,10 @@ pmseries_setup_request_parameters(struct client *client,
 	if (parameters != NULL) {
 	    if ((entry = dictFind(parameters, PARAM_SERIES)) != NULL) {
 		series = dictGetVal(entry);
-		length = sdslen(series);
-		baton->sids = sdssplitlen(series, length, ",", 1, &baton->nsids);
+		if (series == NULL || (length = sdslen(series)) == 0)
+		    client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
+		else
+		    baton->sids = sdssplitlen(series, length, ",", 1, &baton->nsids);
 	    } else if ((entry = dictFind(parameters, PARAM_MATCH)) != NULL) {
 		baton->match = dictGetVal(entry);
 		baton->sids = &baton->match;
@@ -661,8 +751,10 @@ pmseries_setup_request_parameters(struct client *client,
 		entry = dictFind(parameters, PARAM_NAMES);	/* synonym */
 	    if (entry != NULL) {
 		names = dictGetVal(entry);
-		length = sdslen(names);
-		baton->names = sdssplitlen(names, length, ",", 1, &baton->nnames);
+		if (names == NULL || (length = sdslen(names)) == 0)
+		    client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
+		else
+		    baton->names = sdssplitlen(names, length, ",", 1, &baton->nnames);
 	    }
 	}
 	break;
@@ -672,8 +764,10 @@ pmseries_setup_request_parameters(struct client *client,
 	if (parameters != NULL) {
 	    if ((entry = dictFind(parameters, PARAM_SOURCE)) != NULL) {
 		series = dictGetVal(entry);
-		length = sdslen(series);
-		baton->sids = sdssplitlen(series, length, ",", 1, &baton->nsids);
+		if (series == NULL || (length = sdslen(series)) == 0)
+		    client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
+		else
+		    baton->sids = sdssplitlen(series, length, ",", 1, &baton->nsids);
 	    } else if ((entry = dictFind(parameters, PARAM_MATCH)) != NULL) {
 		baton->match = dictGetVal(entry);
 		baton->sids = &baton->match;
@@ -686,7 +780,6 @@ pmseries_setup_request_parameters(struct client *client,
     case RESTKEY_PING:
 	break;
 
-    case RESTKEY_NONE:
     default:
 	client->u.http.parser.status_code = HTTP_STATUS_BAD_REQUEST;
 	break;
@@ -702,15 +795,16 @@ static int
 pmseries_request_url(struct client *client, sds url, dict *parameters)
 {
     pmSeriesBaton	*baton;
-    pmSeriesRestKey	key;
+    pmSeriesRestCommand	*command;
 
-    if ((key = pmseries_lookup_restkey(url)) == RESTKEY_NONE)
+    if ((command = pmseries_lookup_rest_command(url)) == NULL)
 	return 0;
 
     if ((baton = calloc(1, sizeof(*baton))) != NULL) {
 	client->u.http.data = baton;
 	baton->client = client;
-	baton->restkey = key;
+	baton->restkey = command->key;
+	baton->options = command->options;
 	pmseries_setup_request_parameters(client, baton, parameters);
     } else {
 	client->u.http.parser.status_code = HTTP_STATUS_INTERNAL_SERVER_ERROR;
@@ -794,10 +888,12 @@ pmseries_request_load(struct client *client, pmSeriesBaton *baton)
 
     if (baton->query == NULL) {
 	message = sdsnewlen(failed, sizeof(failed) - 1);
-	http_reply(client, message, HTTP_STATUS_BAD_REQUEST, HTTP_FLAG_JSON);
+	http_reply(client, message, HTTP_STATUS_BAD_REQUEST,
+			HTTP_FLAG_JSON, baton->options);
     } else if (baton->working) {
 	message = sdsnewlen(loading, sizeof(loading) - 1);
-	http_reply(client, message, HTTP_STATUS_CONFLICT, HTTP_FLAG_JSON);
+	http_reply(client, message, HTTP_STATUS_CONFLICT,
+			HTTP_FLAG_JSON, baton->options);
     } else {
 	uv_queue_work(client->proxy->events, &baton->loading,
 			pmseries_load_work, pmseries_load_done);
@@ -810,8 +906,17 @@ pmseries_request_done(struct client *client)
     pmSeriesBaton	*baton = (pmSeriesBaton *)client->u.http.data;
     int			sts;
 
-    if (client->u.http.parser.status_code)
+    if (client->u.http.parser.status_code) {
+	on_pmseries_done(-EINVAL, baton);
+	return 1;
+    }
+
+    if (client->u.http.parser.method == HTTP_OPTIONS ||
+	client->u.http.parser.method == HTTP_TRACE ||
+	client->u.http.parser.method == HTTP_HEAD) {
+	on_pmseries_done(0, baton);
 	return 0;
+    }
 
     switch (baton->restkey) {
     case RESTKEY_QUERY:
