@@ -29,6 +29,7 @@ static struct {
 	{ .group = "discover" },	/* METRICS_DISCOVER */
 	{ .group = "series" },		/* METRICS_SERIES */
 	{ .group = "webgroup" },	/* METRICS_WEBGROUP */
+	{ .group = "search" },          /* METRICS_SEARCH */
 };
 
 void
@@ -333,14 +334,17 @@ on_write_callback(uv_callback_t *handle, void *data)
 {
     stream_write_baton	*request = (stream_write_baton *)data;
     struct client	*client = (struct client *)request->writer.data;
+    int			sts;
 
     if (pmDebugOptions.af)
 	fprintf(stderr, "%s: client=%p\n", "on_write_callback", client);
 
-    if (client->stream.secure == 0)
-	uv_write(&request->writer, (uv_stream_t *)&client->stream,
+    if (client->stream.secure == 0) {
+	sts = uv_write(&request->writer, (uv_stream_t *)&client->stream,
 		 &request->buffer[0], request->nbuffers, request->callback);
-    else
+	if (sts != 0)
+	    fprintf(stderr, "%s: ERROR uv_write failed\n", "on_write_callback");
+    } else
 	secure_client_write(client, request);
     (void)handle;
     return 0;
@@ -394,8 +398,12 @@ client_protocol(int key)
     case '-':	/* RESP error */
     case '+':	/* RESP status */
     case ':':	/* RESP integer */
+    case ',':	/* RESP double */
     case '$':	/* RESP string */
     case '*':	/* RESP array */
+    case '#':	/* RESP bool */
+    case '%':	/* RESP map */
+    case '~':	/* RESP set */
 	return STREAM_REDIS;
     case 0x14:	/* TLS ChangeCipherSpec */
     case 0x15:	/* TLS Alert */
